@@ -1,13 +1,16 @@
+/* vim: set sw=8 ts=8: -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /**
- * ms-excel-write.h: MS Excel support for Gnumeric
+ * ms-excel-write.h: MS Excel export
  *
- * Author:
+ * Authors:
+ *    Jody Goldberg (jody@gnome.org)
  *    Michael Meeks (michael@ximian.com)
  *
- * (C) 1998, 1999, 2000 Michael Meeks
+ * (C) 1998-2001 Michael Meeks
+ * (C) 2002-2005 Jody Goldberg
  **/
-#ifndef GNUMERIC_MS_EXCEL_WRITE_H
-#define GNUMERIC_MS_EXCEL_WRITE_H
+#ifndef GNM_MS_EXCEL_WRITE_H
+#define GNM_MS_EXCEL_WRITE_H
 
 #include "ms-biff.h"
 #include "ms-excel-biff.h"
@@ -26,13 +29,15 @@ typedef struct {
 	 */
 	guint32    color;
 	char const *font_name;
-	double	  size_pts;
-	gboolean  is_bold;
-	gboolean  is_italic;
-	gboolean  is_auto;
-	StyleUnderlineType	underline;
-	gboolean		strikethrough;
-} ExcelFont;
+	char       *font_name_copy; /* some times we need to keep a local copy */
+	double	  	size_pts;
+	gboolean  	is_bold;
+	gboolean  	is_italic;
+	gboolean  	is_auto;
+	GnmUnderline	underline;
+	gboolean	strikethrough;
+	unsigned	script;
+} ExcelWriteFont;
 
 typedef struct {
 	ExcelWriteState	*ewb;
@@ -41,8 +46,10 @@ typedef struct {
 	guint32		 boundsheetPos;
 	gint32		 max_col, max_row;
 	guint16		 col_xf    [SHEET_MAX_COLS];
-	GnmStyle		*col_style [SHEET_MAX_COLS];
-	GnmStyleList 	*validations;
+	GnmStyle	*col_style [SHEET_MAX_COLS];
+	GnmStyleList 	*conditions, *validations;
+	GSList           *blips, *textboxes;
+	unsigned	 cur_obj, num_objs, num_blips;
 } ExcelWriteSheet;
 
 struct _ExcelWriteState {
@@ -71,6 +78,7 @@ struct _ExcelWriteState {
 
 	GHashTable    *function_map;
 	GHashTable    *sheet_pairs;
+	GHashTable    *cell_markup;
 
 	/* we use the ewb as a closure for things, this is useful */
 	int tmp_counter;
@@ -88,7 +96,7 @@ struct _ExcelWriteState {
 		GPtrArray  *indicies;
 	} sst;
 
-	unsigned obj_count;
+	unsigned num_obj_groups, cur_obj_group, cur_blip;
 	gboolean export_macros;
 };
 
@@ -119,14 +127,18 @@ typedef enum {
 	STR_SUPPRESS_HEADER	= 8
 } WriteStringFlags;
 
-unsigned excel_write_string_len (guint8 const *txt, unsigned *bytes);
-unsigned excel_write_string	(BiffPut *bp, guint8 const *txt,
-				 WriteStringFlags flags);
+unsigned excel_write_string_len (guint8 const *txt, size_t *bytes);
+unsigned excel_write_string	(BiffPut *bp, WriteStringFlags flags,
+				 guint8 const *txt);
+unsigned excel_write_BOF	(BiffPut *bp, MsBiffFileType type);
+void	 excel_write_SETUP	(BiffPut *bp, ExcelWriteSheet *esheet);
+void	 excel_write_SCL	(BiffPut *bp, double zoom, gboolean force);
 
+gint palette_get_index (ExcelWriteState const *ewb, guint c);
 int excel_write_get_externsheet_idx (ExcelWriteState *wb,
 				     Sheet *gnum_sheeta,
 				     Sheet *gnum_sheetb);
 
 int excel_write_map_errcode (GnmValue const *v);
 
-#endif
+#endif /* GNM_MS_EXCEL_WRITE_H */
