@@ -3,9 +3,12 @@
 /*
  * ms-obj.c: MS Excel Object support for Gnumeric
  *
- * Copyright (C) 2000-2004
- *	Jody Goldberg (jody@gnome.org)
- *	Michael Meeks (mmeeks@gnu.org)
+ * Authors:
+ *    Jody Goldberg (jody@gnome.org)
+ *    Michael Meeks (michael@ximian.com)
+ *
+ * (C) 1998-2001 Michael Meeks
+ * (C) 2002-2005 Jody Goldberg
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -19,7 +22,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  */
 
@@ -166,9 +169,11 @@ ms_obj_attr_get_int  (MSObjAttrBag *attrs, MSObjAttrID id, gint32 default_value)
 }
 
 gpointer
-ms_obj_attr_get_ptr  (MSObjAttrBag *attrs, MSObjAttrID id, gpointer default_value)
+ms_obj_attr_get_ptr (MSObjAttrBag *attrs, MSObjAttrID id,
+		     gpointer default_value, gboolean steal)
 {
 	MSObjAttr *attr;
+	gpointer res;
 
 	g_return_val_if_fail (attrs != NULL, default_value);
 	g_return_val_if_fail (id & MS_OBJ_ATTR_IS_PTR_MASK, default_value);
@@ -176,7 +181,10 @@ ms_obj_attr_get_ptr  (MSObjAttrBag *attrs, MSObjAttrID id, gpointer default_valu
 	attr = ms_obj_attr_bag_lookup (attrs, id);
 	if (attr == NULL)
 		return default_value;
-	return attr->v.v_ptr;
+	res = attr->v.v_ptr;
+	if (steal)
+		attr->v.v_ptr = NULL;
+	return res;
 }
 
 GArray *
@@ -199,9 +207,11 @@ ms_obj_attr_get_array (MSObjAttrBag *attrs, MSObjAttrID id,
 }
 
 GnmExpr const *
-ms_obj_attr_get_expr (MSObjAttrBag *attrs, MSObjAttrID id, GnmExpr const *default_value)
+ms_obj_attr_get_expr (MSObjAttrBag *attrs, MSObjAttrID id,
+		      GnmExpr const *default_value, gboolean steal)
 {
 	MSObjAttr *attr;
+	GnmExpr const *res;
 
 	g_return_val_if_fail (attrs != NULL, default_value);
 	g_return_val_if_fail (id & MS_OBJ_ATTR_IS_EXPR_MASK, default_value);
@@ -209,13 +219,18 @@ ms_obj_attr_get_expr (MSObjAttrBag *attrs, MSObjAttrID id, GnmExpr const *defaul
 	attr = ms_obj_attr_bag_lookup (attrs, id);
 	if (attr == NULL)
 		return default_value;
-	return attr->v.v_expr;
+	res = attr->v.v_expr;
+	if (steal)
+		attr->v.v_expr = NULL;
+	return res;
 }
 
 PangoAttrList *
-ms_obj_attr_get_markup (MSObjAttrBag *attrs, MSObjAttrID id, PangoAttrList *default_value)
+ms_obj_attr_get_markup (MSObjAttrBag *attrs, MSObjAttrID id,
+			PangoAttrList *default_value, gboolean steal)
 {
 	MSObjAttr *attr;
+	PangoAttrList *res;
 
 	g_return_val_if_fail (attrs != NULL, default_value);
 	g_return_val_if_fail (id & MS_OBJ_ATTR_IS_PANGO_ATTR_LIST_MASK, default_value);
@@ -223,7 +238,10 @@ ms_obj_attr_get_markup (MSObjAttrBag *attrs, MSObjAttrID id, PangoAttrList *defa
 	attr = ms_obj_attr_bag_lookup (attrs, id);
 	if (attr == NULL)
 		return default_value;
-	return attr->v.v_markup;
+	res = attr->v.v_markup;
+	if (steal)
+		attr->v.v_markup = NULL;
+	return res;
 }
 
 static void
@@ -357,7 +375,7 @@ ms_read_TXO (BiffQuery *q, MSContainer *c, PangoAttrList **markup)
 
 	guint16 const options     = GSF_LE_GET_GUINT16 (q->data);
 	guint16 const orient      = GSF_LE_GET_GUINT16 (q->data + 2);
-	guint16 const text_len    = GSF_LE_GET_GUINT16 (q->data + 10);
+	guint16	      text_len    = GSF_LE_GET_GUINT16 (q->data + 10);
 /*	guint16 const num_formats = GSF_LE_GET_GUINT16 (q->data + 12);*/
 	int const halign = (options >> 1) & 0x7;
 	int const valign = (options >> 4) & 0x7;
@@ -373,14 +391,29 @@ ms_read_TXO (BiffQuery *q, MSContainer *c, PangoAttrList **markup)
 	g_return_val_if_fail (1 <= valign && valign <= 4, NULL);
 
 	if (ms_biff_query_peek_next (q, &op) && op == BIFF_CONTINUE) {
+		gboolean use_utf16;
+
 		ms_biff_query_next (q);
 
-		if ((int)q->length < text_len) {
-			g_warning ("Broken continue in TXO record");
-			text = g_strdup ("Broken continue");
-		} else
-			text = ms_biff_get_chars (q->data + 1, text_len,
-						  *(q->data) != 0);
+		use_utf16 = q->data[0] != 0;
+		text = excel_get_chars (c->importer,
+			q->data + 1, MIN (text_len, q->length-1), use_utf16);
+		if (q->length < text_len) {
+			GString *accum = g_string_new (text);
+			g_free (text);
+			text_len -= q->length - 1;
+			while (ms_biff_query_peek_next (q, &op) && op == BIFF_CONTINUE) {
+				ms_biff_query_next (q);
+				text = excel_get_chars (c->importer, q->data,
+					MIN (q->length, text_len), use_utf16);
+				g_string_append (accum, text);
+				g_free (text);
+				if (text_len <= q->length)
+					break;
+				text_len -= q->length;
+			}
+			text = g_string_free (accum, FALSE);
+		}
 
 		if (ms_biff_query_peek_next (q, &op) && op == BIFF_CONTINUE) {
 			ms_biff_query_next (q);
@@ -428,44 +461,91 @@ ms_obj_dump_impl (guint8 const *data, int len, int data_left, char const *name)
 #endif
 
 static gboolean
-read_pre_biff8_read_str (BiffQuery *q, MSContainer *container, MSObj *obj,
-			 MSObjAttrID text_id, guint8 const **first,
-			 unsigned len, unsigned txo_len)
+read_pre_biff8_read_text (BiffQuery *q, MSContainer *c, MSObj *obj,
+			  guint8 const *first,
+			  unsigned len, unsigned txo_len)
 {
-	guint8 const *last = q->data + q->length;
+	PangoAttrList *markup = NULL;
+	GByteArray    *markup_data = NULL;
 	char *str;
+	unsigned remaining;
+	guint16  op;
 
-	g_return_val_if_fail (*first + len <= last, TRUE);
-	g_return_val_if_fail (text_id != MS_OBJ_ATTR_NONE, TRUE);
+	if (first == NULL)
+		return TRUE;
 
-	str = ms_biff_get_chars (*first, len, FALSE);
-	ms_obj_attr_bag_insert (obj->attrs, ms_obj_attr_new_ptr (text_id, str));
+	remaining = q->data + q->length - first;
 
-	*first += len;
-	if (((*first - q->data) & 1))
-		(*first)++; /* pad to word bound */
+	/* CONTINUE handling here is very odd.
+	 * If the text needs CONTINUEs but the markup does not, the markup is
+	 * stored at the end of the OBJ rather than after the text.  */
+	if (txo_len > 0 && txo_len < remaining) {
+		markup_data = g_byte_array_new ();
+		g_byte_array_append (markup_data, q->data + q->length - txo_len, txo_len);
+		remaining -= txo_len;
+	}
 
-	if (txo_len > 0) {
-		guint8 const *last = q->data + q->length;
-		PangoAttrList *markup;
+	str = excel_get_chars (c->importer, first, MIN (remaining, len), FALSE);
+	if (len > remaining) {
+		GString *accum = g_string_new (str);
+		g_free (str);
+		len -= remaining;
+		while (ms_biff_query_peek_next (q, &op) && op == BIFF_CONTINUE) {
+			ms_biff_query_next (q);
+			str = excel_get_chars (c->importer, q->data,
+				MIN (q->length, len), FALSE);
+			g_string_append (accum, str);
+			g_free (str);
+			if (len < q->length)
+				break;
+			len -= q->length;
+		}
+		str = g_string_free (accum, FALSE);
+		first = q->data + len;
+	} else
+		first += len;
+	if (((first - q->data) & 1))
+		first++; /* pad to word bound */
 
-		g_return_val_if_fail ((*first + txo_len) <= last, TRUE);
+	ms_obj_attr_bag_insert (obj->attrs,
+		ms_obj_attr_new_ptr (MS_OBJ_ATTR_TEXT, str));
 
-		markup = ms_container_read_markup (container, *first, txo_len, str);
+	if (NULL != markup_data) {
+		markup = ms_container_read_markup (c, markup_data->data, markup_data->len, str);
+		g_byte_array_free (markup_data, TRUE);
+	} else if (txo_len > 0) {
+		remaining = q->data + q->length - first;
+		if (txo_len > remaining) {
+			GByteArray *accum = g_byte_array_new ();
+			g_byte_array_append (accum, first, remaining);
+			txo_len -= remaining;
+			while (ms_biff_query_peek_next (q, &op) && op == BIFF_CONTINUE) {
+				ms_biff_query_next (q);
+				g_byte_array_append (accum, q->data, MIN (q->length, txo_len));
+				if (txo_len <= q->length)
+					break;
+				txo_len -= q->length;
+			}
+			first = q->data + txo_len;
+			markup = ms_container_read_markup (c, accum->data, accum->len, str);
+			g_byte_array_free (accum, TRUE);
+		} else {
+			markup = ms_container_read_markup (c, first, txo_len, str);
+			first += txo_len;
+		}
+	}
+	if (NULL != markup) {
 		ms_obj_attr_bag_insert (obj->attrs,
 			ms_obj_attr_new_markup (MS_OBJ_ATTR_MARKUP, markup));
 		pango_attr_list_unref (markup);
-
-		*first += txo_len;
 	}
 
 	return FALSE;
 }
 
 static gboolean
-read_pre_biff8_read_expr (BiffQuery *q, MSContainer *container, MSObj *obj,
-			  MSObjAttrID id, guint8 const **first,
-			  unsigned total_len) /* including extras */
+read_pre_biff8_read_expr (BiffQuery *q, MSContainer *c, MSObj *obj,
+			  guint8 const **first, unsigned total_len) /* including extras */
 {
 	guint8 const *ptr  = *first;
 	guint8 const *last = q->data + q->length;
@@ -481,7 +561,7 @@ read_pre_biff8_read_expr (BiffQuery *q, MSContainer *container, MSObj *obj,
 
 	g_return_val_if_fail (ptr + 6 + len <= last, TRUE);
 
-	ref = ms_container_parse_expr (container, ptr + 6, len);
+	ref = ms_container_parse_expr (c, ptr + 6, len);
 	if (ref != NULL)
 		ms_obj_attr_bag_insert (obj->attrs,
 			ms_obj_attr_new_expr (MS_OBJ_ATTR_LINKED_TO_CELL, ref));
@@ -494,18 +574,28 @@ read_pre_biff8_read_expr (BiffQuery *q, MSContainer *container, MSObj *obj,
 }
 
 static guint8 const *
-read_pre_biff8_read_name_and_fmla (BiffQuery *q, MSContainer *container, MSObj *obj,
+read_pre_biff8_read_name_and_fmla (BiffQuery *q, MSContainer *c, MSObj *obj,
 				   gboolean has_name, unsigned offset)
 {
 	guint8 const *data = q->data + offset;
 	gboolean const fmla_len = GSF_LE_GET_GUINT16 (q->data+26);
 
-	if (has_name &&
-	    read_pre_biff8_read_str (q, container, obj, MS_OBJ_ATTR_OBJ_NAME,
-				     &data, *(data++), 0))
-		return NULL;
-	if (read_pre_biff8_read_expr (q, container, obj,
-				      MS_OBJ_ATTR_NONE, &data, fmla_len))
+	if (has_name) {
+		guint8 const *last = q->data + q->length;
+		unsigned len = *data++;
+		char *str;
+
+		g_return_val_if_fail (data + len <= last, NULL);
+
+		str = excel_get_chars (c->importer, data, len, FALSE);
+		data += len;
+		if (((data - q->data) & 1))
+			data++; /* pad to word bound */
+
+		ms_obj_attr_bag_insert (obj->attrs,
+			ms_obj_attr_new_ptr (MS_OBJ_ATTR_OBJ_NAME, str));
+	}
+	if (read_pre_biff8_read_expr (q, c, obj, &data, fmla_len))
 		return NULL;
 	return data;
 }
@@ -591,11 +681,8 @@ ms_obj_read_pre_biff8_obj (BiffQuery *q, MSContainer *c, MSObj *obj)
 			if_empty = GSF_LE_GET_GUINT16 (q->data + 50);
 
 			data = read_pre_biff8_read_name_and_fmla (q, c, obj, has_name, 70);
-			if (data == NULL ||
-			    read_pre_biff8_read_str (q, c, obj,
-				MS_OBJ_ATTR_TEXT, &data, len, txo_len))
+			if (read_pre_biff8_read_text (q, c, obj, data, len, txo_len))
 				return TRUE;
-
 			if (txo_len == 0)
 				ms_obj_attr_bag_insert (obj->attrs,
 					ms_obj_attr_new_markup (MS_OBJ_ATTR_MARKUP,
@@ -662,11 +749,8 @@ ms_obj_read_pre_biff8_obj (BiffQuery *q, MSContainer *c, MSObj *obj)
 	case 0xE  : /* label */
 /* 70 name len, name, fmla (respect cbMacro), cbtext, text */
 		len = GSF_LE_GET_GUINT16 (q->data + 44);
-
 		data = read_pre_biff8_read_name_and_fmla (q, c, obj, has_name, 70);
-		if (data == NULL ||
-		    read_pre_biff8_read_str (q, c, obj,
-			MS_OBJ_ATTR_TEXT, &data, len, 16))
+		if (read_pre_biff8_read_text (q, c, obj, data, len, 16))
 			return TRUE;
 		break;
 	case 0xF  : /* dialog frame */
@@ -745,7 +829,7 @@ ms_obj_read_pre_biff8_obj (BiffQuery *q, MSContainer *c, MSObj *obj)
 
 
 static void
-ms_obj_map_forms_obj (MSObj *obj, MSContainer *container, guint8 const *data, gint32 len)
+ms_obj_map_forms_obj (MSObj *obj, MSContainer *c, guint8 const *data, gint32 len)
 {
 	static struct {
 		char const	*key;
@@ -766,10 +850,13 @@ ms_obj_map_forms_obj (MSObj *obj, MSContainer *container, guint8 const *data, gi
 	};
 	int i = G_N_ELEMENTS (map_forms);
 	char const *key;
-	int key_len;
+	int key_len = 0;
+	guint8 const *ptr;
+	guint16 expr_len;
+	GnmExpr const *ref;
 
 	if (obj->excel_type != 8 || len <= 27 ||
-	    strncmp (data+21, "Forms.", 6))
+	    strncmp ((char *)(data+21), "Forms.", 6))
 		return;
 
 	while (i-- > 0) {
@@ -777,29 +864,33 @@ ms_obj_map_forms_obj (MSObj *obj, MSContainer *container, guint8 const *data, gi
 		key_len = strlen (key);
 		if (map_forms [i].excel_type > 0 &&
 		    len >= (27+key_len) &&
-		    0 == strncmp (data+27, map_forms [i].key, key_len))
+		    0 == strncmp ((char *)(data+27), map_forms [i].key, key_len))
 			break;
 	}
 
-	if (i >= 0) {
-		obj->excel_type = map_forms [i].excel_type;
-		if (map_forms [i].offset_to_link != 0) {
-			guint8 const *ptr = data + 27 + key_len + map_forms [i].offset_to_link;
-			guint16 expr_len;
-			GnmExpr const *ref;
-			g_return_if_fail (ptr + 2 <= (data + len));
-			expr_len = GSF_LE_GET_GUINT16 (ptr);
-			g_return_if_fail (ptr + 2 + expr_len <= (data + len));
-			ref = ms_container_parse_expr (container, ptr + 6, expr_len);
-			if (ref != NULL)
-				ms_obj_attr_bag_insert (obj->attrs,
-					ms_obj_attr_new_expr (MS_OBJ_ATTR_LINKED_TO_CELL, ref));
-		}
-	}
+	if (i < 0)
+		return;
+	obj->excel_type = map_forms [i].excel_type;
+
+	if (map_forms [i].offset_to_link == 0)
+		return;
+
+	ptr = data + 27 + key_len + map_forms [i].offset_to_link;
+	if (ptr + 2 > (data + len))
+		return;
+
+	expr_len = GSF_LE_GET_GUINT16 (ptr);
+
+	g_return_if_fail (ptr + 2 + expr_len <= (data + len));
+
+	ref = ms_container_parse_expr (c, ptr + 6, expr_len);
+	if (ref != NULL)
+		ms_obj_attr_bag_insert (obj->attrs,
+			ms_obj_attr_new_expr (MS_OBJ_ATTR_LINKED_TO_CELL, ref));
 }
 
 static gboolean
-ms_obj_read_biff8_obj (BiffQuery *q, MSContainer *container, MSObj *obj)
+ms_obj_read_biff8_obj (BiffQuery *q, MSContainer *c, MSObj *obj)
 {
 	guint8 *data;
 	gint32 data_len_left;
@@ -877,7 +968,7 @@ ms_obj_read_biff8_obj (BiffQuery *q, MSContainer *container, MSObj *obj)
 
 		case GR_PICTURE_FORMULA :
 			/* Check for form objects stored here for no apparent reason */
-			ms_obj_map_forms_obj (obj, container, data, len);
+			ms_obj_map_forms_obj (obj, c, data, len);
 			break;
 
 		case GR_CHECKBOX_LINK :
@@ -913,7 +1004,7 @@ ms_obj_read_biff8_obj (BiffQuery *q, MSContainer *container, MSObj *obj)
 
 		case GR_SCROLLBAR_FORMULA : {
 			guint16 const expr_len = GSF_LE_GET_GUINT16 (data+4);
-			GnmExpr const *ref = ms_container_parse_expr (container, data+10, expr_len);
+			GnmExpr const *ref = ms_container_parse_expr (c, data+10, expr_len);
 			if (ref != NULL)
 				ms_obj_attr_bag_insert (obj->attrs,
 					ms_obj_attr_new_expr (MS_OBJ_ATTR_LINKED_TO_CELL, ref));
@@ -951,7 +1042,7 @@ ms_obj_read_biff8_obj (BiffQuery *q, MSContainer *container, MSObj *obj)
 
 		case GR_CHECKBOX_FORMULA : {
 			guint16 const expr_len = GSF_LE_GET_GUINT16 (data+4);
-			GnmExpr const *ref = ms_container_parse_expr (container, data+10, expr_len);
+			GnmExpr const *ref = ms_container_parse_expr (c, data+10, expr_len);
 			if (ref != NULL)
 				ms_obj_attr_bag_insert (obj->attrs,
 					ms_obj_attr_new_expr (MS_OBJ_ATTR_LINKED_TO_CELL, ref));
@@ -1049,8 +1140,7 @@ ms_obj_read_biff8_obj (BiffQuery *q, MSContainer *container, MSObj *obj)
 
 	/* FIXME : Throw away the IMDATA that may follow.
 	 * I am not sure when the IMDATA does follow, or how to display it,
-	 * but very careful in case it is not there.
-	 */
+	 * but very careful in case it is not there. */
 	if (next_biff_record_maybe_imdata) {
 		guint16 op;
 
@@ -1071,11 +1161,11 @@ ms_obj_read_biff8_obj (BiffQuery *q, MSContainer *container, MSObj *obj)
 /**
  * ms_read_OBJ :
  * @q : The biff record to start with.
- * @container : The object's container
+ * @c : The object's container
  * @attrs : an OPTIONAL hash of object attributes.
  */
 void
-ms_read_OBJ (BiffQuery *q, MSContainer *container, MSObjAttrBag *attrs)
+ms_read_OBJ (BiffQuery *q, MSContainer *c, MSObjAttrBag *attrs)
 {
 	static char const * const object_type_names[] = {
 		"Group", 	/* 0x00 */
@@ -1106,15 +1196,20 @@ ms_read_OBJ (BiffQuery *q, MSContainer *container, MSObjAttrBag *attrs)
 	};
 
 	gboolean errors;
-	MSObj *obj = ms_obj_new (attrs);
+	MSObj *obj;
+
+	/* no decent docs for this */
+	if  (c->importer->ver <= MS_BIFF_V4)
+		return;
 
 #ifndef NO_DEBUG_EXCEL
 	if (ms_excel_object_debug > 0)
 		printf ("{ /* OBJ start */\n");
 #endif
-	errors = (container->ver >= MS_BIFF_V8)
-		? ms_obj_read_biff8_obj (q, container, obj)
-		: ms_obj_read_pre_biff8_obj (q, container, obj);
+	obj = ms_obj_new (attrs);
+	errors = (c->importer->ver >= MS_BIFF_V8)
+		? ms_obj_read_biff8_obj (q, c, obj)
+		: ms_obj_read_pre_biff8_obj (q, c, obj);
 
 	if (errors) {
 #ifndef NO_DEBUG_EXCEL
@@ -1138,20 +1233,18 @@ ms_read_OBJ (BiffQuery *q, MSContainer *container, MSObjAttrBag *attrs)
 	}
 #endif
 
-	if (container->vtbl->create_obj != NULL)
-		obj->gnum_obj = (*container->vtbl->create_obj) (container, obj);
+	if (c->vtbl->create_obj != NULL)
+		obj->gnum_obj = (*c->vtbl->create_obj) (c, obj);
 
 	/* Chart, There should be a BOF next */
-	if (obj->excel_type == 0x5 &&
-	    ms_excel_chart_read_BOF (q, container, obj->gnum_obj)) {
-		ms_obj_delete (obj);
-		return;
+	if (obj->excel_type == 0x5) {
+		if (ms_excel_chart_read_BOF (q, c, obj->gnum_obj)) {
+			ms_obj_delete (obj);
+			return;
+		}
 	}
 
-#if 0
-	g_warning ("registered obj %d\n", obj->id);
-#endif
-	ms_container_add_obj (container, obj);
+	ms_container_add_obj (c, obj);
 }
 
 /**********************************************************************/
@@ -1209,8 +1302,3 @@ ms_objv8_write_listbox (BiffPut *bp, gboolean filtered)
 		GSF_LE_SET_GUINT16 (buf + 14, 0xa);
 	ms_biff_put_var_write (bp, buf, sizeof data);
 }
-
-void
-ms_objv8_write_chart (BiffPut *bp, SheetObject *sog)
-{
-}
