@@ -51,10 +51,33 @@ GNM_PLUGIN_MODULE_HEADER;
 static GnmValue *
 make_date (GnmValue *res)
 {
-	value_set_fmt (res, style_format_default_date ());
+	value_set_fmt (res, go_format_default_date ());
 	return res;
 }
 
+static int
+float_to_secs (gnm_float d)
+{
+	int secs;
+
+	/* Ok, we have a positive number.  Add epsilon before we scale
+	   and translate because otherwise it will not be enough.  */
+	d = gnm_add_epsilon (d);
+
+	/* Get the number down between 0 and 1 before we scale.  */
+	d -= gnm_floor (d);
+
+	/* Scale and round.  */
+	secs = (int)(gnm_add_epsilon (d) * DAY_SECONDS + 0.5);
+
+	/* We rounded, so we might have gone too far.  */
+	if (secs >= DAY_SECONDS)
+		secs -= DAY_SECONDS;
+
+	return secs;
+}
+
+
 /***************************************************************************/
 
 static GnmFuncHelp const help_date[] = {
@@ -82,39 +105,46 @@ static GnmFuncHelp const help_date[] = {
 };
 
 static GnmValue *
-gnumeric_date (FunctionEvalInfo *ei, GnmValue **argv)
+gnumeric_date (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
-	int year, month, day;
+	gnm_float year  = value_get_as_float (argv [0]);
+	gnm_float month = value_get_as_float (argv [1]);
+	gnm_float day   = value_get_as_float (argv [2]);
 	GDate date;
 	GODateConventions const *conv = DATE_CONV (ei->pos);
 
-	year  = value_get_as_int (argv [0]);
-	month = value_get_as_int (argv [1]);
-	day   = value_get_as_int (argv [2]);
-
-	if (year < 0 || year > 9999)
+	if (year < 0 || year >= 10000)
 		goto error;
-
 	if (year < 1900) /* 1900, not 100.  Ick!  */
 		year += 1900;
 
+	/* This uses floor and not trunc on purpose.  */
+	month = gnm_floor (month);
+	if (gnm_abs (month) > 120000)  /* Actual number not critical.  */
+		goto error;
+
+	/* This uses floor and not trunc on purpose.  */
+	day = gnm_floor (day);
+	if (day < -32768 || day >= 32768)
+		day = 32767;  /* Absurd, but yes.  */
+
         g_date_clear (&date, 1);
 
-	g_date_set_dmy (&date, 1, 1, year);
+	g_date_set_dmy (&date, 1, 1, (int)year);
 	if (!g_date_valid (&date))
 		goto error;
 
-	if (month > 0)
-		g_date_add_months (&date, month - 1);
+	if ((int)month > 0)
+		g_date_add_months (&date, (int)month - 1);
 	else
-		g_date_subtract_months (&date, 1 - month);
+		g_date_subtract_months (&date, 1 - (int)month);
 	if (!g_date_valid (&date))
 		goto error;
 
-	if (day > 0)
-                g_date_add_days (&date, day - 1);
+	if ((int)day > 0)
+                g_date_add_days (&date, (int)day - 1);
 	else
-		g_date_subtract_days (&date, 1 - day);
+		g_date_subtract_days (&date, 1 - (int)day);
 	if (!g_date_valid (&date))
 		goto error;
 
@@ -149,7 +179,7 @@ static GnmFuncHelp const help_unix2date[] = {
 };
 
 static GnmValue *
-gnumeric_unix2date (FunctionEvalInfo *ei, GnmValue **argv)
+gnumeric_unix2date (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
 	gnm_float futime = value_get_as_float (argv [0]);
 	time_t     utime  = (time_t)futime;
@@ -185,7 +215,7 @@ static GnmFuncHelp const help_date2unix[] = {
 };
 
 static GnmValue *
-gnumeric_date2unix (FunctionEvalInfo *ei, GnmValue **argv)
+gnumeric_date2unix (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
 	gnm_float fserial = value_get_as_float (argv [0]);
 	int        serial  = (int)fserial;
@@ -222,7 +252,7 @@ static GnmFuncHelp const help_datevalue[] = {
 };
 
 static GnmValue *
-gnumeric_datevalue (FunctionEvalInfo *ei, GnmValue **argv)
+gnumeric_datevalue (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
 	return value_new_int (datetime_value_to_serial (argv[0], DATE_CONV (ei->pos)));
 }
@@ -344,7 +374,7 @@ datedif_opt_md (GDate *gdate1, GDate *gdate2, gboolean excel_compat)
 }
 
 static GnmValue *
-gnumeric_datedif (FunctionEvalInfo *ei, GnmValue **argv)
+gnumeric_datedif (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
 	int date1, date2;
 	char const *opt;
@@ -405,7 +435,7 @@ static GnmFuncHelp const help_edate[] = {
 };
 
 static GnmValue *
-gnumeric_edate (FunctionEvalInfo *ei, GnmValue **argv)
+gnumeric_edate (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
 	int    serial, months;
 	GDate  date;
@@ -428,7 +458,7 @@ gnumeric_edate (FunctionEvalInfo *ei, GnmValue **argv)
                   return value_new_error_NUM (ei->pos);
 
 	res = value_new_int (datetime_g_to_serial (&date, conv));
-	value_set_fmt (res, style_format_default_date ());
+	value_set_fmt (res, go_format_default_date ());
 	return res;
 }
 
@@ -453,7 +483,7 @@ static GnmFuncHelp const help_today[] = {
 };
 
 static GnmValue *
-gnumeric_today (FunctionEvalInfo *ei, GnmValue **argv)
+gnumeric_today (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
 	return make_date (value_new_int (datetime_timet_to_serial (time (NULL), DATE_CONV (ei->pos))));
 }
@@ -469,7 +499,7 @@ static GnmFuncHelp const help_now[] = {
 	   "NOW returns the serial number for the date and time at the time "
 	   "it is evaluated.\n"
 	   "\n"
-	   "Serial Numbers in Gnumeric are represented as follows:"
+	   "Serial Numbers in Gnumeric are represented as follows: "
 	   "The integral part is the number of days since the 1st of "
 	   "January of 1900.  The decimal part represent the fraction "
 	   "of the day and is mapped into hour, minutes and seconds.\n"
@@ -487,7 +517,7 @@ static GnmFuncHelp const help_now[] = {
 };
 
 static GnmValue *
-gnumeric_now (FunctionEvalInfo *ei, GnmValue **argv)
+gnumeric_now (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
 	return value_new_float (datetime_timet_to_serial_raw (time (NULL), DATE_CONV (ei->pos)));
 }
@@ -512,7 +542,7 @@ static GnmFuncHelp const help_time[] = {
 };
 
 static GnmValue *
-gnumeric_time (FunctionEvalInfo *ei, GnmValue **argv)
+gnumeric_time (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
 	gnm_float hours, minutes, seconds;
 
@@ -546,7 +576,7 @@ static GnmFuncHelp const help_timevalue[] = {
 };
 
 static GnmValue *
-gnumeric_timevalue (FunctionEvalInfo *ei, GnmValue **argv)
+gnumeric_timevalue (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
 	gnm_float raw = datetime_value_to_serial_raw (argv[0], DATE_CONV (ei->pos));
 	return value_new_float (raw - (int)raw);
@@ -577,11 +607,16 @@ static GnmFuncHelp const help_hour[] = {
 };
 
 static GnmValue *
-gnumeric_hour (FunctionEvalInfo *ei, GnmValue **argv)
+gnumeric_hour (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
-	int secs;
-	secs = datetime_value_to_seconds (argv[0]);
-	return value_new_int (secs / 3600);
+	gnm_float d = value_get_as_float (argv[0]);
+
+	if (d < 0)
+		return value_new_error_NUM (ei->pos);
+	else {
+		int secs = float_to_secs (d);
+		return value_new_int (secs / 3600);
+	}
 }
 
 /***************************************************************************/
@@ -609,12 +644,16 @@ static GnmFuncHelp const help_minute[] = {
 };
 
 static GnmValue *
-gnumeric_minute (FunctionEvalInfo *ei, GnmValue **argv)
+gnumeric_minute (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
-	int secs;
+	gnm_float d = value_get_as_float (argv[0]);
 
-	secs = datetime_value_to_seconds (argv[0]);
-	return value_new_int ((secs / 60) % 60);
+	if (d < 0)
+		return value_new_error_NUM (ei->pos);
+	else {
+		int secs = float_to_secs (d);
+		return value_new_int (secs / 60 % 60);
+	}
 }
 
 /***************************************************************************/
@@ -642,12 +681,16 @@ static GnmFuncHelp const help_second[] = {
 };
 
 static GnmValue *
-gnumeric_second (FunctionEvalInfo *ei, GnmValue **argv)
+gnumeric_second (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
-	int secs;
+	gnm_float d = value_get_as_float (argv[0]);
 
-	secs = datetime_value_to_seconds (argv[0]);
-	return value_new_int (secs % 60);
+	if (d < 0)
+		return value_new_error_NUM (ei->pos);
+	else {
+		int secs = float_to_secs (d);
+		return value_new_int (secs % 60);
+	}
 }
 
 /***************************************************************************/
@@ -674,7 +717,7 @@ static GnmFuncHelp const help_year[] = {
 };
 
 static GnmValue *
-gnumeric_year (FunctionEvalInfo *ei, GnmValue **argv)
+gnumeric_year (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
 	int res = 1900;
 	GDate date;
@@ -708,7 +751,7 @@ static GnmFuncHelp const help_month[] = {
 };
 
 static GnmValue *
-gnumeric_month (FunctionEvalInfo *ei, GnmValue **argv)
+gnumeric_month (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
 	int res = 1;
 	GDate date;
@@ -742,7 +785,7 @@ static GnmFuncHelp const help_day[] = {
 };
 
 static GnmValue *
-gnumeric_day (FunctionEvalInfo *ei, GnmValue **argv)
+gnumeric_day (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
 	int res = 1;
 	GDate date;
@@ -783,7 +826,7 @@ static GnmFuncHelp const help_weekday[] = {
 };
 
 static GnmValue *
-gnumeric_weekday (FunctionEvalInfo *ei, GnmValue **argv)
+gnumeric_weekday (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
 	GDate date;
 	int   res;
@@ -820,7 +863,7 @@ static GnmFuncHelp const help_days360[] = {
 	   "* If @method is 1, the European method will be used.  In this "
 	   "case, if the day of the month is 31 it will be considered as 30."
 	   "\n"
-	   "* If @method is 0 or omitted, the XL US method will be used.  "
+	   "* If @method is 0 or omitted, the MS Excel (tm) US method will be used.  "
 	   "This is a somewhat complicated industry standard method "
 	   "where the last day of February is considered to be the 30th day "
 	   "of the month, but only for the first date."
@@ -842,7 +885,7 @@ static GnmFuncHelp const help_days360[] = {
 };
 
 static GnmValue *
-gnumeric_days360 (FunctionEvalInfo *ei, GnmValue **argv)
+gnumeric_days360 (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
 	basis_t basis;
 	GDate date1, date2;
@@ -852,8 +895,8 @@ gnumeric_days360 (FunctionEvalInfo *ei, GnmValue **argv)
 	int method = argv[2] ? value_get_as_int (argv[2]) : 0;
 
 	switch (method) {
-	default: 
 	case 0: basis = BASIS_MSRB_30_360; break;
+	default: 
 	case 1: basis = BASIS_30E_360; break;
 	case 2: basis = BASIS_MSRB_30_360_SYM; break;
 	}
@@ -887,7 +930,7 @@ static GnmFuncHelp const help_eomonth[] = {
 };
 
 static GnmValue *
-gnumeric_eomonth (FunctionEvalInfo *ei, GnmValue **argv)
+gnumeric_eomonth (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
 	GnmValue *res;
 	int months = 0;
@@ -911,7 +954,7 @@ gnumeric_eomonth (FunctionEvalInfo *ei, GnmValue **argv)
 						  g_date_get_year (&date)));
 
 	res = value_new_int (datetime_g_to_serial (&date, conv));
-	value_set_fmt (res, style_format_default_date ());
+	value_set_fmt (res, go_format_default_date ());
 	return res;
 }
 
@@ -940,7 +983,7 @@ static GnmFuncHelp const help_workday[] = {
 };
 
 static GnmValue *
-gnumeric_workday (FunctionEvalInfo *ei, GnmValue **argv)
+gnumeric_workday (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
 	int days;
 	GDateWeekday weekday;
@@ -1067,7 +1110,7 @@ networkdays_holiday_callback (GnmValue const *v, GnmEvalPos const *ep,
 }
 
 static GnmValue *
-gnumeric_networkdays (FunctionEvalInfo *ei, GnmValue **argv)
+gnumeric_networkdays (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
 	int start_serial;
 	int end_serial;
@@ -1141,7 +1184,7 @@ static GnmFuncHelp const help_isoweeknum[] = {
 };
 
 static GnmValue *
-gnumeric_isoweeknum (FunctionEvalInfo *ei, GnmValue **argv)
+gnumeric_isoweeknum (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
 	GDate date;
 	datetime_value_to_g (&date, argv[0], DATE_CONV (ei->pos));
@@ -1179,7 +1222,7 @@ static GnmFuncHelp const help_isoyear[] = {
 };
 
 static GnmValue *
-gnumeric_isoyear (FunctionEvalInfo *ei, GnmValue **argv)
+gnumeric_isoyear (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
 	GDate date;
 	int year;
@@ -1232,7 +1275,7 @@ static GnmFuncHelp const help_weeknum[] = {
 };
 
 static GnmValue *
-gnumeric_weeknum (FunctionEvalInfo *ei, GnmValue **argv)
+gnumeric_weeknum (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
 	GDate date;
 	int method = argv[1] ? value_get_as_int (argv[1]) : 1;
@@ -1268,7 +1311,7 @@ static GnmFuncHelp const help_yearfrac[] = {
 };
 
 static GnmValue *
-gnumeric_yearfrac (FunctionEvalInfo *ei, GnmValue **argv)
+gnumeric_yearfrac (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
 	GODateConventions const *conv = DATE_CONV (ei->pos);
 	GDate start_date, end_date;
