@@ -20,7 +20,7 @@
  */
 
 #include <gnumeric-config.h>
-#include <gnumeric-i18n.h>
+#include <glib/gi18n.h>
 #include <gnumeric.h>
 #include "workbook-view.h"
 #include "workbook.h"
@@ -29,9 +29,8 @@
 #include "roff.h"
 #include "font.h"
 #include "cell.h"
-#include "io-context.h"
-#include "error-info.h"
-#include "plugin-util.h"
+#include <goffice/app/io-context.h>
+#include <goffice/app/error-info.h>
 #include "cellspan.h"
 
 #include <string.h>
@@ -41,18 +40,18 @@
  * escape special characters .. needs work
  */
 static int
-roff_fprintf (GsfOutput *output, const Cell *cell)
+roff_fprintf (GsfOutput *output, GnmCell *cell)
 {
 	int len, i;
 	const char *p;
 	char * s;
-	MStyle *mstyle;
+	GnmStyle *mstyle;
 
-	if (cell_is_blank (cell))
+	if (cell_is_empty (cell))
 		return 0;
 
 	mstyle = cell_get_mstyle (cell);
-	if (mstyle != NULL && mstyle_get_content_hidden (mstyle))
+	if (mstyle != NULL && gnm_style_get_content_hidden (mstyle))
 		return 0;
 
 	s = cell_get_rendered_text (cell);
@@ -85,11 +84,11 @@ roff_fprintf (GsfOutput *output, const Cell *cell)
  * FIXME: Should roff quote sheet name (and everything else)
  */
 void
-roff_file_save (GnmFileSaver const *fs, IOContext *io_context,
+roff_file_save (GOFileSaver const *fs, IOContext *io_context,
                 WorkbookView const *wb_view, GsfOutput *output)
 {
 	GList *sheets, *ptr;
-	Cell *cell;
+	GnmCell *cell;
 	int row, col, fontsize, v_size;
 	Workbook *wb = wb_view_workbook (wb_view);
 
@@ -100,7 +99,7 @@ roff_file_save (GnmFileSaver const *fs, IOContext *io_context,
 	sheets = workbook_sheets (wb);
 	for (ptr = sheets ; ptr != NULL ; ptr = ptr->next) {
 		Sheet *sheet = ptr->data;
-		Range r = sheet_get_extent (sheet, FALSE);
+		GnmRange r = sheet_get_extent (sheet, FALSE);
 
 		gsf_output_printf (output, "%s\n\n", sheet->name_unquoted);
 		gsf_output_printf (output, ".TS H\n");
@@ -123,48 +122,48 @@ roff_file_save (GnmFileSaver const *fs, IOContext *io_context,
 				if (!cell) {
 					gsf_output_printf (output, "l");
 				} else {
-					MStyle *mstyle = cell_get_mstyle (cell);
+					GnmStyle *mstyle = cell_get_mstyle (cell);
 					if (!mstyle)
 						break;
-					if (mstyle_get_align_h (mstyle) & HALIGN_RIGHT)
+					if (gnm_style_get_align_h (mstyle) & HALIGN_RIGHT)
 						gsf_output_printf (output, "r");
-					else if (mstyle_get_align_h (mstyle) == HALIGN_CENTER ||
+					else if (gnm_style_get_align_h (mstyle) == HALIGN_CENTER ||
 						 /* FIXME : center across selection is different */
-						 mstyle_get_align_h (mstyle) == HALIGN_CENTER_ACROSS_SELECTION)
+						 gnm_style_get_align_h (mstyle) == HALIGN_CENTER_ACROSS_SELECTION)
 						gsf_output_printf (output, "c");
 					else
 						gsf_output_printf (output, "l");
 					if (font_is_monospaced (mstyle)) {
-						if (mstyle_get_font_bold (mstyle) &&
-						    mstyle_get_font_italic (mstyle))
+						if (gnm_style_get_font_bold (mstyle) &&
+						    gnm_style_get_font_italic (mstyle))
 							gsf_output_printf (output, "fCBI");
-						else if (mstyle_get_font_bold (mstyle))
+						else if (gnm_style_get_font_bold (mstyle))
 							gsf_output_printf (output, "fCB");
-						else if (mstyle_get_font_italic (mstyle))
+						else if (gnm_style_get_font_italic (mstyle))
 							gsf_output_printf (output, "fCI");
 						else
 							gsf_output_printf (output, "fCR");
 					} else if (font_is_helvetica (mstyle)) {
-						if (mstyle_get_font_bold (mstyle) &&
-						    mstyle_get_font_italic (mstyle))
+						if (gnm_style_get_font_bold (mstyle) &&
+						    gnm_style_get_font_italic (mstyle))
 							gsf_output_printf (output, "fHBI");
-						else if (mstyle_get_font_bold (mstyle))
+						else if (gnm_style_get_font_bold (mstyle))
 							gsf_output_printf (output, "fHB");
-						else if (mstyle_get_font_italic (mstyle))
+						else if (gnm_style_get_font_italic (mstyle))
 							gsf_output_printf (output, "fHI");
 						else
 							gsf_output_printf (output, "fHR");
 					} else {
 						/* default is times */
-						if (mstyle_get_font_bold (mstyle) &&
-						    mstyle_get_font_italic (mstyle))
+						if (gnm_style_get_font_bold (mstyle) &&
+						    gnm_style_get_font_italic (mstyle))
 							gsf_output_printf (output, "fTBI");
-						else if (mstyle_get_font_bold (mstyle))
+						else if (gnm_style_get_font_bold (mstyle))
 							gsf_output_printf (output, "fTB");
-						else if (mstyle_get_font_italic (mstyle))
+						else if (gnm_style_get_font_italic (mstyle))
 							gsf_output_printf (output, "fTI");
 					}
-					fontsize = mstyle_get_font_size (mstyle);
+					fontsize = gnm_style_get_font_size (mstyle);
 					if (fontsize) {
 						gsf_output_printf (output, "p%d", fontsize);
 						v_size = v_size > fontsize ? v_size :
