@@ -4,15 +4,15 @@
  *
  * Kevin Handy <kth@srv.net>
  *
- * $Id: pln.h,v 1.4 1999/12/30 09:11:58 unammx Exp $
+ * $Id: pln.h,v 1.6 2005/02/08 06:32:16 jody Exp $
  */
 
 #ifndef GNUMERIC_PLN_H
 #define GNUMERIC_PLN_H
 
 #include "sheet.h"
-#include "plugin.h"
+#include <goffice/app/go-plugin.h>
 
-PluginInitResult       init_plugin (CommandContext *context, PluginData *pd);
+PluginInitResult       init_plugin (GOCmdContext *context, PluginData *pd);
 
 #endif /* GNUMERIC_PLN_H */
