@@ -17,7 +17,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  */
 
@@ -25,7 +25,7 @@
 
 /* TODO: Limit the number of include files a bit */
 #include <gnumeric-config.h>
-#include <gnumeric-i18n.h>
+#include <glib/gi18n.h>
 #include <gnumeric.h>
 #include "psiconv-plugin.h"
 #include "application.h"
@@ -42,27 +42,21 @@
 #include "selection.h"
 #include "position.h"
 #include "ranges.h"
-#include "io-context.h"
+#include <goffice/app/io-context.h>
 #include "workbook-view.h"
 #include "workbook.h"
-#include "error-info.h"
+#include <goffice/app/error-info.h>
 
 #include <string.h>
 #include <stdlib.h>
 
 #include <psiconv/parse.h>
 
-typedef struct {
-	GIConv converter;
-} PsiState;
-
-
 static GnmValue *
-psi_new_string (PsiState *state, gchar const *data)
+psi_new_string (psiconv_ucs2 const *data)
 {
 	return value_new_string_nocopy (
-		g_convert_with_iconv (data, -1, state->converter,
-				      NULL, NULL, NULL));
+		g_utf16_to_utf8 (data, -1, NULL, NULL, NULL));
 }
 
 
@@ -80,9 +74,9 @@ append_zeros (char *s, int n)
 
 
 static GnmCellRef *
-cellref_init (GnmCellRef *res,
-	      int row, gboolean row_abs,
-	      int col, gboolean col_abs)
+p_cellref_init (GnmCellRef *res,
+		int row, gboolean row_abs,
+		int col, gboolean col_abs)
 {
 	res->sheet = NULL;
 	res->row = row;
@@ -93,7 +87,7 @@ cellref_init (GnmCellRef *res,
 }
 
 static void
-set_format(MStyle *style, const psiconv_sheet_numberformat psi_numberformat)
+set_format(GnmStyle *style, const psiconv_sheet_numberformat psi_numberformat)
 {
 	/* 100 should be long enough, but to be really safe, use strncpy */
 	char fmt_string[100];
@@ -180,42 +174,43 @@ set_format(MStyle *style, const psiconv_sheet_numberformat psi_numberformat)
 	}  /* TODO: Add True/False */
 
 	if (fmt_string[0])
-		mstyle_set_format_text (style, fmt_string);
+		gnm_style_set_format_text (style, fmt_string);
 }
 
-static StyleColor *
+static GnmColor *
 get_color(const psiconv_color color)
 {
-	return style_color_new(color->red*256,color->green*256,color->blue*256);
+	return style_color_new_i8 (color->red, color->green, color->blue);
 }
 
 static void
-set_layout(MStyle * style,const psiconv_sheet_cell_layout psi_layout)
+set_layout(GnmStyle * style,const psiconv_sheet_cell_layout psi_layout)
 {
-	StyleColor *color;
+	GnmColor *color;
 
 	set_format(style,psi_layout->numberformat);
-	mstyle_set_font_size(style,psi_layout->character->font_size);
-	mstyle_set_font_italic(style,psi_layout->character->italic?TRUE:FALSE);
-	mstyle_set_font_bold(style,psi_layout->character->bold?TRUE:FALSE);
-	mstyle_set_font_uline(style,
+	gnm_style_set_font_size(style,psi_layout->character->font_size);
+	gnm_style_set_font_italic(style,psi_layout->character->italic?TRUE:FALSE);
+	gnm_style_set_font_bold(style,psi_layout->character->bold?TRUE:FALSE);
+	gnm_style_set_font_uline(style,
 	                      psi_layout->character->underline?TRUE:FALSE);
-	mstyle_set_font_strike(style,
+	gnm_style_set_font_strike(style,
 	                       psi_layout->character->strikethrough?TRUE:FALSE);
-	mstyle_set_font_name(style,psi_layout->character->font->name);
+	gnm_style_set_font_name(style,
+			     (const char *) psi_layout->character->font->name);
 	color = get_color(psi_layout->character->color);
 	if (color)
-		mstyle_set_color(style,MSTYLE_COLOR_FORE,color);
+		gnm_style_set_font_color (style, color);
 	/* TODO: Character level layouts: super_sub */
 	/* TODO: Paragraph level layouts: all */
 	/* TODO: Background color: add transparant if white */
 #if 0
 	color = get_color(psi_layout->paragraph->back_color);
 	if (color) {
-		mstyle_set_color(style,MSTYLE_COLOR_BACK,color);
-		mstyle_set_color(style,MSTYLE_COLOR_PATTERN,color);
+		gnm_style_set_back_color(style, color);
+		gnm_style_set_pattern_color(style, color);
 		/* TODO: Replace 24 with some symbol */
-		mstyle_set_pattern(style,1);
+		gnm_style_set_pattern(style,1);
 	}
 #endif
 }
@@ -224,10 +219,9 @@ set_layout(MStyle * style,const psiconv_sheet_cell_layout psi_layout)
 static void
 set_style(Sheet *sheet, int row, int col,
           const psiconv_sheet_cell_layout psi_layout,
-          const MStyle *default_style)
+          const GnmStyle *default_style)
 {
-	MStyle *style;
-	style = mstyle_copy(default_style);
+	GnmStyle *style = gnm_style_dup(default_style);
 	if (!style)
 		return;
 	set_layout(style,psi_layout);
@@ -235,7 +229,7 @@ set_style(Sheet *sheet, int row, int col,
 }
 
 static GnmValue *
-value_new_from_psi_cell(PsiState *state, const psiconv_sheet_cell psi_cell)
+value_new_from_psi_cell(const psiconv_sheet_cell psi_cell)
 {
 	switch (psi_cell->type) {
 	case psiconv_cell_int :
@@ -243,7 +237,7 @@ value_new_from_psi_cell(PsiState *state, const psiconv_sheet_cell psi_cell)
 	case psiconv_cell_float :
 		return value_new_float(psi_cell->data.dat_float);
 	case psiconv_cell_string :
-		return psi_new_string(state, psi_cell->data.dat_string);
+		return psi_new_string(psi_cell->data.dat_string);
 	case psiconv_cell_bool :
 		return value_new_bool(psi_cell->data.dat_bool);
 	case psiconv_cell_blank :
@@ -258,15 +252,14 @@ value_new_from_psi_cell(PsiState *state, const psiconv_sheet_cell psi_cell)
 	return NULL;
 }
 
-static GnmExpr *
-parse_subexpr(PsiState *state, const psiconv_formula psi_formula)
+static GnmExpr const *
+parse_subexpr(const psiconv_formula psi_formula)
 {
 	int nrargs=0; /* -1 for variable */
 	int kind=-1; /* 0 for dat, 1 for operator, 2 for formula, 3 for special,
 	               -1 for unknown */
 	psiconv_formula psi_form1,psi_form2;
-	GnmExpr *expr1=NULL,*expr2=NULL;
-	GnmCellRef *cr1=NULL,*cr2=NULL;
+	GnmExpr const *expr1=NULL,*expr2=NULL;
 
 	switch(psi_formula->type) {
 		/* Translates values */
@@ -314,8 +307,8 @@ parse_subexpr(PsiState *state, const psiconv_formula psi_formula)
 	/*	case psiconv_formula_mark_opsep: */
 	/*	case psiconv_formula_mark_opend: */
 		default:
-	/*		kind = -1; */
-	/*		break;     */
+			kind = -1;
+			break;
 	}
 
 	if (kind == -1) {
@@ -332,17 +325,17 @@ parse_subexpr(PsiState *state, const psiconv_formula psi_formula)
 			v = value_new_int(psi_formula->data.dat_int);
 			break;
 		case psiconv_formula_dat_string:
-			v = psi_new_string(state, psi_formula->data.dat_string);
+			v = psi_new_string(psi_formula->data.dat_string);
 			break;
 		case psiconv_formula_dat_cellblock: {
 			GnmCellRef cr1, cr2;
 
-			cellref_init (&cr1,
+			p_cellref_init (&cr1,
 				psi_formula->data.dat_cellblock.first.row.offset,
 				psi_formula->data.dat_cellblock.first.row.absolute,
 				psi_formula->data.dat_cellblock.first.column.offset,
 				psi_formula->data.dat_cellblock.first.column.absolute);
-			cellref_init (&cr2,
+			p_cellref_init (&cr2,
 				psi_formula->data.dat_cellblock.last.row.offset,
 				psi_formula->data.dat_cellblock.last.row.absolute,
 				psi_formula->data.dat_cellblock.last.column.offset,
@@ -352,6 +345,7 @@ parse_subexpr(PsiState *state, const psiconv_formula psi_formula)
 			break;
 		}
 		default:
+			break;
 		}
 		if (!v)
 			return NULL;
@@ -362,7 +356,7 @@ parse_subexpr(PsiState *state, const psiconv_formula psi_formula)
 			if (!(psi_form1 = psiconv_list_get
 			                  (psi_formula->data.fun_operands,0)))
 				return NULL;
-			if (!(expr1 = parse_subexpr(state, psi_form1)))
+			if (!(expr1 = parse_subexpr(psi_form1)))
 				return NULL;
 		}
 		if (nrargs >= 2) {
@@ -371,7 +365,7 @@ parse_subexpr(PsiState *state, const psiconv_formula psi_formula)
 				gnm_expr_unref(expr1);
 				return NULL;
 			}
-			if (!(expr2 = parse_subexpr(state, psi_form2))) {
+			if (!(expr2 = parse_subexpr(psi_form2))) {
 				gnm_expr_unref(expr1);
 				return NULL;
 			}
@@ -410,7 +404,7 @@ parse_subexpr(PsiState *state, const psiconv_formula psi_formula)
 		switch(psi_formula->type) {
 		case psiconv_formula_dat_cellref: {
 			GnmCellRef cr;
-			return gnm_expr_new_cellref (cellref_init (&cr,
+			return gnm_expr_new_cellref (p_cellref_init (&cr,
 				psi_formula->data.dat_cellref.row.offset,
 			        psi_formula->data.dat_cellref.row.absolute,
 			        psi_formula->data.dat_cellref.column.offset,
@@ -421,63 +415,64 @@ parse_subexpr(PsiState *state, const psiconv_formula psi_formula)
 			if (!(psi_form1 = psiconv_list_get
 			                  (psi_formula->data.fun_operands,0)))
 				return NULL;
-			return parse_subexpr(state, psi_form1);
+			return parse_subexpr(psi_form1);
 		default:
+			break;
 		}
 	}
 
 	return NULL;
 }
 
-static GnmExpr *
-expr_new_from_formula (PsiState *state,
-		       const psiconv_sheet_cell psi_cell,
+static GnmExpr const *
+expr_new_from_formula (const psiconv_sheet_cell psi_cell,
 		       const psiconv_formula_list psi_formulas)
 {
 	psiconv_formula formula;
 
 	formula = psiconv_get_formula (psi_formulas, psi_cell->ref_formula);
 
-	return (formula != NULL) ?  parse_subexpr (state, formula) : NULL;
+	return (formula != NULL) ?  parse_subexpr (formula) : NULL;
 }
 
 static void
-add_cell (PsiState *state, Sheet *sheet, const psiconv_sheet_cell psi_cell,
-	  const psiconv_formula_list psi_formulas, const MStyle * default_style)
+add_cell (Sheet *sheet, const psiconv_sheet_cell psi_cell,
+	  const psiconv_formula_list psi_formulas, const GnmStyle * default_style)
 {
-	Cell *cell;
+	GnmCell *cell;
 	GnmValue *val;
-	GnmExpr *expr;
-	psiconv_formula psi_formula;
+	GnmExpr const *expr = NULL;
 
 	cell = sheet_cell_fetch (sheet, psi_cell->column, psi_cell->row);
 	if (!cell)
 		return;
 
-	val = value_new_from_psi_cell (state, psi_cell);
+	val = value_new_from_psi_cell (psi_cell);
 
 	if (psi_cell->calculated)
-		expr = expr_new_from_formula (state, psi_cell, psi_formulas);
+		expr = expr_new_from_formula (psi_cell, psi_formulas);
 
 	if (expr != NULL) {
 		/* TODO : is there a notion of parse format ?
 		 * How does it store a user entered date ?
 		 */
 		if (val != NULL)
-			cell_set_expr_and_value (cell, expr, val, NULL, TRUE);
+			cell_set_expr_and_value (cell, expr, val, TRUE);
 		else
-			cell_set_expr (cell, expr, NULL);
+			cell_set_expr (cell, expr);
 	} else if (val != NULL) {
 		/* TODO : is there a notion of parse format ?
 		 * How does it store a user entered date ?
 		 */
-		cell_set_value (cell, val, NULL);
+		cell_set_value (cell, val);
 	} else {
 		/* TODO : send this warning to iocontext with details of
 		 * which sheet and cell.
 		 */
 		g_warning ("Cell with no value or expression ?");
 	}
+	if (expr)
+		gnm_expr_unref (expr);
 
 	/* TODO: Perhaps this must be moved above set_format */
 	set_style(sheet,psi_cell->row,psi_cell->column,psi_cell->layout,
@@ -485,10 +480,9 @@ add_cell (PsiState *state, Sheet *sheet, const psiconv_sheet_cell psi_cell,
 }
 
 static void
-add_cells(PsiState *state,
-	  Sheet *sheet, const psiconv_sheet_cell_list psi_cells,
+add_cells(Sheet *sheet, const psiconv_sheet_cell_list psi_cells,
           const psiconv_formula_list psi_formulas,
-          const MStyle *default_style)
+          const GnmStyle *default_style)
 {
 	psiconv_u32 i;
 	psiconv_sheet_cell psi_cell;
@@ -500,17 +494,17 @@ add_cells(PsiState *state,
 	for (i = 0; i < psiconv_list_length(psi_cells); i++) {
 		/* If psiconv_list_get fails, something is very wrong... */
 		if ((psi_cell = psiconv_list_get(psi_cells,i)))
-			add_cell(state, sheet,psi_cell,psi_formulas, default_style);
+			add_cell(sheet,psi_cell,psi_formulas, default_style);
 	}
 }
 
 static void
-add_worksheet(PsiState *state, Workbook *wb, psiconv_sheet_worksheet psi_worksheet,int nr,
+add_worksheet(Workbook *wb, psiconv_sheet_worksheet psi_worksheet,int nr,
               psiconv_formula_list psi_formulas)
 {
 	Sheet *sheet;
 	char *sheet_name;
-	MStyle *default_style;
+	GnmStyle *default_style;
 
 	sheet_name = g_strdup_printf (_("Sheet%d"),nr);
 	sheet = sheet_new (wb, sheet_name);
@@ -519,20 +513,21 @@ add_worksheet(PsiState *state, Workbook *wb, psiconv_sheet_worksheet psi_workshe
 		return;
 
 	/* Default layout */
-	default_style = mstyle_new_default();
+	default_style = gnm_style_new_default();
 	if (!default_style) {
-		sheet_destroy(sheet);
+		g_object_unref (sheet);
 		return;
 	}
 	set_layout(default_style,psi_worksheet->default_layout);
 
 	/* TODO: Add show_zeros */
 
-	add_cells(state, sheet,psi_worksheet->cells,psi_formulas,default_style);
+	add_cells(sheet,psi_worksheet->cells,psi_formulas,default_style);
 
 	/* TODO: What about the NULL? */
 	sheet_flag_recompute_spans(sheet);
-	workbook_sheet_attach (wb,sheet,NULL);
+	workbook_sheet_attach (wb, sheet);
+	gnm_style_unref (default_style);
 }
 
 static void
@@ -546,7 +541,7 @@ add_workbook(Workbook *wb, psiconv_sheet_workbook_section psi_workbook)
 		/* If psiconv_list_get fails, something is very wrong... */
 		if ((psi_worksheet = psiconv_list_get
 		                                 (psi_workbook->worksheets,i)))
-			add_worksheet(state, wb,psi_worksheet,i,
+			add_worksheet(wb,psi_worksheet,i,
 			              psi_workbook->formulas);
 	}
 
@@ -563,64 +558,104 @@ add_sheetfile(Workbook *wb, psiconv_sheet_f psi_file)
 
 
 static psiconv_buffer
-psiconv_file_to_buffer (GsfInput *input)
+psiconv_stream_to_buffer (GsfInput *input, int maxlen)
 {
 	psiconv_buffer buf;
+	gsf_off_t size;
+	int len;
 
-	if (!file) {
+	if (!input)
 		return NULL;
-	}
-	if (!(buf = psiconv_buffer_new())) {
+	if ((buf = psiconv_buffer_new()) == NULL)
 		return NULL;
-	}
-#warning BROKEN BROKEN BROKEN someone using this library needs to supply a way to read from GsfInputs
-	if (psiconv_buffer_fread_all(buf,file)) {
+	if (gsf_input_seek (input, 0, G_SEEK_SET) == TRUE) {
 		psiconv_buffer_free(buf);
 		return NULL;
 	}
+
+	size = gsf_input_size (input);
+	if (maxlen > 0 && size > maxlen)
+		size = maxlen;
+	for (; size > 0 ; size -= len) {
+		guint8 const *chunk;
+		int   i;
+
+		len = MIN (4096, size);
+		chunk = gsf_input_read (input, len, NULL);
+		if (chunk == NULL)
+			break;
+		for (i = 0; i<len; i++) {
+			if (psiconv_buffer_add(buf, chunk[i]) != 0) {
+				psiconv_buffer_free(buf);
+				return NULL;
+			}
+		}
+	}
+		
 	return buf;
 }
 
 void
 psiconv_read (IOContext *io_context, Workbook *wb, GsfInput *input)
 {
-	psiconv_buffer buf;
-	psiconv_file psi_file;
+	psiconv_buffer buf ;
+	psiconv_config config   = NULL;
+	psiconv_file   psi_file = NULL;
 
-	if (!(buf = psiconv_file_to_buffer (input))) {
+	if ((buf = psiconv_stream_to_buffer (input, -1)) == NULL) {
 		gnumeric_io_error_info_set (io_context,
 		                            error_info_new_str(_("Error while reading psiconv file.")));
-		return;
+		goto out;
 	}
 
-	/* For temporary debugging purposes */
-	/* psiconv_verbosity = PSICONV_VERB_DEBUG; */
-
-	if (psiconv_parse(buf,&psi_file)) {
+	if ((config = psiconv_config_default()) == NULL)
+		goto out;
+	psiconv_config_read(NULL,&config);
+	if (psiconv_parse(config, buf, &psi_file) != 0) {
+		psi_file = NULL;
 		gnumeric_io_error_info_set (io_context,
-		                            error_info_new_str(_("Error while parsing psiconv file.")));
-		psiconv_buffer_free(buf);
-		return;
+		                            error_info_new_str(_("Error while parsing Psion file.")));
+		goto out;
 	}
 
-	psiconv_buffer_free(buf);
+	if (psi_file->type == psiconv_sheet_file)
+		add_sheetfile(wb,psi_file->file);
+	else
+		gnumeric_io_error_info_set (io_context,
+		                            error_info_new_str(_("This Psion file is not a Sheet file.")));
+		
 
-	add_sheetfile(wb,psi_file->file);
-
-	psiconv_free_file(psi_file);
+out:
+	if (config)
+		psiconv_config_free(config);
+	if (buf)
+		psiconv_buffer_free(buf);
+	if (psi_file)
+		psiconv_free_file(psi_file);
 }
 
 gboolean
 psiconv_read_header (GsfInput *input)
 {
-	gboolean res;
-	psiconv_buffer buf;
+	gboolean res = FALSE;
+	psiconv_buffer buf = NULL;
+	psiconv_config config;
 
-	if (!(buf = psiconv_file_to_buffer (input)))
-		return FALSE;
+	if ((config = psiconv_config_default()) == NULL)
+		goto out;
+	psiconv_config_read(NULL,&config);
 
-	res =  psiconv_file_type(buf,NULL,NULL) == psiconv_sheet_file;
-	psiconv_buffer_free(buf);
+	if ((buf = psiconv_stream_to_buffer (input, 1024)) == NULL)
+		goto out;
+
+	if (psiconv_file_type(config, buf,NULL,NULL) == psiconv_sheet_file)
+		res = TRUE;
+
+out:
+	if (config)
+		psiconv_config_free(config);
+	if (buf)
+		psiconv_buffer_free(buf);
 	return res;
 }
 
