@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  */
 #include <gnumeric-config.h>
@@ -26,8 +26,8 @@
 #include "qpro.h"
 
 #include <gutils.h>
-#include <plugin-util.h>
-#include <file.h>
+#include <func.h>
+#include <goffice/app/file.h>
 #include <workbook-view.h>
 #include <workbook.h>
 #include <sheet.h>
@@ -37,19 +37,19 @@
 #include <sheet-style.h>
 #include <style-color.h>
 #include <parse-util.h>
-#include <module-plugin-defs.h>
-#include <io-context.h>
+#include <gnm-plugin.h>
+#include <goffice/app/io-context.h>
 
 #include <gsf/gsf-utils.h>
 #include <gsf/gsf-input.h>
 #include <gsf/gsf-infile.h>
 #include <gsf/gsf-infile-msole.h>
 
-GNUMERIC_MODULE_PLUGIN_INFO_DECL;
+GNM_PLUGIN_MODULE_HEADER;
 
-gboolean qpro_file_probe (GnmFileOpener const *fo, GsfInput *input,
+gboolean qpro_file_probe (GOFileOpener const *fo, GsfInput *input,
 			  FileProbeLevel pl);
-void     qpro_file_open (GnmFileOpener const *fo, IOContext *context,
+void     qpro_file_open (GOFileOpener const *fo, IOContext *context,
 			 WorkbookView *new_wb_view, GsfInput *input);
 
 static gboolean
@@ -71,7 +71,7 @@ qpro_check_signature (GsfInput *input)
 }
 
 gboolean
-qpro_file_probe (GnmFileOpener const *fo, GsfInput *input, FileProbeLevel pl)
+qpro_file_probe (GOFileOpener const *fo, GsfInput *input, FileProbeLevel pl)
 {
 	GsfInfile *ole;
 	GsfInput  *stream;
@@ -397,7 +397,7 @@ qpro_parse_formula (QProReadState *state, int col, int row,
 			ref.col_relative = (tmp & 0x4000) ? TRUE : FALSE;
 			ref.row_relative = (tmp & 0x2000) ? TRUE : FALSE;
 			if (ref.row_relative)
-				ref.row = ((gint16)(tmp & 0x1fff) << 3) >> 3;
+				ref.row = (int)(((gint16)((tmp & 0x1fff) << 3)) >> 3);
 			else
 				ref.row = tmp & 0x1fff;
 			expr = gnm_expr_new_cellref (&ref);
@@ -415,7 +415,7 @@ qpro_parse_formula (QProReadState *state, int col, int row,
 			a.col_relative = (tmp & 0x4000) ? TRUE : FALSE;
 			a.row_relative = (tmp & 0x2000) ? TRUE : FALSE;
 			if (a.row_relative)
-				a.row = ((gint16)(tmp & 0x1fff) << 3) >> 3;
+				a.row = (int)(((gint16)((tmp & 0x1fff) << 3)) >> 3);
 			else
 				a.row = tmp & 0x1fff;
 
@@ -425,7 +425,7 @@ qpro_parse_formula (QProReadState *state, int col, int row,
 			b.col_relative = (tmp & 0x4000) ? TRUE : FALSE;
 			b.row_relative = (tmp & 0x2000) ? TRUE : FALSE;
 			if (b.row_relative)
-				b.row = ((gint16)(tmp & 0x1fff) << 3) >> 3;
+				b.row = (int)(((gint16)((tmp & 0x1fff) << 3)) >> 3);
 			else
 				b.row = tmp & 0x1fff;
 
@@ -652,7 +652,7 @@ qpro_read_sheet (QProReadState *state)
 	Sheet *sheet = sheet_new (state->wb, def_name);
 
 	state->cur_sheet = sheet;
-	workbook_sheet_attach (state->wb, sheet, NULL);
+	workbook_sheet_attach (state->wb, sheet);
 	sheet_flag_recompute_spans (sheet);
 #if 0
 	printf ("----------> start %s\n", def_name);
@@ -720,7 +720,9 @@ qpro_read_sheet (QProReadState *state)
 
 		case QPRO_PROTECTION:
 			if (validate (QPRO_PROTECTION, 1))
-				sheet->is_protected = (data[0] == 0xff);
+				g_object_set (sheet,
+					      "protected", (data[0] == 0xff),
+					      NULL);
 			break;
 
 		case QPRO_PAGE_NAME:
@@ -729,8 +731,8 @@ qpro_read_sheet (QProReadState *state)
 					g_convert_with_iconv (data, -1,
 							      state->converter,
 							      NULL, NULL, NULL);
-#warning this is wrong, but the workbook interface is confused and needs a control.
-				sheet_rename (sheet, utf8name);
+#warning "This is wrong, but the workbook interface is confused and needs a control."
+				g_object_set (sheet, "name", utf8name, NULL);
 				g_free (utf8name);
 			}
 			break;
@@ -759,9 +761,14 @@ qpro_read_sheet (QProReadState *state)
 			break;
 
 		case QPRO_PAGE_TAB_COLOR :
-			if (validate (QPRO_PAGE_TAB_COLOR, 4))
-				sheet_set_tab_color (sheet, style_color_new_i8 (
-					data[0], data[1], data[2]), NULL);
+			if (validate (QPRO_PAGE_TAB_COLOR, 4)) {
+				GnmColor *bc = style_color_new_i8 (
+					data[0], data[1], data[2]);
+				g_object_set (sheet,
+					      "tab-background", bc,
+					      NULL);
+				style_color_unref (bc);
+			}
 			break;
 
 		case QPRO_PAGE_ZOOM_FACTOR :
@@ -774,8 +781,7 @@ qpro_read_sheet (QProReadState *state)
 						gnm_io_warning (state->io_context,
 							_("Invalid zoom %hd %%"), high);
 					else
-						sheet_set_zoom_factor (sheet, ((double)high) / 100.,
-							FALSE, FALSE);
+						g_object_set (sheet, "zoom-factor", high / 100.0, NULL);
 				}
 			}
 			break;
@@ -823,7 +829,7 @@ qpro_read_workbook (QProReadState *state, GsfInput *input)
 }
 
 void
-qpro_file_open (GnmFileOpener const *fo, IOContext *context,
+qpro_file_open (GOFileOpener const *fo, IOContext *context,
 		WorkbookView *new_wb_view, GsfInput *input)
 {
 	QProReadState state;
