@@ -31,7 +31,9 @@
 #include <gtk/gtkmain.h>
 #include <gtk/gtkiconfactory.h>
 #include <gtk/gtkselection.h>
+#include <gtk/gtkicontheme.h>
 #include <glib/gi18n.h>
+#include <goffice/utils/go-glib-extras.h>
 
 #define GNM_APP(o)		(G_TYPE_CHECK_INSTANCE_CAST((o), GNM_APP_TYPE, GnmApp))
 #define GNM_APP_CLASS(k)	(G_TYPE_CHECK_CLASS_CAST((k),	 GNM_APP_TYPE, GnmAppClass))
@@ -70,8 +72,6 @@ struct _GnmApp {
 	GtkWidget       *pref_dialog;
 
 	GList		*workbook_list;
-
-	GHashTable      *named_pixbufs;
 };
 
 typedef struct {
@@ -110,7 +110,11 @@ add_icon (GtkIconFactory *factory,
 		gtk_icon_set_add_source (set, src);	/* copies the src */
 	}
 
-	if (sized_data != NULL) {
+	/*
+	 * For now, don't register a fixed-sized icon as doing so without
+	 * catching style changes kills things like bug 302902.
+	 */
+	if (scalable_data == NULL && sized_data != NULL) {
 		gtk_icon_source_set_size (src, GTK_ICON_SIZE_MENU);
 		gtk_icon_source_set_size_wildcarded (src, FALSE);
 		gtk_icon_source_set_pixbuf (src,
@@ -124,10 +128,10 @@ add_icon (GtkIconFactory *factory,
 }
 
 /**
- * gnm_app_workbook_list_remove :
+ * gnm_app_workbook_list_add :
  * @wb :
  *
- * Remove @wb from the application's list of workbooks.
+ * Add @wb to the application's list of workbooks.
  **/
 void
 gnm_app_workbook_list_add (Workbook *wb)
@@ -240,29 +244,31 @@ gnm_app_clipboard_cut_copy (WorkbookControl *wbc, gboolean is_cut,
 			    SheetView *sv, GnmRange const *area,
 			    gboolean animate_cursor)
 {
+	Sheet *sheet;
+
 	g_return_if_fail (IS_SHEET_VIEW (sv));
 	g_return_if_fail (area != NULL);
 	g_return_if_fail (app != NULL);
 
 	gnm_app_clipboard_clear (FALSE);
+	sheet = sv_sheet (sv);
+	g_free (app->clipboard_cut_range);
+	app->clipboard_cut_range = range_dup (area);
+	sv_weak_ref (sv, &(app->clipboard_sheet_view));
+
+	if (!is_cut)
+		app->clipboard_copied_contents =
+			clipboard_copy_range (sheet, area);
+	if (animate_cursor) {
+		GList *l = g_list_append (NULL, (gpointer)area);
+		sv_ant (sv, l);
+		g_list_free (l);
+	}
+
 	if (wb_control_claim_selection (wbc)) {
-		Sheet *sheet = sv_sheet (sv);
-		g_free (app->clipboard_cut_range);
-		app->clipboard_cut_range = range_dup (area);
-		sv_weak_ref (sv, &(app->clipboard_sheet_view));
-
-		if (!is_cut)
-			app->clipboard_copied_contents =
-				clipboard_copy_range (sheet, area);
-
 		g_signal_emit (G_OBJECT (app), signals [CLIPBOARD_MODIFIED], 0);
-
-		if (animate_cursor) {
-			GList *l = g_list_append (NULL, (gpointer)area);
-			sv_ant (sv, l);
-			g_list_free (l);
-		}
 	} else {
+		gnm_app_clipboard_clear (FALSE);
 		g_warning ("Unable to set selection ?");
 	}
 }
@@ -279,38 +285,24 @@ void
 gnm_app_clipboard_cut_copy_obj (WorkbookControl *wbc, gboolean is_cut,
 				SheetView *sv, GSList *objects)
 {
-	SheetObject *so;
-	GnmRange *r;
-	GSList *ptr;
-
 	g_return_if_fail (IS_SHEET_VIEW (sv));
 	g_return_if_fail (objects != NULL);
 	g_return_if_fail (app != NULL);
 
 	gnm_app_clipboard_clear (FALSE);
+	g_free (app->clipboard_cut_range);
+	app->clipboard_cut_range = NULL;
+	sv_weak_ref (sv, &(app->clipboard_sheet_view));
+	app->clipboard_copied_contents 
+		= clipboard_copy_obj (sv_sheet (sv), objects);
+	if (is_cut) {
+		cmd_objects_delete (wbc, objects, _("Cut Object"));
+		objects = NULL;
+	}
 	if (wb_control_claim_selection (wbc)) {
-		g_free (app->clipboard_cut_range);
-		app->clipboard_cut_range = NULL;
-		sv_weak_ref (sv, &(app->clipboard_sheet_view));
-		app->clipboard_copied_contents = cellregion_new	(sv_sheet (sv));
-		for (ptr = objects ; ptr != NULL ; ptr = ptr->next) {
-			so = sheet_object_dup (ptr->data);
-			if (so != NULL) {
-				r = (GnmRange *) sheet_object_get_range	(so);
-				range_translate (r,
-					-MIN (r->start.col, r->end.col),
-					-MIN (r->start.row, r->end.row));
-				app->clipboard_copied_contents->objects = g_slist_prepend (
-					app->clipboard_copied_contents->objects, so);
-			}
-		}
-
-		if (is_cut) {
-			cmd_objects_delete (wbc, objects, _("Cut Object"));
-			objects = NULL;
-		}
 		g_signal_emit (G_OBJECT (app), signals [CLIPBOARD_MODIFIED], 0);
 	} else {
+		gnm_app_clipboard_clear (FALSE);
 		g_warning ("Unable to set selection ?");
 	}
 	g_slist_free (objects);
@@ -474,7 +466,7 @@ gnm_app_history_get_list (gboolean force_reload)
 		if (force_reload) {
 			GSList *tmp = app->history_list;
 			app->history_list = NULL;
-			gnm_slist_free_custom (tmp, g_free);
+			go_slist_free_custom (tmp, g_free);
 		} else
 			return app->history_list;
 	}
@@ -505,7 +497,7 @@ gnm_app_history_add (char const *uri)
 	/* force a reload in case max_entries has changed */
 	gnm_app_history_get_list (TRUE);
 	exists = g_slist_find_custom (app->history_list,
-				      uri, gnm_str_compare);
+				      uri, go_str_compare);
 
 	if (exists != NULL) {
 		/* its already the top of the stack no need to do anything */
@@ -525,14 +517,14 @@ gnm_app_history_add (char const *uri)
 	while (*ptr != NULL && max_entries-- > 0)
 		ptr = &((*ptr)->next);
 	if (*ptr != NULL) {
-		gnm_slist_free_custom (*ptr, g_free);
+		go_slist_free_custom (*ptr, g_free);
 		*ptr = NULL;
 	}
 
 	g_object_notify (G_OBJECT (app), "file-history-list");
 	gnm_gconf_set_file_history_files (
-		gnm_string_slist_copy (app->history_list));
-	go_conf_sync ();
+		go_string_slist_copy (app->history_list));
+	go_conf_sync (NULL);
 }
 
 gboolean gnm_app_use_auto_complete	  (void) { return gnm_app_prefs->auto_complete; }
@@ -544,17 +536,6 @@ void     gnm_app_set_transition_keys  (gboolean state)
 	((GnmAppPrefs *)gnm_app_prefs)->transition_keys = state;
 }
 
-/*
- * Get a named pixbuf.
- */
-GdkPixbuf *
-gnm_app_get_pixbuf (const char *name)
-{
-	g_return_val_if_fail (app != NULL, NULL);
-	return g_hash_table_lookup (app->named_pixbufs, name);
-}
-
-
 gpointer
 gnm_app_get_pref_dialog (void)
 {
@@ -650,9 +631,9 @@ gnumeric_application_setup_pixbufs (GnmApp *app)
 	for (ui = 0; ui < G_N_ELEMENTS (entry); ui++) {
 		GdkPixbuf *pixbuf = gdk_pixbuf_new_from_inline
 			(-1, entry[ui].scalable_data, FALSE, NULL);
-		g_hash_table_insert (app->named_pixbufs,
-				     (char *)entry[ui].name,
-				     pixbuf);
+		gtk_icon_theme_add_builtin_icon (entry[ui].name,
+			gdk_pixbuf_get_width (pixbuf), pixbuf);
+		g_object_unref (pixbuf);
 	}
 }
 
@@ -764,8 +745,10 @@ gnumeric_application_setup_icons (void)
 		unsigned int ui = 0;
 		GtkIconFactory *factory = gtk_icon_factory_new ();
 		for (ui = 0; ui < G_N_ELEMENTS (entry) ; ui++)
-			add_icon (factory, entry[ui].scalable_data,
-				  entry[ui].sized_data, entry[ui].stock_id);
+			add_icon (factory,
+				  entry[ui].scalable_data,
+				  entry[ui].sized_data,
+				  entry[ui].stock_id);
 		gtk_icon_factory_add_default (factory);
 		g_object_unref (G_OBJECT (factory));
 		done = TRUE;
@@ -780,7 +763,10 @@ gnumeric_application_finalize (GObject *obj)
 	g_slist_foreach (application->history_list, (GFunc)g_free, NULL);
 	g_slist_free (application->history_list);
 	application->history_list = NULL;
-	g_hash_table_destroy (application->named_pixbufs);
+
+	g_free (application->clipboard_cut_range);
+	application->clipboard_cut_range = NULL;
+
 	app = NULL;
 	G_OBJECT_CLASS (parent_klass)->finalize (obj);
 }
@@ -810,48 +796,48 @@ gnm_app_class_init (GObjectClass *gobject_klass)
 	g_object_class_install_property (gobject_klass, APPLICATION_PROP_FILE_HISTORY_LIST,
 		g_param_spec_pointer ("file-history-list", "File History List",
 			"A GSlist of filenames that have been read recently",
-			G_PARAM_READABLE));
+			GSF_PARAM_STATIC | G_PARAM_READABLE));
 
 	signals [WORKBOOK_ADDED] = g_signal_new ("workbook_added",
 		GNM_APP_TYPE,
 		G_SIGNAL_RUN_LAST,
 		G_STRUCT_OFFSET (GnmAppClass, workbook_added),
-		(GSignalAccumulator) NULL, NULL,
+		NULL, NULL,
 		g_cclosure_marshal_VOID__OBJECT,
 		G_TYPE_NONE, 1, WORKBOOK_TYPE);
 	signals [WORKBOOK_REMOVED] = g_signal_new ("workbook_removed",
 		GNM_APP_TYPE,
 		G_SIGNAL_RUN_LAST,
 		G_STRUCT_OFFSET (GnmAppClass, workbook_removed),
-		(GSignalAccumulator) NULL, NULL,
+		NULL, NULL,
 		g_cclosure_marshal_VOID__POINTER,
 		G_TYPE_NONE, 1, G_TYPE_POINTER);
 	signals [WINDOW_LIST_CHANGED] = g_signal_new ("window-list-changed",
 		GNM_APP_TYPE,
 		G_SIGNAL_RUN_LAST,
 		G_STRUCT_OFFSET (GnmAppClass, window_list_changed),
-		(GSignalAccumulator) NULL, NULL,
+		NULL, NULL,
 		g_cclosure_marshal_VOID__VOID,
 		G_TYPE_NONE, 0);
 	signals [CUSTOM_UI_ADDED] = g_signal_new ("custom-ui-added",
 		GNM_APP_TYPE,
 		G_SIGNAL_RUN_LAST,
 		G_STRUCT_OFFSET (GnmAppClass, custom_ui_added),
-		(GSignalAccumulator) NULL, NULL,
+		NULL, NULL,
 		g_cclosure_marshal_VOID__POINTER,
 		G_TYPE_NONE, 1, G_TYPE_POINTER);
 	signals [CUSTOM_UI_REMOVED] = g_signal_new ("custom-ui-removed",
 		GNM_APP_TYPE,
 		G_SIGNAL_RUN_LAST,
 		G_STRUCT_OFFSET (GnmAppClass, custom_ui_removed),
-		(GSignalAccumulator) NULL, NULL,
+		NULL, NULL,
 		g_cclosure_marshal_VOID__POINTER,
 		G_TYPE_NONE, 1, G_TYPE_POINTER);
 	signals [CLIPBOARD_MODIFIED] = g_signal_new ("clipboard_modified",
 		GNM_APP_TYPE,
 		G_SIGNAL_RUN_LAST,
 		G_STRUCT_OFFSET (GnmAppClass, clipboard_modified),
-		(GSignalAccumulator) NULL, NULL,
+		NULL, NULL,
 		g_cclosure_marshal_VOID__VOID,
 		G_TYPE_NONE, 0);
 }
@@ -866,10 +852,6 @@ gnm_app_init (GObject *obj)
 
 	gnm_app->workbook_list = NULL;
 
-	gnm_app->named_pixbufs = g_hash_table_new_full (g_str_hash, g_str_equal,
-							NULL,
-							(GDestroyNotify)g_object_unref);
-
 	gnumeric_application_setup_pixbufs (gnm_app);
 	gnumeric_application_setup_icons ();
 
