@@ -12,7 +12,7 @@
 #include "bonobo-io.h"
 
 #include "command-context.h"
-#include "io-context.h"
+#include <goffice/app/io-context.h>
 #include "workbook-control-component.h"
 #include "workbook-view.h"
 #include "workbook.h"
@@ -59,8 +59,8 @@ gnm_persist_stream_load (PortableServer_Servant  servant,
 
 	object        = bonobo_object_from_servant (servant);
 	wbc = GNM_PERSIST_STREAM (object)->wbc;
-	ioc = gnumeric_io_context_new (GNM_CMD_CONTEXT (wbc));
-	input = GSF_INPUT (gsf_input_bonobo_new (stream, NULL));
+	ioc = gnumeric_io_context_new (GO_CMD_CONTEXT (wbc));
+	input = gsf_input_bonobo_new (stream, NULL);
 	wb_view = wb_view_new_from_input  (input, NULL, ioc, NULL);
 	if (gnumeric_io_error_occurred (ioc) || wb_view == NULL) {
 		gnumeric_io_error_display (ioc);
@@ -81,8 +81,8 @@ gnm_persist_stream_load (PortableServer_Servant  servant,
 		 *        read only mode. For now, it doesn't mean a thing. */
 		/* goto exit_error; */
 	}
-	g_object_ref (G_OBJECT (wbc));
-	workbook_unref (old_wb);
+	g_object_ref (wbc);
+	g_object_unref (old_wb);
 	wb_control_set_view (wbc, wb_view, NULL);
 	wb_control_init_state (wbc);
 }
