@@ -88,7 +88,7 @@ cell_calc_layout (GnmCell const *cell, RenderedValue *rv, int y_direction,
 	}
 
 	if (rv->rotation && !rv->noborders) {
-		RenderedRotatedValue *rrv = (RenderedRotatedValue *)rv;
+		RenderedRotatedValue const *rrv = (RenderedRotatedValue *)rv;
 		double sin_a = rrv->rotmat.xy;
 		if (sin_a < 0) {
 			hoffset += (width - indent) - rv->layout_natural_width;
@@ -228,7 +228,7 @@ cell_draw (GnmCell const *cell, GdkGC *gc, GdkDrawable *drawable,
 			      height * PANGO_SCALE,
 			      h_center == -1 ? -1 : (h_center * PANGO_SCALE),
 			      &fore_color, &x, &y)) {
-		GdkColor tmp;
+		GdkColor fore_gdk;
 
 		/* +1 to get past left grid-line.  */
 		GdkRectangle rect;
@@ -253,23 +253,17 @@ cell_draw (GnmCell const *cell, GdkGC *gc, GdkDrawable *drawable,
 		 * approximation to the right effect.  (The right way
 		 * would be to create a proper cellspan type.)
 		 */
-		if (rv->rotation) {
-			rect.x = 0;
-			rect.y = 0;
-			rect.width = G_MAXINT;
-			rect.height = G_MAXINT;
-		}
-
-		gdk_gc_set_clip_rectangle (gc, &rect);
+		gdk_gc_set_clip_rectangle (gc,
+					   rv->rotation ? NULL : &rect);
 
 		/* See http://bugzilla.gnome.org/show_bug.cgi?id=105322 */
-		gdk_gc_set_rgb_fg_color (gc,
-			go_color_to_gdk (fore_color, &tmp));
+		go_color_to_gdk (fore_color, &fore_gdk);
+		gdk_gc_set_rgb_fg_color (gc, &fore_gdk);
 
 		if (rv->rotation) {
 			RenderedRotatedValue *rrv = (RenderedRotatedValue *)rv;
 			PangoContext *context = pango_layout_get_context (rv->layout);
-			const struct RenderedRotatedValueInfo *li = rrv->lines;
+			struct RenderedRotatedValueInfo const *li = rrv->lines;
 			GSList *lines;
 
 			pango_context_set_matrix (context, &rrv->rotmat);
@@ -278,17 +272,16 @@ cell_draw (GnmCell const *cell, GdkGC *gc, GdkDrawable *drawable,
 			     lines;
 			     lines = lines->next, li++) {
 				gdk_draw_layout_line (drawable, gc,
-						      x1 + PANGO_PIXELS (x + li->dx),
-						      y1 + PANGO_PIXELS (y + li->dy),
-						      lines->data);
+					x1 + PANGO_PIXELS (x + li->dx),
+					y1 + PANGO_PIXELS (y + li->dy),
+					lines->data);
 			}
 			pango_context_set_matrix (context, NULL);
 			pango_layout_context_changed (rv->layout);
-		} else {
+		} else
 			gdk_draw_layout (drawable, gc,
-					 x1 + PANGO_PIXELS (x),
-					 y1 + PANGO_PIXELS (y),
-					 rv->layout);
-		}
+				x1 + PANGO_PIXELS (x),
+				y1 + PANGO_PIXELS (y),
+				rv->layout);
 	}
 }
