@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  */
 #include <gnumeric-config.h>
@@ -153,8 +153,6 @@ colrow_foreach (ColRowCollection const *infos, int first, int last,
 {
 	int i;
 
-	g_return_val_if_fail (first >= 0, FALSE);
-
 	/* TODO : Do we need to support right -> left as an option */
 
 	/* clip */
