@@ -36,6 +36,7 @@
 #include "rendered-value.h" /* FIXME : should not be needed with JIT-R */
 #include "ranges.h"
 #include "gutils.h"
+#include "sheet-view.h"
 
 #include <string.h>
 #include <goffice/utils/go-glib-extras.h>
@@ -551,7 +552,7 @@ link_single_dep (GnmDependent *dep, GnmCellPos const *pos, GnmCellRef const *ref
 		deps = dep->sheet->deps;
 
 	/* Inserts if it is not already there */
-	cellref_get_abs_pos (ref, pos, &lookup.pos);
+	gnm_cellpos_init_cellref (&lookup.pos, ref, pos);
 	single = g_hash_table_lookup (deps->single_hash, &lookup);
 	if (single == NULL) {
 		single = go_mem_chunk_alloc (deps->single_pool);
@@ -574,7 +575,7 @@ unlink_single_dep (GnmDependent *dep, GnmCellPos const *pos, GnmCellRef const *a
 	if (!deps)
 		return;
 
-	cellref_get_abs_pos (a, pos, &lookup.pos);
+	gnm_cellpos_init_cellref (&lookup.pos, a, pos);
 	single = g_hash_table_lookup (deps->single_hash, &lookup);
 	if (single != NULL) {
 		micro_hash_remove (&single->deps, dep);
@@ -649,8 +650,8 @@ link_cellrange_dep (GnmDependent *dep, GnmCellPos const *pos,
 	DependencyRange range;
 	DependentFlags flag = DEPENDENT_NO_FLAG;
 
-	cellref_get_abs_pos (a, pos, &range.range.start);
-	cellref_get_abs_pos (b, pos, &range.range.end);
+	gnm_cellpos_init_cellref (&range.range.start, a, pos);
+	gnm_cellpos_init_cellref (&range.range.end, b, pos);
 	range_normalize (&range.range);
 
 	if (a->sheet != NULL) {
@@ -686,8 +687,8 @@ unlink_cellrange_dep (GnmDependent *dep, GnmCellPos const *pos,
 {
 	DependencyRange range;
 
-	cellref_get_abs_pos (a, pos, &range.range.start);
-	cellref_get_abs_pos (b, pos, &range.range.end);
+	gnm_cellpos_init_cellref (&range.range.start, a, pos);
+	gnm_cellpos_init_cellref (&range.range.end, b, pos);
 	range_normalize (&range.range);
 
 	if (a->sheet != NULL) {
@@ -711,27 +712,25 @@ unlink_cellrange_dep (GnmDependent *dep, GnmCellPos const *pos,
 }
 
 static DependentFlags
-link_expr_dep (GnmDependent *dep, GnmExpr const *tree)
+link_expr_dep (GnmEvalPos *ep, GnmExpr const *tree)
 {
 	g_return_val_if_fail (tree != NULL, DEPENDENT_NO_FLAG);
 
 	switch (tree->any.oper) {
 	case GNM_EXPR_OP_ANY_BINARY:
-		return  link_expr_dep (dep, tree->binary.value_a) |
-			link_expr_dep (dep, tree->binary.value_b);
+		return  link_expr_dep (ep, tree->binary.value_a) |
+			link_expr_dep (ep, tree->binary.value_b);
 	case GNM_EXPR_OP_ANY_UNARY:
-		return link_expr_dep (dep, tree->unary.value);
+		return link_expr_dep (ep, tree->unary.value);
 	case GNM_EXPR_OP_CELLREF:
-		return link_single_dep (dep, dependent_pos (dep), &tree->cellref.ref);
+		return link_single_dep (ep->dep, dependent_pos (ep->dep), &tree->cellref.ref);
 
 	case GNM_EXPR_OP_CONSTANT:
 		/* TODO: pass in eval flags so that we can use implicit
 		 * intersection
 		 */
 		if (VALUE_CELLRANGE == tree->constant.value->type)
-			return link_cellrange_dep
-				(dep,
-				 dependent_pos (dep),
+			return link_cellrange_dep (ep->dep, dependent_pos (ep->dep),
 				 &tree->constant.value->v_range.cell.a,
 				 &tree->constant.value->v_range.cell.b);
 		return DEPENDENT_NO_FLAG;
@@ -743,49 +742,53 @@ link_expr_dep (GnmDependent *dep, GnmExpr const *tree)
 		if (tree->func.func->fn_type == GNM_FUNC_TYPE_STUB)
 			gnm_func_load_stub (tree->func.func);
 		if (tree->func.func->linker) {
-			GnmEvalPos		 ep;
 			FunctionEvalInfo fei;
-			fei.pos = eval_pos_init_dep (&ep, dep);
+			fei.pos = ep;
 			fei.func_call = &tree->func;
 			flag = tree->func.func->linker (&fei);
 		}
-		for (l = tree->func.arg_list; l; l = l->next)
-			flag |= link_expr_dep (dep, l->data);
+		if (!(flag & DEPENDENT_IGNORE_ARGS))
+			for (l = tree->func.arg_list; l; l = l->next)
+				flag |= link_expr_dep (ep, l->data);
 		return flag;
 	}
 
 	case GNM_EXPR_OP_NAME:
-		expr_name_add_dep (tree->name.name, dep);
+		expr_name_add_dep (tree->name.name, ep->dep);
 		if (tree->name.name->active)
-			return link_expr_dep (dep, tree->name.name->expr) | DEPENDENT_USES_NAME;
+			return link_expr_dep (ep, tree->name.name->expr) | DEPENDENT_USES_NAME;
 		return DEPENDENT_USES_NAME;
 
 	case GNM_EXPR_OP_ARRAY:
 		if (tree->array.x != 0 || tree->array.y != 0) {
 			/* Non-corner cells depend on the corner */
 			GnmCellRef a;
-			GnmCellPos const *pos = dependent_pos (dep);
+			GnmCellPos const *pos = dependent_pos (ep->dep);
 			/* We cannot support array expressions unless
 			 * we have a position.
 			 */
 			g_return_val_if_fail (pos != NULL, DEPENDENT_NO_FLAG);
 
 			a.col_relative = a.row_relative = FALSE;
-			a.sheet = dep->sheet;
+			a.sheet = ep->dep->sheet;
 			a.col   = pos->col - tree->array.x;
 			a.row   = pos->row - tree->array.y;
 
-			return link_single_dep (dep, pos, &a);
-		} else
+			return link_single_dep (ep->dep, pos, &a);
+		} else {
+			GnmEvalPos pos = *ep;
+			pos.cols = tree->array.cols;
+			pos.rows = tree->array.rows;
 			/* Corner cell depends on the contents of the expr */
-			return link_expr_dep (dep, tree->array.corner.expr);
+			return link_expr_dep (&pos, tree->array.corner.expr);
+		}
 
 	case GNM_EXPR_OP_SET: {
 		GnmExprList *l;
 		DependentFlags res = DEPENDENT_NO_FLAG;
 
 		for (l = tree->set.set; l; l = l->next)
-			res |= link_expr_dep (dep, l->data);
+			res |= link_expr_dep (ep, l->data);
 		return res;
 	}
 	case GNM_EXPR_OP_RANGE_CTOR:
@@ -942,12 +945,12 @@ dynamic_dep_debug_name (GnmDependent const *dep, GString *target)
 }
 
 void
-dependent_add_dynamic_dep (GnmDependent *dep, GnmValueRange const *v)
+dependent_add_dynamic_dep (GnmDependent *dep, GnmRangeRef const *rr)
 {
-	DependentFlags   flags;
-	DynamicDep	*dyn;
-	GnmCellPos const	*pos;
-	DependencyRange  range;
+	DependentFlags    flags;
+	DynamicDep	 *dyn;
+	GnmCellPos const *pos;
+	DependencyRange   range;
 
 	g_return_if_fail (dep != NULL);
 
@@ -967,14 +970,14 @@ dependent_add_dynamic_dep (GnmDependent *dep, GnmValueRange const *v)
 		g_hash_table_insert (dep->sheet->deps->dynamic_deps, dep, dyn);
 	}
 
-	cellref_get_abs_pos (&v->cell.a, pos, &range.range.start);
-	cellref_get_abs_pos (&v->cell.b, pos, &range.range.end);
+	gnm_cellpos_init_cellref (&range.range.start, &rr->a, pos);
+	gnm_cellpos_init_cellref (&range.range.end, &rr->b, pos);
 	if (range_is_singleton (&range.range)) {
-		flags = link_single_dep (&dyn->base, pos, &v->cell.a);
-		dyn->singles = g_slist_prepend (dyn->singles, value_dup ((GnmValue *)v));
+		flags = link_single_dep (&dyn->base, pos, &rr->a);
+		dyn->singles = g_slist_prepend (dyn->singles, gnm_rangeref_dup (rr));
 	} else {
-		flags = link_cellrange_dep (&dyn->base, pos, &v->cell.a, &v->cell.b);
-		dyn->ranges = g_slist_prepend (dyn->ranges, value_dup ((GnmValue *)v));
+		flags = link_cellrange_dep (&dyn->base, pos, &rr->a, &rr->b);
+		dyn->ranges = g_slist_prepend (dyn->ranges, gnm_rangeref_dup (rr));
 	}
 	if (flags & DEPENDENT_HAS_3D)
 		workbook_link_3d_dep (dep);
@@ -997,7 +1000,8 @@ dependent_clear_dynamic_deps (GnmDependent *dep)
 void
 dependent_link (GnmDependent *dep)
 {
-	Sheet *sheet;
+	Sheet	   *sheet;
+	GnmEvalPos  ep;
 
 	g_return_if_fail (dep != NULL);
 	g_return_if_fail (dep->expression != NULL);
@@ -1015,9 +1019,8 @@ dependent_link (GnmDependent *dep)
 	else
 		sheet->deps->head = dep; /* first element */
 	sheet->deps->tail = dep;
-	dep->flags |=
-		DEPENDENT_IS_LINKED |
-		link_expr_dep (dep, dep->expression);
+	dep->flags |= DEPENDENT_IS_LINKED |
+		link_expr_dep (eval_pos_init_dep (&ep, dep), dep->expression);
 
 	if (dep->flags & DEPENDENT_HAS_3D)
 		workbook_link_3d_dep (dep);
@@ -1540,7 +1543,8 @@ cb_range_contained_collect (DependencyRange const *deprange,
 
 	if (range_overlap (user->target, range))
 		micro_hash_foreach_dep (deprange->deps, dep, {
-			if (!(dep->flags & (DEPENDENT_FLAGGED | DEPENDENT_CAN_RELOCATE))) {
+			if (!(dep->flags & (DEPENDENT_FLAGGED | DEPENDENT_CAN_RELOCATE)) &&
+			    dependent_type (dep) != DEPENDENT_DYNAMIC_DEP) {
 				dep->flags |= DEPENDENT_FLAGGED;
 				user->list = g_slist_prepend (user->list, dep);
 			}});
@@ -1553,7 +1557,8 @@ cb_single_contained_collect (DependencySingle const *depsingle,
 {
 	if (range_contains (user->target, depsingle->pos.col, depsingle->pos.row))
 		micro_hash_foreach_dep (depsingle->deps, dep, {
-			if (!(dep->flags & (DEPENDENT_FLAGGED | DEPENDENT_CAN_RELOCATE))) {
+			if (!(dep->flags & (DEPENDENT_FLAGGED | DEPENDENT_CAN_RELOCATE)) &&
+			    dependent_type (dep) != DEPENDENT_DYNAMIC_DEP) {
 				dep->flags |= DEPENDENT_FLAGGED;
 				user->list = g_slist_prepend (user->list, dep);
 			}});
@@ -1705,8 +1710,9 @@ cb_collect_range (G_GNUC_UNUSED gpointer key,
 static void
 dep_hash_destroy (GHashTable *hash, GSList **dyn_deps, Sheet *sheet, gboolean destroy)
 {
-	GSList *deps = NULL, *l2;
+	GSList *deps = NULL, *l;
 	GnmExprRewriteInfo rwinfo;
+	GSList *deplist = NULL;
 
 	/* We collect first because we will be changing the hash.  */
 	if (destroy) {
@@ -1718,10 +1724,8 @@ dep_hash_destroy (GHashTable *hash, GSList **dyn_deps, Sheet *sheet, gboolean de
 		g_hash_table_foreach (hash, (GHFunc)cb_collect_range, &deps);
 	}
 
-	rwinfo.rw_type = GNM_EXPR_REWRITE_INVALIDATE_SHEETS;
-	for (l2 = deps; l2; l2 = l2->next) {
-		DependencyAny *depany = l2->data;
-		GSList *deplist = NULL, *l;
+	for (l = deps; l; l = l->next) {
+		DependencyAny *depany = l->data;
 
 		micro_hash_foreach_dep (depany->deps, dep, {
 			if (dependent_type (dep) == DEPENDENT_DYNAMIC_DEP) {
@@ -1739,34 +1743,43 @@ dep_hash_destroy (GHashTable *hash, GSList **dyn_deps, Sheet *sheet, gboolean de
 			}
 		});
 
-		for (l = deplist; l; l = l->next) {
-			GnmDependent *dep = l->data;
-			GnmExpr *e = (GnmExpr *)dep->expression;
-			/* We are told this dependent depends on this region, hence if
-			 * newtree is null then either
-			 * 1) we did not depend on it (ie., serious breakage )
-			 * 2) we had a duplicate reference and we have already removed it.
-			 * 3) We depended on things via a name which will be
-			 *    invalidated elsewhere */
-			GnmExpr const *newtree = gnm_expr_rewrite (e, &rwinfo);
-			if (newtree != NULL) {
-				if (!destroy) {
-					gnm_expr_ref (e);
-					sheet->revive.dep_exprs =
-						g_slist_prepend
-						(g_slist_prepend (sheet->revive.dep_exprs, e),
-						 dep);
-				}
-				dependent_set_expr (dep, newtree);
-				gnm_expr_unref (newtree);
-			}
-		}
-		g_slist_free (deplist);
-
 		if (destroy)
 			micro_hash_release (&depany->deps);
 	}
 	g_slist_free (deps);
+
+	/*
+	 * We do this after the above loop as this loop will
+	 * invalidate some of the DependencyAny pointers from
+	 * above.  The testcase for that is 314207, deleting
+	 * all but the first sheet in one go.
+	 */
+	rwinfo.rw_type = GNM_EXPR_REWRITE_INVALIDATE_SHEETS;
+	for (l = deplist; l; l = l->next) {
+		GnmDependent *dep = l->data;
+		GnmExpr *e = (GnmExpr *)dep->expression;
+		/* We are told this dependent depends on this region, hence if
+		 * newtree is null then either
+		 * 1) we did not depend on it (ie., serious breakage )
+		 * 2) we had a duplicate reference and we have already removed it.
+		 * 3) We depended on things via a name which will be
+		 *    invalidated elsewhere */
+		GnmExpr const *newtree = gnm_expr_rewrite (e, &rwinfo);
+		if (newtree != NULL) {
+			if (!destroy) {
+				gnm_expr_ref (e);
+				sheet->revive.dep_exprs =
+					g_slist_prepend
+					(g_slist_prepend (sheet->revive.dep_exprs, e),
+					 dep);
+			}
+			dependent_set_expr (dep, newtree);
+			gnm_expr_unref (newtree);
+		}
+	}
+	g_slist_free (deplist);
+
+
 }
 
 static void
@@ -2233,7 +2246,9 @@ workbook_recalc (Workbook *wb)
 
 	WORKBOOK_FOREACH_DEPENDENT (wb, dep, redraw |= dependent_eval (dep););
 	if (redraw) {
-		WORKBOOK_FOREACH_SHEET (wb, sheet, sheet_redraw_all (sheet, FALSE););
+		WORKBOOK_FOREACH_SHEET (wb, sheet, {
+			SHEET_FOREACH_VIEW (sheet, sv, sv_flag_selection_change (sv););
+			sheet_redraw_all (sheet, FALSE);});
 	}
 }
 
@@ -2257,21 +2272,21 @@ dynamic_dep_free (DynamicDep *dyn)
 {
 	GnmDependent *dep = dyn->container;
 	GnmCellPos const *pos = dependent_pos (dep);
-	GnmValueRange *v;
+	GnmRangeRef *rr;
 	GSList *ptr;
 
 	for (ptr = dyn->singles ; ptr != NULL ; ptr = ptr->next) {
-		v = ptr->data;
-		unlink_single_dep (&dyn->base, pos, &v->cell.a);
-		value_release ((GnmValue *)v);
+		rr = ptr->data;
+		unlink_single_dep (&dyn->base, pos, &rr->a);
+		g_free (rr);
 	}
 	g_slist_free (dyn->singles);
 	dyn->singles = NULL;
 
 	for (ptr = dyn->ranges ; ptr != NULL ; ptr = ptr->next) {
-		v = ptr->data;
-		unlink_cellrange_dep (&dyn->base, pos, &v->cell.a, &v->cell.b);
-		value_release ((GnmValue *)v);
+		rr = ptr->data;
+		unlink_cellrange_dep (&dyn->base, pos, &rr->a, &rr->b);
+		g_free (rr);
 	}
 	g_slist_free (dyn->ranges);
 	dyn->ranges = NULL;
@@ -2383,16 +2398,14 @@ dump_dynamic_dep (gpointer key, G_GNUC_UNUSED gpointer value,
 
 	g_string_append (target, ", s=[");
 	for (l = dyn->singles; l; l = l->next) {
-		GnmValueRange const *v = l->data;
-		rangeref_as_string (target, gnm_expr_conventions_default, &v->cell, &pp);
+		rangeref_as_string (target, gnm_expr_conventions_default, l->data, &pp);
 		if (l->next)
 			g_string_append (target, ", ");
 	}
 
 	g_string_append (target, "], r=[");
 	for (l = dyn->ranges; l; l = l->next) {
-		GnmValueRange const *v = l->data;
-		rangeref_as_string (target, gnm_expr_conventions_default, &v->cell, &pp);
+		rangeref_as_string (target, gnm_expr_conventions_default, l->data, &pp);
 		if (l->next)
 			g_string_append (target, ", ");
 	}
