@@ -163,6 +163,7 @@ typedef struct _FormatState {
 	} align;
 	struct {
 		FontSelector	*selector;
+		GtkToggleButton	*superscript, *subscript;
 		ColorPicker      color;
 	} font;
 	struct {
@@ -301,7 +302,8 @@ setup_pattern_button (GdkScreen *screen,
 		      PatternPicker *picker,
 		      gboolean const flag,
 		      int const index,
-		      int const select_index)
+		      int const select_index,
+		      unsigned size)
 {
 	GtkWidget *tmp = glade_xml_get_widget (gui, name);
 	if (tmp != NULL) {
@@ -309,7 +311,7 @@ setup_pattern_button (GdkScreen *screen,
 		if (flag) {
 			GdkPixbuf *pixbuf = gtk_icon_theme_load_icon (
 				gtk_icon_theme_get_for_screen (screen),
-				name, 16, 0, NULL);
+				name, size, 0, NULL);
 			GtkWidget *image = gtk_image_new_from_pixbuf (pixbuf);
 			g_object_unref (pixbuf);
 			gtk_widget_show (image);
@@ -688,6 +690,7 @@ cb_font_changed (G_GNUC_UNUSED GtkWidget *widget,
 		MSTYLE_FONT_ITALIC,
 		MSTYLE_FONT_UNDERLINE,
 		MSTYLE_FONT_STRIKETHROUGH,
+		MSTYLE_FONT_SCRIPT,
 		MSTYLE_FONT_COLOR
 	};
 	int i;
@@ -742,6 +745,25 @@ cb_font_strike_toggle (GtkToggleButton *button, FormatState *state)
 			gtk_toggle_button_get_active (button));
 	}
 }
+static void
+cb_font_script_toggle (GtkToggleButton *button, FormatState *state)
+{
+	if (state->enable_edit) {
+		GOFontScript script = GO_FONT_SCRIPT_STANDARD;
+		if (gtk_toggle_button_get_active (button)) {
+			state->enable_edit = FALSE;
+			if (button == state->font.superscript) {
+				script = GO_FONT_SCRIPT_SUPER;
+				gtk_toggle_button_set_active (state->font.subscript, FALSE);
+			} else {
+				script = GO_FONT_SCRIPT_SUB;
+				gtk_toggle_button_set_active (state->font.superscript, FALSE);
+			}
+			state->enable_edit = TRUE;
+		}
+		font_selector_set_script (state->font.selector, script);
+	}
+}
 
 static gboolean
 cb_font_underline_changed (G_GNUC_UNUSED GtkWidget *ct,
@@ -775,6 +797,7 @@ fmt_dialog_init_font_page (FormatState *state)
 	char const *uline_str;
 	GtkWidget *strike = glade_xml_get_widget (state->gui, "strikethrough_button");
 	gboolean   strikethrough = FALSE;
+	GOFontScript script = GO_FONT_SCRIPT_STANDARD;
 	int i;
 
 	g_return_if_fail (container != NULL);
@@ -825,14 +848,30 @@ fmt_dialog_init_font_page (FormatState *state)
 
 	if (0 == (state->conflicts & (1 << MSTYLE_FONT_STRIKETHROUGH)))
 		strikethrough = gnm_style_get_font_strike (state->style);
-
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (strike), strikethrough);
 	font_selector_set_strike (state->font.selector, strikethrough);
-
 	g_signal_connect (G_OBJECT (strike),
 		"toggled",
 		G_CALLBACK (cb_font_strike_toggle), state);
 
+	if (0 == (state->conflicts & (1 << MSTYLE_FONT_SCRIPT)))
+		script = gnm_style_get_font_script (state->style);
+	font_selector_set_script (state->font.selector, script);
+	if (NULL != (tmp = glade_xml_get_widget (state->gui, "superscript_button"))) {
+		state->font.superscript = GTK_TOGGLE_BUTTON (tmp);
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (tmp),
+			script == GO_FONT_SCRIPT_SUPER);
+		g_signal_connect (G_OBJECT (tmp), "toggled",
+			G_CALLBACK (cb_font_script_toggle), state);
+	}
+	if (NULL != (tmp =  glade_xml_get_widget (state->gui, "subscript_button"))) {
+		state->font.subscript = GTK_TOGGLE_BUTTON (tmp);
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (tmp),
+			script == GO_FONT_SCRIPT_SUB);
+		g_signal_connect (G_OBJECT (tmp), "toggled",
+			G_CALLBACK (cb_font_script_toggle), state);
+	}
+
 	if (0 == (state->conflicts & (1 << MSTYLE_FONT_COLOR)))
 		font_selector_set_color (
 			state->font.selector,
@@ -1476,7 +1515,7 @@ fmt_dialog_init_protection_page (FormatState *state)
 		G_CALLBACK (cb_protection_hidden_toggle), state);
 
 	state->protection.sheet_protected_changed = FALSE;
-	flag = wb_control_view (WORKBOOK_CONTROL (state->wbcg))->is_protected;
+	flag = state->sheet->is_protected; 
 	w = glade_xml_get_widget (state->gui, "protection_sheet_protected");
 	state->protection.sheet_protected = GTK_CHECK_BUTTON (w);
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w), flag);
@@ -1888,6 +1927,27 @@ fmt_dialog_init_input_msg_page (FormatState *state)
 static void
 cb_fmt_dialog_dialog_buttons (GtkWidget *btn, FormatState *state)
 {
+#if 0
+	static StyleBorderLocation const bmap_ltr[] = {
+		STYLE_BORDER_TOP,	STYLE_BORDER_BOTTOM,
+		STYLE_BORDER_LEFT,	STYLE_BORDER_RIGHT,
+		STYLE_BORDER_REV_DIAG,	STYLE_BORDER_DIAG,
+		STYLE_BORDER_HORIZ,	STYLE_BORDER_VERT
+	};
+	static StyleBorderLocation const bmap_rtl[] = {
+		STYLE_BORDER_TOP,	STYLE_BORDER_BOTTOM,
+		/* reverse */
+		STYLE_BORDER_RIGHT,	STYLE_BORDER_LEFT,
+		/* reverse */
+		STYLE_BORDER_DIAG,	STYLE_BORDER_REV_DIAG,
+		STYLE_BORDER_HORIZ,	STYLE_BORDER_VERT
+	};
+	StyleBorderLocation const *bmap = bmap_ltr;
+
+	if (NULL != state->sheet && state->sheet->text_is_rtl)
+		bmap = bmap_rtl;
+#endif
+
 	if (btn == state->apply_button || btn == state->ok_button) {
 		GnmBorder *borders[STYLE_BORDER_EDGE_MAX];
 		int i;
@@ -1915,8 +1975,7 @@ cb_fmt_dialog_dialog_buttons (GtkWidget *btn, FormatState *state)
 		}
 
 		if (state->protection.sheet_protected_changed) {
-			WorkbookView *wbv = wb_control_view (WORKBOOK_CONTROL (state->wbcg));
-			wbv->is_protected = state->protection.sheet_protected_value;
+			state->sheet->is_protected = state->protection.sheet_protected_value;
 			state->protection.sheet_protected_changed = FALSE;
 		}
 
@@ -1926,7 +1985,7 @@ cb_fmt_dialog_dialog_buttons (GtkWidget *btn, FormatState *state)
 			borders[i] = border_get_mstyle (state, i);
 
 		cmd_selection_format (WORKBOOK_CONTROL (state->wbcg),
-			    state->result, borders, NULL);
+			state->result, borders, NULL);
 
 		gnm_style_unref (state->result);
 		sheet_update (state->sheet);
@@ -2141,7 +2200,7 @@ fmt_dialog_impl (FormatState *state, FormatDialogPosition_t pageno)
 				      state->gui, name, &state->border.pattern,
 				      i != 0, /* No image for None */
 				      line_pattern_buttons[i].pattern,
-				      default_border_style);
+				      default_border_style, 54);
 
 	setup_color_pickers (state, &state->border.color,	"border_color_group",
 			     "border_color_hbox",		"border_color_label",
@@ -2195,7 +2254,7 @@ fmt_dialog_impl (FormatState *state, FormatDialogPosition_t pageno)
 				      state->gui, name,
 				      &state->back.pattern, TRUE,
 				      i+1, /* Pattern #s start at 1 */
-				      selected);
+				      selected, 16);
 
 	/* If the pattern is 0 indicating no background colour
 	 * Set background to No colour.  This will set states correctly.
