@@ -31,6 +31,7 @@
 #include <expr.h>
 #include <str.h>
 #include <expr-name.h>
+#include <selection.h>
 #include <sheet.h>
 #include <sheet-view.h>
 #include <workbook.h>
@@ -247,7 +248,8 @@ cb_scope_changed (G_GNUC_UNUSED GtkToggleButton *button, NameGuruState *state)
 	err = expr_name_set_scope (state->cur_name,
 		name_guru_scope_is_sheet (state) ? state->sheet : NULL);
 	if (err != NULL) {
-		gnumeric_notice (state->wbcg, GTK_MESSAGE_ERROR, err);
+		go_gtk_notice_dialog (GTK_WINDOW (state->dialog),
+				 GTK_MESSAGE_ERROR, err);
 		g_free (err);
 		name_guru_display_scope (state); /* flip it back */
 	} else
@@ -345,7 +347,8 @@ name_guru_add (NameGuruState *state)
 		if (perr.err == NULL)
 			return TRUE;
 
-		gnumeric_notice (state->wbcg, GTK_MESSAGE_ERROR, perr.err->message);
+		go_gtk_notice_dialog (GTK_WINDOW (state->dialog),
+				 GTK_MESSAGE_ERROR, perr.err->message);
 		gtk_widget_grab_focus (GTK_WIDGET (state->expr_entry));
 		parse_error_free (&perr);
 		return FALSE;
@@ -355,7 +358,7 @@ name_guru_add (NameGuruState *state)
 	 * because it will be would disappear from the lists.
 	 */
 	if (gnm_expr_is_err (expr, GNM_ERROR_NAME)) {
-		gnumeric_notice (state->wbcg, GTK_MESSAGE_ERROR,
+		go_gtk_notice_dialog (GTK_WINDOW (state->dialog), GTK_MESSAGE_ERROR,
 			_("Why would you want to define a name to be #NAME?"));
 		gtk_widget_grab_focus (GTK_WIDGET (state->expr_entry));
 		parse_error_free (&perr);
@@ -428,7 +431,7 @@ cb_name_guru_destroy (NameGuruState *state)
 		state->gui = NULL;
 	}
 
-	wbcg_edit_finish (state->wbcg, FALSE, NULL);
+	wbcg_edit_finish (state->wbcg, WBC_EDIT_REJECT, NULL);
 
 	state->dialog = NULL;
 
@@ -450,19 +453,30 @@ cb_entry_activate (G_GNUC_UNUSED GtkWidget *item,
 	name_guru_add (state);
 }
 
+static void
+load_selection (NameGuruState *state)
+{
+	GnmRange const *first = selection_first_range (state->sv, NULL, NULL);
+
+	if (first != NULL)
+		gnm_expr_entry_load_from_range (state->expr_entry,
+						state->sheet, first);
+}
+
+
 static gboolean
 name_guru_init (NameGuruState *state, WorkbookControlGUI *wbcg)
 {
 	Workbook *wb = wb_control_workbook (WORKBOOK_CONTROL (wbcg));
-	GtkTable *table2;
+	GtkTable *definition_table;
 	GtkTreeViewColumn *column;
 
 	state->wbcg  = wbcg;
 	state->wb   = wb;
 	state->sv = wb_control_cur_sheet_view (WORKBOOK_CONTROL (wbcg));
 	state->sheet = sv_sheet (state->sv);
-	state->gui = gnm_glade_xml_new (GNM_CMD_CONTEXT (wbcg),
-		"names.glade", NULL, NULL);
+	state->gui = gnm_glade_xml_new (GO_CMD_CONTEXT (wbcg),
+		"define-name.glade", NULL, NULL);
         if (state->gui == NULL)
                 return TRUE;
 
@@ -471,11 +485,11 @@ name_guru_init (NameGuruState *state, WorkbookControlGUI *wbcg)
 			state->sv->edit_pos.row);
 
 	state->dialog = glade_xml_get_widget (state->gui, "NameGuru");
-	table2 = GTK_TABLE (glade_xml_get_widget (state->gui, "table2"));
+	definition_table = GTK_TABLE (glade_xml_get_widget (state->gui, "definition_table"));
 	state->name  = GTK_ENTRY (glade_xml_get_widget (state->gui, "name"));
 	state->expr_entry = gnm_expr_entry_new (state->wbcg, TRUE);
 	gnm_expr_entry_set_absolute (state->expr_entry);
-	gtk_table_attach (table2, GTK_WIDGET (state->expr_entry),
+	gtk_table_attach (definition_table, GTK_WIDGET (state->expr_entry),
 			  1, 2, 1, 2,
 			  GTK_EXPAND | GTK_FILL, 0,
 			  0, 0);
@@ -510,7 +524,9 @@ name_guru_init (NameGuruState *state, WorkbookControlGUI *wbcg)
 	state->ok_button     = name_guru_init_button (state, "ok_button");
 	state->close_button  = name_guru_init_button (state, "close_button");
 	state->add_button    = name_guru_init_button (state, "add_button");
+	gtk_button_set_alignment (GTK_BUTTON (state->add_button), 0., .5);
 	state->delete_button = name_guru_init_button (state, "delete_button");
+	gtk_button_set_alignment (GTK_BUTTON (state->delete_button), 0., .5);
 	state->update_button = name_guru_init_button (state, "update_button");
 
 	g_signal_connect (G_OBJECT (state->selection),
@@ -535,9 +551,10 @@ name_guru_init (NameGuruState *state, WorkbookControlGUI *wbcg)
 		G_CALLBACK (cb_entry_activate), state);
 
 	name_guru_populate_list (state);
+	load_selection (state);
 
 	gnumeric_init_help_button (
-		glade_xml_get_widget (state->gui, "helpbutton"),
+		glade_xml_get_widget (state->gui, "help_button"),
 		GNUMERIC_HELP_LINK_DEFINE_NAMES);
 
 	/* a candidate for merging into attach guru */
@@ -545,7 +562,7 @@ name_guru_init (NameGuruState *state, WorkbookControlGUI *wbcg)
 			       DEFINE_NAMES_KEY);
 	g_object_set_data_full (G_OBJECT (state->dialog),
 		"state", state, (GDestroyNotify)cb_name_guru_destroy);
-	gnumeric_non_modal_dialog (wbcg_toplevel (state->wbcg),
+	go_gtk_nonmodal_dialog (wbcg_toplevel (state->wbcg),
 				   GTK_WINDOW (state->dialog));
 	wbcg_edit_attach_guru (state->wbcg, state->dialog);
 	gtk_widget_show_all (GTK_WIDGET (state->dialog));
@@ -576,7 +593,7 @@ dialog_define_names (WorkbookControlGUI *wbcg)
 
 	state = g_new0 (NameGuruState, 1);
 	if (name_guru_init (state, wbcg)) {
-		gnumeric_notice (wbcg, GTK_MESSAGE_ERROR,
+		go_gtk_notice_dialog (wbcg_toplevel (wbcg), GTK_MESSAGE_ERROR,
 				 _("Could not create the Name Guru."));
 		g_free (state);
 		return;
