@@ -33,7 +33,7 @@
 #include <expr.h>
 #include <commands.h>
 #include <dependent.h>
-#include <format.h>
+#include <gnm-format.h>
 #include <value.h>
 #include <mstyle.h>
 #include <ranges.h>
@@ -102,7 +102,7 @@ goal_seek_eval (gnm_float x, gnm_float *y, void *vevaldata)
 
 	if (evaldata->ycell->value) {
 	        *y = value_get_as_float (evaldata->ycell->value) - evaldata->ytarget;
-		if (finitegnum (*y))
+		if (gnm_finite (*y))
 			return GOAL_SEEK_OK;
 	}
 
@@ -119,7 +119,7 @@ gnumeric_goal_seek (GoalSeekState *state)
 	gboolean hadold;
 	gnm_float oldx;
 
-	goal_seek_initialise (&seekdata);
+	goal_seek_initialize (&seekdata);
 	seekdata.xmin = state->xmin;
 	seekdata.xmax = state->xmax;
 
@@ -347,7 +347,7 @@ cb_dialog_apply_clicked (G_GNUC_UNUSED GtkWidget *button,
   	gnm_float max_range_val = 1e24;
 	GnmValue *target;
 	GnmRangeRef const *r;
-	GnmFormat *format;
+	GOFormat *format;
 
 	if (state->warning_dialog != NULL)
 		gtk_widget_destroy (state->warning_dialog);
@@ -356,7 +356,7 @@ cb_dialog_apply_clicked (G_GNUC_UNUSED GtkWidget *button,
 	target = gnm_expr_entry_parse_as_value (state->set_cell_entry,
 						state->sheet);
 	if (target == NULL) {
-		gnumeric_notice_nonmodal (GTK_WINDOW(state->dialog),
+		go_gtk_notice_nonmodal_dialog (GTK_WINDOW(state->dialog),
 					  &(state->warning_dialog),
 					  GTK_MESSAGE_ERROR,
 					  _("You should introduce a valid cell "
@@ -368,7 +368,7 @@ cb_dialog_apply_clicked (G_GNUC_UNUSED GtkWidget *button,
 	state->set_cell = sheet_cell_get (r->a.sheet, r->a.col, r->a.row);
 	value_release (target);
 	if (state->set_cell == NULL || !cell_has_expr (state->set_cell)) {
-		gnumeric_notice_nonmodal (GTK_WINDOW(state->dialog),
+		go_gtk_notice_nonmodal_dialog (GTK_WINDOW(state->dialog),
 					  &(state->warning_dialog),
 					  GTK_MESSAGE_ERROR,
 					  _("The cell named in 'Set Cell:' "
@@ -381,7 +381,7 @@ cb_dialog_apply_clicked (G_GNUC_UNUSED GtkWidget *button,
 	target = gnm_expr_entry_parse_as_value (state->change_cell_entry,
 						state->sheet);
 	if (target == NULL) {
-		gnumeric_notice_nonmodal (GTK_WINDOW(state->dialog),
+		go_gtk_notice_nonmodal_dialog (GTK_WINDOW(state->dialog),
 					  &(state->warning_dialog),
 					  GTK_MESSAGE_ERROR,
 					  _("You should introduce a valid cell "
@@ -394,7 +394,7 @@ cb_dialog_apply_clicked (G_GNUC_UNUSED GtkWidget *button,
 	state->change_cell = sheet_cell_fetch (r->a.sheet, r->a.col, r->a.row);
 	value_release (target);
 	if (cell_has_expr (state->change_cell)) {
-		gnumeric_notice_nonmodal (GTK_WINDOW(state->dialog),
+		go_gtk_notice_nonmodal_dialog (GTK_WINDOW(state->dialog),
 					  &(state->warning_dialog),
 					  GTK_MESSAGE_ERROR,
 					  _("The cell named in 'By changing cell' "
@@ -404,10 +404,10 @@ cb_dialog_apply_clicked (G_GNUC_UNUSED GtkWidget *button,
 	}
 
 
-	format = mstyle_get_format (cell_get_mstyle (state->set_cell));
+	format = gnm_style_get_format (cell_get_mstyle (state->set_cell));
 	if (entry_to_float_with_format (GTK_ENTRY(state->to_value_entry),
 					&state->target_value, TRUE, format)){
-		gnumeric_notice_nonmodal (GTK_WINDOW(state->dialog),
+		go_gtk_notice_nonmodal_dialog (GTK_WINDOW(state->dialog),
 					  &(state->warning_dialog),
 					  GTK_MESSAGE_ERROR,
 					  _("The value given in 'To Value:' "
@@ -416,7 +416,7 @@ cb_dialog_apply_clicked (G_GNUC_UNUSED GtkWidget *button,
 		return;
 	}
 
-	format = mstyle_get_format (cell_get_mstyle (state->change_cell));
+	format = gnm_style_get_format (cell_get_mstyle (state->change_cell));
 	if (entry_to_float_with_format (GTK_ENTRY(state->at_least_entry),
 					 &state->xmin, TRUE, format)) {
 		state->xmin = -max_range_val;
@@ -444,7 +444,7 @@ cb_dialog_apply_clicked (G_GNUC_UNUSED GtkWidget *button,
 	case GOAL_SEEK_OK: {
 		const char *actual_str;
 		const char *solution_str;
-		GnmFormat *format = style_format_general ();
+		GOFormat *format = go_format_general ();
 		GnmValue *error_value = value_new_float (state->target_value -
 						      value_get_as_float (state->set_cell->value));
   		char *target_str = format_value (format, error_value, NULL, 0,
@@ -637,7 +637,7 @@ dialog_goal_seek (WorkbookControlGUI *wbcg, Sheet *sheet)
 	/* Only pop up one copy per workbook */
 	if (gnumeric_dialog_raise_if_exists (wbcg, GOALSEEK_KEY))
 		return;
-	gui = gnm_glade_xml_new (GNM_CMD_CONTEXT (wbcg),
+	gui = gnm_glade_xml_new (GO_CMD_CONTEXT (wbcg),
 		"goalseek.glade", NULL, NULL);
         if (gui == NULL)
                 return;
@@ -651,7 +651,7 @@ dialog_goal_seek (WorkbookControlGUI *wbcg, Sheet *sheet)
 	state->cancelled = TRUE;
 
 	if (dialog_init (state)) {
-		gnumeric_notice (wbcg_toplevel (wbcg), GTK_MESSAGE_ERROR,
+		go_gtk_notice_dialog (wbcg_toplevel (wbcg), GTK_MESSAGE_ERROR,
 				 _("Could not create the Goal-Seek dialog."));
 		g_free (state);
 		return;
