@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  */
 
@@ -37,7 +37,7 @@
 #include <sheet-style.h>
 #include <value.h>
 #include <workbook-edit.h>
-#include <glib/gi18n.h>
+#include <goffice/utils/go-file.h>
 
 #include <gtk/gtkbox.h>
 #include <gtk/gtkimage.h>
@@ -49,7 +49,8 @@
 #include <gtk/gtkcelllayout.h>
 #include <gtk/gtkcellrenderertext.h>
 #include <gtk/gtkcellrendererpixbuf.h>
-#include <goffice/utils/go-file.h>
+#include <glib/gi18n.h>
+
 #include <string.h>
 
 typedef struct {
@@ -138,7 +139,7 @@ dhl_get_target_cur_wb (HyperlinkState *state, gboolean *success)
 			ret = (char *) target;
 			value_release (val);
 		} else {
-			gnumeric_notice (GTK_WINDOW (state->dialog), 
+			go_gtk_notice_dialog (GTK_WINDOW (state->dialog), 
 					 GTK_MESSAGE_ERROR,
 					 _("Not a range or name"));
 			gnm_expr_entry_grab_focus (gee, TRUE);
@@ -332,11 +333,10 @@ dhl_cb_ok (G_GNUC_UNUSED GtkWidget *button, HyperlinkState *state)
 	if (target) {
 		gnm_hlink_set_target (state->link, target);
 		gnm_hlink_set_tip (state->link, dhl_get_tip (state));
-		style = mstyle_new ();
-		mstyle_set_hlink (style, g_object_ref (state->link));
-		mstyle_set_font_uline (style, UNDERLINE_SINGLE);
-		mstyle_set_color (style, MSTYLE_COLOR_FORE,
-			style_color_new_name ("blue"));
+		style = gnm_style_new ();
+		gnm_style_set_hlink (style, g_object_ref (state->link));
+		gnm_style_set_font_uline (style, UNDERLINE_SINGLE);
+		gnm_style_set_font_color (style, style_color_new_name ("blue"));
 
 		if (state->is_new)
 			cmdname = _("Add Hyperlink");
@@ -345,8 +345,8 @@ dhl_cb_ok (G_GNUC_UNUSED GtkWidget *button, HyperlinkState *state)
 		cmd_selection_format (WORKBOOK_CONTROL (state->wbcg), style,
 				      NULL, cmdname);
 	} else if (!state->is_new) {
-		style = mstyle_new ();
-		mstyle_set_hlink (style, NULL);
+		style = gnm_style_new ();
+		gnm_style_set_hlink (style, NULL);
 		cmdname = _("Remove Hyperlink");
 		cmd_selection_format (WORKBOOK_CONTROL (state->wbcg), style,
 				      NULL, cmdname);
@@ -511,7 +511,7 @@ dialog_hyperlink (WorkbookControlGUI *wbcg, SheetControl *sc)
 	if (gnumeric_dialog_raise_if_exists (wbcg, DIALOG_KEY))
 		return;
 
-	gui = gnm_glade_xml_new (GNM_CMD_CONTEXT (wbcg),
+	gui = gnm_glade_xml_new (GO_CMD_CONTEXT (wbcg),
 		"hyperlink.glade", NULL, NULL);
         if (gui == NULL)
                 return;
@@ -538,7 +538,7 @@ dialog_hyperlink (WorkbookControlGUI *wbcg, SheetControl *sc)
 
 	state->link = link;
 	if (dhl_init (state)) {
-		gnumeric_notice (wbcg_toplevel (wbcg), GTK_MESSAGE_ERROR,
+		go_gtk_notice_dialog (wbcg_toplevel (wbcg), GTK_MESSAGE_ERROR,
 				 _("Could not create the hyperlink dialog."));
 		g_free (state);
 		return;
@@ -554,7 +554,7 @@ dialog_hyperlink (WorkbookControlGUI *wbcg, SheetControl *sc)
 			       DIALOG_KEY);
 	g_object_set_data_full (G_OBJECT (state->dialog),
 		"state", state, (GDestroyNotify) dhl_free);
-	gnumeric_non_modal_dialog (wbcg_toplevel (state->wbcg),
+	go_gtk_nonmodal_dialog (wbcg_toplevel (state->wbcg),
 				   GTK_WINDOW (state->dialog));
 	wbcg_edit_attach_guru (state->wbcg, state->dialog);
 	gtk_widget_show (state->dialog);
