@@ -31,7 +31,7 @@
 
 #include "mstyle.h"
 #include "value.h"
-#include "format.h"
+#include <gnm-format.h>
 #include "workbook.h"
 #include "number-match.h"
 #include "widgets/widget-font-selector.h"
@@ -74,6 +74,7 @@ typedef struct {
 	GtkTreeStore    *store;
 	GtkTreeView     *view;
 	Workbook	*wb;
+	GOConfNode	*root;
 } PrefState;
 
 typedef void (* double_conf_setter_t) (gnm_float value);
@@ -114,18 +115,6 @@ dialog_pref_page_open (PrefState *state)
 	gtk_text_buffer_delete (buffer, &start, &end);
 }
 
-static void
-dialog_pref_load_description_from_key (PrefState *state, char const *key)
-{
-	char *long_desc = go_conf_get_long_desc (key);
-
-	g_return_if_fail (long_desc != NULL);
-
-	gtk_text_buffer_set_text (
-		gtk_text_view_get_buffer (state->description), long_desc, -1);
-	g_free (long_desc);
-}
-
 static void
 dialog_pref_load_description (PrefState *state, char const *text)
 {
@@ -135,9 +124,22 @@ dialog_pref_load_description (PrefState *state, char const *text)
 }
 
 static void
-set_tip (char const *key, GtkWidget *item)
+dialog_pref_load_description_from_key (PrefState *state, char const *key)
 {
-	char *desc = go_conf_get_long_desc (key);
+	char *long_desc = go_conf_get_long_desc (state->root, key);
+
+	if (long_desc == NULL)
+		dialog_pref_load_description (state, "");
+	else {
+		dialog_pref_load_description (state, long_desc);
+		g_free (long_desc);
+	}
+}
+
+static void
+set_tip (GOConfNode *node, char const *key, GtkWidget *item)
+{
+	char *desc = go_conf_get_long_desc (node, key);
 	if (desc != NULL) {
 		GtkTooltips *the_tip = gtk_tooltips_new ();
 		gtk_tooltips_set_tip (the_tip, item, desc, NULL);
@@ -152,10 +154,10 @@ cb_pref_notification_destroy (gpointer handle)
 }
 
 static void
-connect_notification (char const *key, GOConfMonitorFunc func,
+connect_notification (GOConfNode *node, char const *key, GOConfMonitorFunc func,
 		      gpointer data, GtkWidget *container)
 {
-	guint handle = go_conf_add_monitor (key, func, data);
+	guint handle = go_conf_add_monitor (node, key, func, data);
 	g_signal_connect_swapped (G_OBJECT (container), "destroy",
 		G_CALLBACK (cb_pref_notification_destroy),
 		GUINT_TO_POINTER (handle));
@@ -171,23 +173,26 @@ bool_pref_widget_to_conf (GtkToggleButton *button,
 	setter (gtk_toggle_button_get_active (button));
 }
 static void
-bool_pref_conf_to_widget (char const *key, GtkToggleButton *button)
+bool_pref_conf_to_widget (GOConfNode *node, char const *key, GtkToggleButton *button)
 {
 	gboolean val_in_button = gtk_toggle_button_get_active (button);
-	gboolean val_in_conf = go_conf_get_bool (key);
+	gboolean val_in_conf = go_conf_get_bool (node, key);
 	if ((val_in_button != FALSE) != (val_in_conf != FALSE))
 		gtk_toggle_button_set_active (button, val_in_conf);
 }
 static void
-bool_pref_create_widget (char const *key, GtkWidget *table, gint row, 
-			 gboolean_conf_setter_t setter)
+bool_pref_create_widget (GOConfNode *node, char const *key, GtkWidget *table,
+			 gint row, gboolean_conf_setter_t setter, 
+			 char const *default_text)
 {
-	char *desc = go_conf_get_short_desc (key);
+	char *desc = go_conf_get_short_desc (node, key);
 	GtkWidget *item = gtk_check_button_new_with_label (
-		desc ? desc : "schema missing");
-	g_free (desc);
+		(desc != NULL) ? desc : default_text);
 
-	bool_pref_conf_to_widget (key, GTK_TOGGLE_BUTTON (item));
+	if (desc != NULL)
+		g_free (desc);
+
+	bool_pref_conf_to_widget (node, key, GTK_TOGGLE_BUTTON (item));
 	g_signal_connect (G_OBJECT (item),
 		"toggled",
 		G_CALLBACK (bool_pref_widget_to_conf), (gpointer) setter);
@@ -195,9 +200,9 @@ bool_pref_create_widget (char const *key, GtkWidget *table, gint row,
 		0, 2, row, row + 1,
 		GTK_FILL | GTK_SHRINK, GTK_FILL | GTK_SHRINK, 5, 5);
 
-	connect_notification (key, (GOConfMonitorFunc)bool_pref_conf_to_widget,
+	connect_notification (node, key, (GOConfMonitorFunc)bool_pref_conf_to_widget,
 			      item, table);
-	set_tip (key, item);
+	set_tip (node, key, item);
 }
 
 /*************************************************************************/
@@ -211,20 +216,24 @@ int_pref_widget_to_conf (GtkSpinButton *button, gint_conf_setter_t setter)
 }
 
 static void
-int_pref_conf_to_widget (char const *key, GtkSpinButton *button)
+int_pref_conf_to_widget (GOConfNode *node, char const *key, GtkSpinButton *button)
 {
 	gint val_in_button = gtk_spin_button_get_value_as_int (button);
-	gint val_in_conf = go_conf_get_int (key);
+	gint val_in_conf = go_conf_get_int (node, key);
 	if (val_in_conf != val_in_button)
 		gtk_spin_button_set_value (button, (gdouble) val_in_conf);
 }
 static void
-int_pref_create_widget (char const *key, GtkWidget *table, gint row,
-			gint val, gint from, gint to, gint step, gint_conf_setter_t setter)
+int_pref_create_widget (GOConfNode *node, char const *key, GtkWidget *table,
+			gint row, gint val, gint from, gint to, gint step, 
+			gint_conf_setter_t setter, char const *default_text)
 {
-	char *desc = go_conf_get_short_desc (key);
-	GtkWidget *item = gtk_label_new (desc ? desc : "schema missing");
-	g_free (desc);
+	char *desc = go_conf_get_short_desc (node, key);
+	GtkWidget *item = gtk_label_new 
+		((desc != NULL) ? desc : default_text);
+
+	if (desc != NULL)
+		g_free (desc);
 
 	gtk_label_set_justify (GTK_LABEL (item), GTK_JUSTIFY_LEFT);
 	gtk_misc_set_alignment (GTK_MISC (item), 0, 0);
@@ -235,7 +244,7 @@ int_pref_create_widget (char const *key, GtkWidget *table, gint row,
 				    gtk_adjustment_new (val, from, to, step, 
 							step, step)),
 				    1, 0);
-	int_pref_conf_to_widget (key, GTK_SPIN_BUTTON (item));
+	int_pref_conf_to_widget (node, key, GTK_SPIN_BUTTON (item));
 	g_signal_connect (G_OBJECT (item),
 			  "value-changed",
 			  G_CALLBACK (int_pref_widget_to_conf), (gpointer) setter);
@@ -243,9 +252,9 @@ int_pref_create_widget (char const *key, GtkWidget *table, gint row,
 		1, 2, row, row + 1,
 		GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_SHRINK, 5, 2);
 
-	connect_notification (key, (GOConfMonitorFunc)int_pref_conf_to_widget,
+	connect_notification (node, key, (GOConfMonitorFunc)int_pref_conf_to_widget,
 			      item, table);
-	set_tip (key, item);
+	set_tip (node, key, item);
 }
 
 /*************************************************************************/
@@ -259,21 +268,26 @@ double_pref_widget_to_conf (GtkSpinButton *button, double_conf_setter_t setter)
 }
 
 static void
-double_pref_conf_to_widget (char const *key, GtkSpinButton *button)
+double_pref_conf_to_widget (GOConfNode *node, char const *key, GtkSpinButton *button)
 {
 	double val_in_button = gtk_spin_button_get_value (button);
-	double val_in_conf = go_conf_get_double (key);
-	if (abs (val_in_conf - val_in_button) > 1e-10) /* dead simple */
+	double val_in_conf = go_conf_get_double (node, key);
+
+	if (fabs (val_in_conf - val_in_button) > 1e-10) /* dead simple */
 		gtk_spin_button_set_value (button, val_in_conf);
 }
 static void
-double_pref_create_widget (char const *key, GtkWidget *table, gint row,
-			   gnm_float val, gnm_float from, gnm_float to, gnm_float step,
-			   gint digits, double_conf_setter_t setter)
+double_pref_create_widget (GOConfNode *node, char const *key, GtkWidget *table,
+			   gint row, gnm_float val, gnm_float from,gnm_float to, 
+			   gnm_float step,
+			   gint digits, double_conf_setter_t setter,
+			   char const *default_text)
 {
-	char *desc = go_conf_get_short_desc (key);
-	GtkWidget *item = gtk_label_new (desc ? desc : "schema missing");
-	g_free (desc);
+	char *desc = go_conf_get_short_desc (node, key);
+	GtkWidget *item = gtk_label_new (desc ? desc : default_text);
+	
+	if (desc != NULL)
+		g_free (desc);
 
 	gtk_label_set_justify (GTK_LABEL (item), GTK_JUSTIFY_LEFT);
 	gtk_misc_set_alignment (GTK_MISC (item), 0, 0);
@@ -283,7 +297,7 @@ double_pref_create_widget (char const *key, GtkWidget *table, gint row,
 	item =  gtk_spin_button_new (GTK_ADJUSTMENT (
 				     gtk_adjustment_new (val, from, to, step, step, step)),
 				     1, digits);
-	double_pref_conf_to_widget (key, GTK_SPIN_BUTTON (item));
+	double_pref_conf_to_widget (node, key, GTK_SPIN_BUTTON (item));
 	g_signal_connect (G_OBJECT (item),
 			  "value-changed",
 			  G_CALLBACK (double_pref_widget_to_conf), 
@@ -292,9 +306,9 @@ double_pref_create_widget (char const *key, GtkWidget *table, gint row,
 		1, 2, row, row + 1,
 		GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_SHRINK, 5, 2);
 
-	connect_notification (key, (GOConfMonitorFunc)double_pref_conf_to_widget,
+	connect_notification (node, key, (GOConfMonitorFunc)double_pref_conf_to_widget,
 			      item, table);
-	set_tip (key, item);
+	set_tip (node, key, item);
 }
 
 
@@ -311,17 +325,17 @@ pref_font_page_open (PrefState *state, G_GNUC_UNUSED gpointer data,
 }
 
 static void
-cb_pref_font_set_fonts (char const *key, GtkWidget *page)
+cb_pref_font_set_fonts (GOConfNode *node, char const *key, GtkWidget *page)
 {
-	if (key == NULL || 0 == strcmp (key, GNM_CONF_FONT_NAME))
+	if (!key || g_str_has_suffix (key, GNM_CONF_FONT_NAME))
 		font_selector_set_name (FONT_SELECTOR (page),
 			gnm_app_prefs->default_font.name);
-	if (key == NULL || 0 == strcmp (key, GNM_CONF_FONT_SIZE))
+	if (!key || g_str_has_suffix (key, GNM_CONF_FONT_SIZE))
 		font_selector_set_points (FONT_SELECTOR (page),
 			gnm_app_prefs->default_font.size);
-	if (key == NULL ||
-	    0 == strcmp (key, GNM_CONF_FONT_BOLD) ||
-	    0 == strcmp (key, GNM_CONF_FONT_ITALIC))
+	if (!key ||
+	    g_str_has_suffix (key, GNM_CONF_FONT_BOLD) ||
+	    g_str_has_suffix (key, GNM_CONF_FONT_ITALIC))
 		font_selector_set_style (FONT_SELECTOR (page),
 			gnm_app_prefs->default_font.is_bold,
 			gnm_app_prefs->default_font.is_italic);
@@ -331,18 +345,18 @@ static gboolean
 cb_pref_font_has_changed (G_GNUC_UNUSED FontSelector *fs,
 			  GnmStyle *mstyle, PrefState *state)
 {
-	if (mstyle_is_element_set (mstyle, MSTYLE_FONT_SIZE))
+	if (gnm_style_is_element_set (mstyle, MSTYLE_FONT_SIZE))
 		gnm_gconf_set_default_font_size 
-			(mstyle_get_font_size (mstyle));
-	if (mstyle_is_element_set (mstyle, MSTYLE_FONT_NAME))
+			(gnm_style_get_font_size (mstyle));
+	if (gnm_style_is_element_set (mstyle, MSTYLE_FONT_NAME))
 		gnm_gconf_set_default_font_name (
-			mstyle_get_font_name (mstyle));
-	if (mstyle_is_element_set (mstyle, MSTYLE_FONT_BOLD))
+			gnm_style_get_font_name (mstyle));
+	if (gnm_style_is_element_set (mstyle, MSTYLE_FONT_BOLD))
 		gnm_gconf_set_default_font_bold (
-			mstyle_get_font_bold (mstyle));
-	if (mstyle_is_element_set (mstyle, MSTYLE_FONT_ITALIC))
+			gnm_style_get_font_bold (mstyle));
+	if (gnm_style_is_element_set (mstyle, MSTYLE_FONT_ITALIC))
 		gnm_gconf_set_default_font_italic (
-			mstyle_get_font_italic (mstyle));
+			gnm_style_get_font_italic (mstyle));
 	return TRUE;
 }
 
@@ -354,9 +368,9 @@ pref_font_initializer (PrefState *state,
 {
 	GtkWidget *page = font_selector_new ();
 
-	cb_pref_font_set_fonts (NULL, page);
+	cb_pref_font_set_fonts (NULL, NULL, page);
 
-	connect_notification (GNM_CONF_FONT_DIRECTORY,
+	connect_notification (state->root, GNM_CONF_FONT_DIR,
 		(GOConfMonitorFunc) cb_pref_font_set_fonts,
 		page, page);
 	g_signal_connect (G_OBJECT (page),
@@ -381,23 +395,29 @@ pref_font_hf_page_open (PrefState *state, G_GNUC_UNUSED gpointer data,
 }
 
 static void
-cb_pref_font_hf_set_fonts (char const *key, GtkWidget *page)
+cb_pref_font_hf_set_fonts (GOConfNode *node, char const *key, GtkWidget *page)
 {
-	if (key == NULL || 0 == strcmp (key, PRINTSETUP_GCONF_HF_FONT_NAME)) {
-		char *name = go_conf_load_string (PRINTSETUP_GCONF_HF_FONT_NAME);
+	node = gnm_conf_get_root ();
+	if (!key ||
+	    g_str_has_suffix (key, PRINTSETUP_GCONF_HF_FONT_NAME)) {
+		gchar *name = go_conf_load_string (
+			node, PRINTSETUP_GCONF_DIR "/" PRINTSETUP_GCONF_HF_FONT_NAME);
 		font_selector_set_name (FONT_SELECTOR (page), name);
 		g_free (name);
 	}
-	if (key == NULL || 0 == strcmp (key, PRINTSETUP_GCONF_HF_FONT_SIZE))
+	if (!key ||
+	    g_str_has_suffix (key, PRINTSETUP_GCONF_HF_FONT_SIZE))
 		font_selector_set_points (FONT_SELECTOR (page),
-			go_conf_get_double (PRINTSETUP_GCONF_HF_FONT_SIZE));
-
-	if (key == NULL ||
-	    0 == strcmp (key, PRINTSETUP_GCONF_HF_FONT_BOLD) ||
-	    0 == strcmp (key, PRINTSETUP_GCONF_HF_FONT_ITALIC))
+			go_conf_get_double (
+				node, PRINTSETUP_GCONF_DIR "/" PRINTSETUP_GCONF_HF_FONT_SIZE));
+	if (!key ||
+	    g_str_has_suffix (key, PRINTSETUP_GCONF_HF_FONT_BOLD) ||
+	    g_str_has_suffix (key, PRINTSETUP_GCONF_HF_FONT_ITALIC))
 		font_selector_set_style (FONT_SELECTOR (page),
-			go_conf_get_bool (PRINTSETUP_GCONF_HF_FONT_BOLD),
-			go_conf_get_bool (PRINTSETUP_GCONF_HF_FONT_ITALIC));
+			go_conf_get_bool (
+				node, PRINTSETUP_GCONF_DIR "/" PRINTSETUP_GCONF_HF_FONT_BOLD),
+			go_conf_get_bool (
+				node, PRINTSETUP_GCONF_DIR "/" PRINTSETUP_GCONF_HF_FONT_ITALIC));
 }
 
 static gboolean
@@ -416,8 +436,8 @@ pref_font_hf_initializer (PrefState *state,
 {
 	GtkWidget *page = font_selector_new ();
 
-	cb_pref_font_hf_set_fonts (NULL, page);
-	connect_notification (PRINTSETUP_GCONF_DIRECTORY,
+	cb_pref_font_hf_set_fonts (state->root, NULL, page);
+	connect_notification (state->root, PRINTSETUP_GCONF_DIR,
 		(GOConfMonitorFunc) cb_pref_font_hf_set_fonts,
 		page, page);
 	g_signal_connect (G_OBJECT (page),
@@ -450,15 +470,26 @@ pref_undo_page_initializer (PrefState *state,
 {
 	GtkWidget *page = gtk_table_new (4, 2, FALSE);
 	gint row = 0;
+	GOConfNode *node;
 
-	int_pref_create_widget (GNM_CONF_UNDO_MAX_DESCRIPTOR_WIDTH,
-		page, row++, 5, 5, 200, 1, gnm_gconf_set_max_descriptor_width);
-	int_pref_create_widget (GNM_CONF_UNDO_SIZE,
-		page, row++, 1000, 0, 30000, 100, gnm_gconf_set_undo_size);
-	int_pref_create_widget (GNM_CONF_UNDO_MAXNUM,
-		page, row++, 20, 1, 200, 1, gnm_gconf_set_undo_max_number);
-	bool_pref_create_widget (GNM_CONF_UNDO_SHOW_SHEET_NAME,
-		page, row++, gnm_gconf_set_show_sheet_name);
+	node = go_conf_get_node (state->root, GNM_CONF_UNDO_DIR);
+	int_pref_create_widget (node, GNM_CONF_UNDO_MAX_DESCRIPTOR_WIDTH,
+				page, row++, 5, 5, 200, 1, 
+				gnm_gconf_set_max_descriptor_width,
+				_("Length of Undo Descriptors"));
+	int_pref_create_widget (node, GNM_CONF_UNDO_SIZE,
+				page, row++, 1000, 0, 30000, 100, 
+				gnm_gconf_set_undo_size,
+				_("Maximal Undo Size"));
+	int_pref_create_widget (node, GNM_CONF_UNDO_MAXNUM,
+				page, row++, 20, 1, 200, 1, 
+				gnm_gconf_set_undo_max_number,
+				_("Number of Undo Items"));
+	bool_pref_create_widget (node, GNM_CONF_UNDO_SHOW_SHEET_NAME,
+				 page, row++, 
+				 gnm_gconf_set_show_sheet_name,
+				_("Show Sheet Name in Undo List"));
+	go_conf_free_node (node);
 
 	gtk_widget_show_all (page);
 	return page;
@@ -487,15 +518,26 @@ pref_sort_page_initializer (PrefState *state,
 {
 	GtkWidget *page = gtk_table_new (3, 2, FALSE);
 	gint row = 0;
+	GOConfNode *node;
 
-	int_pref_create_widget (GNM_CONF_SORT_DIALOG_MAX_INITIAL,
-		page, row++, 10, 0, 50, 1, gnm_gconf_set_sort_dialog_max_initial);
-	bool_pref_create_widget (GNM_CONF_SORT_DEFAULT_RETAIN_FORM,
-		page, row++, gnm_gconf_set_sort_retain_form);
-	bool_pref_create_widget (GNM_CONF_SORT_DEFAULT_BY_CASE,
-		page, row++, gnm_gconf_set_sort_by_case);
-	bool_pref_create_widget (GNM_CONF_SORT_DEFAULT_ASCENDING,
-		page, row++, gnm_gconf_set_sort_ascending);
+	node = go_conf_get_node (state->root, GNM_CONF_SORT_DIR);
+	int_pref_create_widget (node, GNM_CONF_SORT_DIALOG_MAX_INITIAL,
+				page, row++, 10, 0, 50, 1, 
+				gnm_gconf_set_sort_dialog_max_initial,
+				_("Number of Automatic Clauses"));
+	bool_pref_create_widget (node, GNM_CONF_SORT_DEFAULT_RETAIN_FORM,
+				 page, row++, 
+				 gnm_gconf_set_sort_retain_form,
+				 _("Sorting Preserves Formats"));
+	bool_pref_create_widget (node, GNM_CONF_SORT_DEFAULT_BY_CASE,
+				 page, row++, 
+				 gnm_gconf_set_sort_by_case,
+				 _("Sorting is Case-Sensitive"));
+	bool_pref_create_widget (node, GNM_CONF_SORT_DEFAULT_ASCENDING,
+				 page, row++, 
+				 gnm_gconf_set_sort_ascending,
+				 _("Sort Ascending"));
+	go_conf_free_node (node);
 
 	gtk_widget_show_all (page);
 	return page;
@@ -524,19 +566,34 @@ pref_window_page_initializer (PrefState *state,
 {
 	GtkWidget *page = gtk_table_new (4, 2, FALSE);
 	gint row = 0;
-
-	double_pref_create_widget (GNM_CONF_GUI_WINDOW_Y,
-		page, row++, 0.75, 0.25, 1, 0.05, 2, gnm_gconf_set_gui_window_y);
-	double_pref_create_widget (GNM_CONF_GUI_WINDOW_X,
-		page, row++, 0.75, 0.25, 1, 0.05, 2, gnm_gconf_set_gui_window_x);
-	double_pref_create_widget (GNM_CONF_GUI_ZOOM,
-		page, row++, 1.00, 0.10, 5.00, 0.05, 2, gnm_gconf_set_gui_zoom);
-	int_pref_create_widget (GNM_CONF_WORKBOOK_NSHEETS,
-		page, row++, 1, 1, 64, 1, gnm_gconf_set_workbook_nsheets);
-	bool_pref_create_widget (GNM_CONF_GUI_ED_TRANSITION_KEYS,
-		page, row++, gnm_gconf_set_gui_transition_keys);
-	bool_pref_create_widget (GNM_CONF_GUI_ED_LIVESCROLLING,
-		page, row++, gnm_gconf_set_gui_livescrolling);
+	GOConfNode *node;
+	
+	node = go_conf_get_node (state->root, GNM_CONF_GUI_DIR);
+	double_pref_create_widget (node, GNM_CONF_GUI_WINDOW_Y,
+				   page, row++, 0.75, 0.25, 1, 0.05, 2, 
+				   gnm_gconf_set_gui_window_y,
+				   _("Default Vertical Window Size"));
+	double_pref_create_widget (node, GNM_CONF_GUI_WINDOW_X,
+				   page, row++, 0.75, 0.25, 1, 0.05, 2, 
+				   gnm_gconf_set_gui_window_x,
+				   _("Default Horizontal Window Size"));
+	double_pref_create_widget (node, GNM_CONF_GUI_ZOOM,
+				   page, row++, 1.00, 0.10, 5.00, 0.05, 2, 
+				   gnm_gconf_set_gui_zoom,
+				   _("Default Zoom Factor"));
+	int_pref_create_widget (state->root, GNM_CONF_WORKBOOK_NSHEETS,
+				page, row++, 1, 1, 64, 1, 
+				gnm_gconf_set_workbook_nsheets,
+				_("Default Number of Sheets"));
+	bool_pref_create_widget (node, GNM_CONF_GUI_ED_TRANSITION_KEYS,
+				 page, row++, 
+				 gnm_gconf_set_gui_transition_keys,
+				 _("Transition Keys"));
+	bool_pref_create_widget (node, GNM_CONF_GUI_ED_LIVESCROLLING,
+				 page, row++, 
+				 gnm_gconf_set_gui_livescrolling,
+				 _("Live Scrolling"));
+	go_conf_free_node (node);
 
 	gtk_widget_show_all (page);
 	return page;
@@ -564,17 +621,33 @@ pref_file_page_initializer (PrefState *state,
 {
 	GtkWidget *page = gtk_table_new (2, 2, FALSE);
 	gint row = 0;
+	GOConfNode *node;
 
-	int_pref_create_widget (GNM_CONF_FILE_HISTORY_N,
-		page, row++, 4, 0, 40, 1, gnm_gconf_set_file_history_number);
-	int_pref_create_widget (GNM_CONF_XML_COMPRESSION,
-		page, row++, 9, 0, 9, 1, gnm_gconf_set_xml_compression);
-	bool_pref_create_widget (GNM_CONF_FILE_OVERWRITE_DEFAULT,
-		page, row++, gnm_gconf_set_file_overwrite);
-	bool_pref_create_widget (GNM_CONF_FILE_SINGLE_SHEET_SAVE,
-		page, row++, gnm_gconf_set_file_single_sheet_save);
-	bool_pref_create_widget (PLUGIN_GCONF_LATEX_USE_UTF8,
-		page, row++, gnm_gconf_set_latex_use_utf8);
+	node = go_conf_get_node (state->root, GNM_CONF_FILE_DIR);
+	int_pref_create_widget (node, GNM_CONF_FILE_HISTORY_N,
+				page, row++, 4, 0, 40, 1, 
+				gnm_gconf_set_file_history_number,
+				_("Length of File History"));
+	int_pref_create_widget (state->root, GNM_CONF_XML_COMPRESSION,
+				page, row++, 9, 0, 9, 1, 
+				gnm_gconf_set_xml_compression,
+				_("Default Compression Level For "
+				  "Gnumeric Files"));
+	bool_pref_create_widget (node, GNM_CONF_FILE_OVERWRITE_DEFAULT,
+				 page, row++, 
+				 gnm_gconf_set_file_overwrite,
+				 _("Default To Overwriting Files"));
+	bool_pref_create_widget (node, GNM_CONF_FILE_SINGLE_SHEET_SAVE,
+				 page, row++, 
+				 gnm_gconf_set_file_single_sheet_save,
+				 _("Warn When Exporting Into Single "
+				   "Sheet Format"));
+	bool_pref_create_widget (state->root,
+				 PLUGIN_GCONF_LATEX "/" PLUGIN_GCONF_LATEX_USE_UTF8,
+				 page, row++, 
+				 gnm_gconf_set_latex_use_utf8,
+				 _("Use UTF-8 in LaTeX Export"));
+	go_conf_free_node (node);
 
 	gtk_widget_show_all (page);
 	return page;
@@ -602,11 +675,18 @@ pref_screen_page_initializer (PrefState *state,
 {
 	GtkWidget *page = gtk_table_new (2, 2, FALSE);
 	gint row = 0;
-
-	double_pref_create_widget (GNM_CONF_GUI_RES_H, page, row++,
-				   96, 50, 250, 1, 1, gnm_gconf_set_gui_resolution_h);
-	double_pref_create_widget (GNM_CONF_GUI_RES_V, page, row++,
-				   96, 50, 250, 1, 1, gnm_gconf_set_gui_resolution_v);
+	GOConfNode *node;
+	
+	node = go_conf_get_node (state->root, GNM_CONF_GUI_DIR);
+	double_pref_create_widget (node, GNM_CONF_GUI_RES_H, page, row++,
+				   96, 50, 250, 1, 1, 
+				   gnm_gconf_set_gui_resolution_h,
+				   _("Horizontal DPI"));
+	double_pref_create_widget (node, GNM_CONF_GUI_RES_V, page, row++,
+				   96, 50, 250, 1, 1, 
+				   gnm_gconf_set_gui_resolution_v,
+				   _("Vertical DPI"));
+	go_conf_free_node (node);
 
 	gtk_widget_show_all (page);
 	return page;
@@ -636,12 +716,22 @@ pref_tool_page_initializer (PrefState *state,
 	GtkWidget *page = gtk_table_new (2, 2, FALSE);
 	gint row = 0;
 
-	int_pref_create_widget (FUNCTION_SELECT_GCONF_NUM_OF_RECENT,
-		page, row++, 10, 0, 40, 1, gnm_gconf_set_num_recent_functions);
-	bool_pref_create_widget (GNM_CONF_GUI_ED_AUTOCOMPLETE,
-		page, row++, gnm_gconf_set_autocomplete);
-	bool_pref_create_widget (DIALOGS_GCONF_UNFOCUSED_RS,
-		page, row++, gnm_gconf_set_unfocused_rs);
+	int_pref_create_widget (state->root,
+				FUNCTION_SELECT_GCONF_DIR "/" FUNCTION_SELECT_GCONF_NUM_OF_RECENT,
+				page, row++, 10, 0, 40, 1, 
+				gnm_gconf_set_num_recent_functions,
+				_("Maximum Length of Recently "
+				  "Used Functions List"));
+	bool_pref_create_widget (state->root,
+				 GNM_CONF_GUI_DIR "/" GNM_CONF_GUI_ED_AUTOCOMPLETE,
+				 page, row++, 
+				 gnm_gconf_set_autocomplete,
+				_("Autocomplete"));
+	bool_pref_create_widget (state->root,
+				 DIALOGS_GCONF_DIR "/" DIALOGS_GCONF_UNFOCUSED_RS,
+				 page, row++, 
+				 gnm_gconf_set_unfocused_rs,
+				_("Allow Unfocused Range Selections"));
 	
 	gtk_widget_show_all (page);
 	return page;
@@ -657,9 +747,9 @@ pref_copypaste_page_open (PrefState *state, G_GNUC_UNUSED gpointer data,
 		     G_GNUC_UNUSED gint page_num)
 {
 	dialog_pref_load_description (state,
-				      _("The items on this page are"
+				      _("The items on this page are "
                                         "related to "
-					"copy. cut and paste."));
+					"copy, cut and paste."));
 }
 
 static GtkWidget *
@@ -671,8 +761,11 @@ pref_copypaste_page_initializer (PrefState *state,
 	GtkWidget *page = gtk_table_new (2, 2, FALSE);
 	gint row = 0;
 
-	bool_pref_create_widget (GNM_CONF_CUTANDPASTE_PREFER_CLIPBOARD,
-		page, row++, gnm_gconf_set_prefer_clipboard);
+	bool_pref_create_widget (state->root,
+				 GNM_CONF_CUTANDPASTE_DIR "/" GNM_CONF_CUTANDPASTE_PREFER_CLIPBOARD,
+				 page, row++, 
+				 gnm_gconf_set_prefer_clipboard,
+				 _("Prefer CLIPBOARD over PRIMARY selection"));
 	
 	gtk_widget_show_all (page);
 	return page;
@@ -693,7 +786,7 @@ typedef struct {
 	gpointer data;
 } page_info_t;
 
-static page_info_t page_info[] = {
+static const page_info_t page_info[] = {
 	{N_("Font"),          GTK_STOCK_ITALIC,	         NULL, &pref_font_initializer,		&pref_font_page_open,	NULL},
 	{N_("Copy and Paste"),GTK_STOCK_PASTE,		 NULL, &pref_copypaste_page_initializer,&pref_copypaste_page_open,	NULL},
 	{N_("Files"),         GTK_STOCK_FLOPPY,	         NULL, &pref_file_page_initializer,	&pref_file_page_open,	NULL},
@@ -745,7 +838,7 @@ cb_dialog_pref_selection_changed (GtkTreeSelection *selection,
 static gboolean
 cb_preferences_destroy (PrefState *state)
 {
-	go_conf_sync ();
+	go_conf_sync (state->root);
 
 	if (state->gui != NULL) {
 		g_object_unref (G_OBJECT (state->gui));
@@ -779,7 +872,7 @@ cb_dialog_pref_switch_page  (GtkNotebook *notebook,
 
 /* Note: The first page listed below is opened through File/Preferences, */
 /*       and the second through Format/Workbook */
-static char const *startup_pages[] = {"1", "0"};
+static char const * const startup_pages[] = {"1", "0"};
 
 void
 dialog_preferences (WorkbookControlGUI *wbcg, gint page)
@@ -798,12 +891,13 @@ dialog_preferences (WorkbookControlGUI *wbcg, gint page)
 		return;
 	}
 
-	gui = gnm_glade_xml_new (GNM_CMD_CONTEXT (wbcg),
+	gui = gnm_glade_xml_new (GO_CMD_CONTEXT (wbcg),
 		"preferences.glade", NULL, NULL);
 	if (gui == NULL)
 		return;
 
 	state = g_new0 (PrefState, 1);
+	state->root = gnm_conf_get_root ();
 	state->gui = gui;
 	state->dialog     = glade_xml_get_widget (gui, "preferences");
 	state->notebook   = glade_xml_get_widget (gui, "notebook");
@@ -854,7 +948,7 @@ dialog_preferences (WorkbookControlGUI *wbcg, gint page)
 	gnm_app_set_pref_dialog (state->dialog);
 
 	for (i = 0; page_info[i].page_initializer; i++) {
-		page_info_t *this_page =  &page_info[i];
+		const page_info_t *this_page =  &page_info[i];
 		GtkWidget *page = this_page->page_initializer (state, this_page->data,
 							       GTK_NOTEBOOK (state->notebook), i);
 		GtkWidget *label = NULL;
