@@ -28,7 +28,7 @@
 #include <workbook.h>
 #include <sheet.h>
 #include <gui-util.h>
-#include <widgets/widget-charmap-selector.h>
+#include <goffice/gtk/go-charmap-sel.h>
 
 #include <gtk/gtkmessagedialog.h>
 #include <gtk/gtkcombobox.h>
@@ -83,11 +83,11 @@ typedef struct {
 		GtkComboBoxEntry      	*quotechar;
 		GtkWidget	*charset;
 		GtkComboBox 	*transliterate;
-		GtkWidget	*preserve;
+		GtkComboBox 	*format;
 	} format;
 
 	TextExportPage	cur_page;
-	StfExportOptions_t *result;
+	GnmStfExport *result;
 } TextExportState;
 
 static void
@@ -119,7 +119,9 @@ stf_export_dialog_format_page_init (TextExportState *state)
 	gtk_combo_box_set_active (state->format.quote, 0);
 	state->format.quotechar   = GTK_COMBO_BOX_ENTRY      (glade_xml_get_widget (state->gui, "format_quotechar"));
 	gtk_combo_box_set_active (GTK_COMBO_BOX (state->format.quotechar), 0);
-	state->format.charset	  = charmap_selector_new (CHARMAP_SELECTOR_FROM_UTF8);
+	state->format.format     = GTK_COMBO_BOX (glade_xml_get_widget (state->gui, "format"));
+	gtk_combo_box_set_active (GTK_COMBO_BOX (state->format.format), 0);
+	state->format.charset	  = go_charmap_sel_new (GO_CHARMAP_SEL_FROM_UTF8);
 	state->format.transliterate = GTK_COMBO_BOX (glade_xml_get_widget (state->gui, "format_transliterate"));
 	gnumeric_editable_enters (state->window, state->format.custom);
 	gnumeric_editable_enters (state->window,
@@ -127,10 +129,10 @@ stf_export_dialog_format_page_init (TextExportState *state)
 
 	if (stf_export_can_transliterate ()) {
 		gtk_combo_box_set_active (state->format.transliterate,
-			TRANSLITERATE_MODE_TRANS);
+			GNM_STF_TRANSLITERATE_MODE_TRANS);
 	} else {
 		gtk_combo_box_set_active (state->format.transliterate,
-			TRANSLITERATE_MODE_ESCAPE);
+			GNM_STF_TRANSLITERATE_MODE_ESCAPE);
 		/* It might be better to render insensitive only one option than
 		the whole list as in the following line but it is not possible
 		with gtk-2.4. May be it should be changed when 2.6 is available	
@@ -138,8 +140,6 @@ stf_export_dialog_format_page_init (TextExportState *state)
 		gtk_widget_set_sensitive (GTK_WIDGET (state->format.transliterate), FALSE);
 	}
 
-	state->format.preserve = glade_xml_get_widget (state->gui, "format_preserve");
-
 	table = glade_xml_get_widget (state->gui, "format_table");
 	gtk_table_attach_defaults (GTK_TABLE (table), state->format.charset, 1, 2, 5, 6);
 	gtk_widget_show_all (table);
@@ -162,79 +162,96 @@ cb_collect_exported_sheets (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter
 		-1);
 	if (exported)
 		stf_export_options_sheet_list_add (state->result, sheet);
+	g_object_unref (sheet);
 	return FALSE;
 }
 static void
 stf_export_dialog_finish (TextExportState *state)
 {
-	StfTerminatorType_t	terminator;
-	StfQuotingMode_t	quotingmode;
-	StfTransliterateMode_t	transliteratemode;
-	char *text;
-	gunichar separator;
+	GsfOutputCsvQuotingMode	quotingmode;
+	GnmStfTransliterateMode transliteratemode;
+	GnmStfFormatMode format;
+	const char *eol;
+	GString *triggers = g_string_new (NULL);
+	char *separator, *quote;
+	const char *charset;
 
-	state->result = stf_export_options_new ();
+	/* What options */
+	switch (gtk_combo_box_get_active (state->format.termination)) {
+	default:
+	case 0: eol = "\n"; break;
+	case 1: eol = "\r"; break;
+	case 2: eol = "\r\n"; break;
+	}
+
+	switch (gtk_combo_box_get_active (state->format.quote)) {
+	default:
+	case 0: quotingmode = GSF_OUTPUT_CSV_QUOTING_MODE_AUTO; break;
+	case 1: quotingmode = GSF_OUTPUT_CSV_QUOTING_MODE_ALWAYS; break;
+	case 2: quotingmode = GSF_OUTPUT_CSV_QUOTING_MODE_NEVER; break;
+	}
+
+	switch (gtk_combo_box_get_active (state->format.transliterate)) {
+	case 0 :  transliteratemode = GNM_STF_TRANSLITERATE_MODE_TRANS; break;
+	default:
+	case 1 :  transliteratemode = GNM_STF_TRANSLITERATE_MODE_ESCAPE; break;
+	}
+
+	switch (gtk_combo_box_get_active (state->format.format)) {
+	default:
+	case 0: format = GNM_STF_FORMAT_AUTO; break;
+	case 1: format = GNM_STF_FORMAT_RAW; break;
+	case 2: format = GNM_STF_FORMAT_PRESERVE; break;
+	}
+
+	quote = gtk_editable_get_chars (GTK_EDITABLE (gtk_bin_get_child (GTK_BIN (state->format.quotechar))), 0, -1);
+
+	switch (gtk_combo_box_get_active (state->format.separator)) {
+	case 0: separator = g_strdup (" "); break;
+	case 1: separator = g_strdup ("\t"); break;
+	case 2: separator = g_strdup ("!"); break;
+	case 3: separator = g_strdup (":"); break;
+	default:
+	case 4: separator = g_strdup (","); break;
+	case 5: separator = g_strdup ("-"); break;
+	case 6: separator = g_strdup ("|"); break;
+	case 7: separator = g_strdup (";"); break;
+	case 8: separator = g_strdup ("/"); break;
+	case 9:
+		separator = gtk_editable_get_chars
+			(GTK_EDITABLE (state->format.custom), 0, -1);
+		break;
+	}
+
+	charset = go_charmap_sel_get_encoding (GO_CHARMAP_SEL (state->format.charset));
+
+	if (quotingmode == GSF_OUTPUT_CSV_QUOTING_MODE_AUTO) {
+		g_string_append (triggers, " \t");
+		g_string_append (triggers, eol);
+		g_string_append (triggers, quote);
+		g_string_append (triggers, separator);
+	}
+
+	state->result = g_object_new
+		(GNM_STF_EXPORT_TYPE,
+		 "eol", eol,
+		 "quote", quote,
+		 "quoting-mode", quotingmode,
+		 "quoting-triggers", triggers->str,
+		 "separator", separator,
+		 "transliterate-mode", transliteratemode,
+		 "format", format,
+		 "charset", charset,
+		 NULL);
 
 	/* Which sheets */
 	stf_export_options_sheet_list_clear (state->result);
 	gtk_tree_model_foreach (GTK_TREE_MODEL (state->sheets.model),
 		(GtkTreeModelForeachFunc) cb_collect_exported_sheets, state);
 
-	/* What options */
-	switch (gtk_combo_box_get_active (state->format.termination)) {
-	case 0 :  terminator = TERMINATOR_TYPE_LINEFEED; break;
-	case 1 :  terminator = TERMINATOR_TYPE_RETURN; break;
-	case 2 :  terminator = TERMINATOR_TYPE_RETURN_LINEFEED; break;
-	default : terminator = TERMINATOR_TYPE_UNKNOWN; break;
-	}
-	stf_export_options_set_terminator_type (state->result, terminator);
-
-	switch (gtk_combo_box_get_active (state->format.quote)) {
-	case 0 :  quotingmode = QUOTING_MODE_AUTO; break;
-	case 1 :  quotingmode = QUOTING_MODE_ALWAYS; break;
-	case 2 :  quotingmode = QUOTING_MODE_NEVER; break;
-	default : quotingmode = QUOTING_MODE_UNKNOWN; break;
-	}
-	stf_export_options_set_quoting_mode (state->result, quotingmode);
-
-	switch (gtk_combo_box_get_active (state->format.transliterate)) {
-	case 0 :  transliteratemode = TRANSLITERATE_MODE_TRANS; break;
-	case 1 :  transliteratemode = TRANSLITERATE_MODE_ESCAPE; break;
-	default : transliteratemode  = TRANSLITERATE_MODE_UNKNOWN; break;
-	}
-	stf_export_options_set_transliterate_mode (state->result, transliteratemode);
-
-	stf_export_options_set_format_mode (state->result,
-		 gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (state->format.preserve)));
-
-	text = gtk_editable_get_chars (GTK_EDITABLE (gtk_bin_get_child (GTK_BIN (state->format.quotechar))), 0, -1);
-	stf_export_options_set_quoting_char (state->result, g_utf8_get_char (text));
-	g_free (text);
-
-	separator = 0;
-	switch (gtk_combo_box_get_active (state->format.separator)) {
-	case 0 : separator = ' '; break;
-	case 1 : separator = '\t'; break;
-	case 2 : separator = '!'; break;
-	case 3 : separator = ':'; break;
-	case 4 : separator = ','; break;
-	case 5 : separator = '-'; break;
-	case 6 : separator = '|'; break;
-	case 7 : separator = ';'; break;
-	case 8 : separator = '/'; break;
-	case 9 :
-		text = gtk_editable_get_chars (GTK_EDITABLE (state->format.custom), 0, -1);
-		separator = g_utf8_get_char (text);
-		g_free (text);
-		break;
-	default :
-		g_warning ("Unknown separator");
-		break;
-	}
-	stf_export_options_set_cell_separator (state->result, separator);
-
-	stf_export_options_set_charset (state->result,
-		charmap_selector_get_encoding (CHARMAP_SELECTOR (state->format.charset)));
+	g_free (separator);
+	g_free (quote);
+	g_string_free (triggers, TRUE);
 
 	gtk_dialog_response (GTK_DIALOG (state->window), GTK_RESPONSE_OK);
 }
@@ -543,16 +560,16 @@ cb_next_page (TextExportState *state)
  * @wb : The #Workbook to export
  *
  * This will start the export assistant.
- * returns : A newly allocated StfExportOptions_t struct on success, NULL otherwise.
+ * returns : A newly allocated GnmStfExport struct on success, NULL otherwise.
  **/
-StfExportOptions_t *
+GnmStfExport *
 stf_export_dialog (WorkbookControlGUI *wbcg, Workbook *wb)
 {
 	TextExportState state;
 
 	g_return_val_if_fail (IS_WORKBOOK (wb), NULL);
 
-	state.gui = gnm_glade_xml_new (GNM_CMD_CONTEXT (wbcg),
+	state.gui = gnm_glade_xml_new (GO_CMD_CONTEXT (wbcg),
 		"dialog-stf-export.glade", NULL, NULL);
 	if (state.gui == NULL)
 		return NULL;
@@ -569,7 +586,8 @@ stf_export_dialog (WorkbookControlGUI *wbcg, Workbook *wb)
 	stf_export_dialog_sheet_page_init (&state);
 	stf_export_dialog_format_page_init (&state);
 	if (state.sheets.non_empty == 0) {
-		gnumeric_notice (wbcg_toplevel (wbcg),  GTK_MESSAGE_ERROR, 
+		gtk_widget_destroy (GTK_WIDGET (state.window));
+		go_gtk_notice_dialog (wbcg_toplevel (wbcg),  GTK_MESSAGE_ERROR, 
 				 _("This workbook does not contain any "
 				   "exportable content."));
 	} else {
@@ -585,10 +603,10 @@ stf_export_dialog (WorkbookControlGUI *wbcg, Workbook *wb)
 					  "clicked",
 					  G_CALLBACK (cb_next_page), &state);
 
-		gnumeric_dialog_run (wbcg_toplevel (wbcg), 
-				     GTK_DIALOG (state.window));
+		go_gtk_dialog_run (GTK_DIALOG (state.window), wbcg_toplevel (wbcg));
 	}
-	g_object_unref (G_OBJECT (state.gui));
+	g_object_unref (state.gui);
+	g_object_unref (state.sheets.model);
 
 	return state.result;
 }
