@@ -18,22 +18,17 @@
 
 #include <gui-gnumeric.h>
 #include <stf.h>
-#include <widgets/gnumeric-lazy-list.h>
 #include <gtk/gtkwidget.h>
 #include <gtk/gtktreeview.h>
 #include <gtk/gtktooltips.h>
 
 #define LINE_DISPLAY_LIMIT 500
-#define RAW_LINE_DISPLAY_LIMIT 4096
-#define X_OVERFLOW_PROTECT 2048
-
 #define COLUMN_CAPTION N_("Column %d")
-#define MOUSE_SENSITIVITY 5
 
 typedef struct {
 	GtkWidget        *data_container;
+	GStringChunk     *lines_chunk;
 	GPtrArray        *lines;
-	GnumericLazyList *ll;
 	GtkTreeView      *tree_view;
 	GtkTooltips      *tooltips;
 
@@ -43,23 +38,27 @@ typedef struct {
 	GPtrArray        *colformats;     /* Array containing the desired column formats */
 	gboolean         ignore_formats;
 
-	GnmDateConventions const *date_conv;
+	GODateConventions const *date_conv;
 } RenderData_t;
 
 /* These are for creation/deletion */
 RenderData_t*      stf_preview_new                       (GtkWidget *data_container,
-							  GnmDateConventions const *date_conv);
+							  GODateConventions const *date_conv);
 void               stf_preview_free                      (RenderData_t *data);
 
 /* These are for manipulation */
-void               stf_preview_set_lines                 (RenderData_t *data, GPtrArray *lines);
+void               stf_preview_set_lines                 (RenderData_t *data,
+							  GStringChunk *lines_chunk,
+							  GPtrArray *lines);
 void               stf_preview_set_startrow              (RenderData_t *data, int startrow);
 
 void               stf_preview_colformats_clear          (RenderData_t *renderdata);
-void               stf_preview_colformats_add            (RenderData_t *renderdata, StyleFormat *format);
+void               stf_preview_colformats_add            (RenderData_t *renderdata, GOFormat *format);
 
 
 GtkTreeViewColumn *stf_preview_get_column                (RenderData_t *renderdata, int col);
 GtkCellRenderer   *stf_preview_get_cell_renderer         (RenderData_t *renderdata, int col);
 
+void               stf_preview_find_column               (RenderData_t *renderdata, int x, int *pcol, int *dx);
+
 #endif /* GNUMERIC_DIALOG_STF_PREVIEW_H */
