@@ -24,14 +24,14 @@
 #include <gnumeric.h>
 #include "dialog-stf.h"
 
-#include <format.h>
+#include <gnm-format.h>
 #include <command-context.h>
 #include <gui-util.h>
-#include <gdk/gdkkeysyms.h>
 #include <sheet-style.h>
 #include <mstyle.h>
 #include <clipboard.h>
 #include <gtk/gtkmain.h>
+#include <gdk/gdkkeysyms.h>
 
 /**********************************************************************************************
  * DIALOG CONTROLLING CODE
@@ -61,7 +61,7 @@ stf_dialog_set_initial_keyboard_focus (StfDialogData *pagedata)
 		focus_widget = GTK_WIDGET (pagedata->fixed.fixed_auto);
 		break;
 	case DPG_FORMAT:
-		number_format_selector_set_focus (pagedata->format.format_selector);
+		go_format_sel_set_focus (pagedata->format.format_selector);
 		break;
 	default:
 		g_assert_not_reached ();
@@ -175,10 +175,10 @@ back_clicked (G_GNUC_UNUSED GtkWidget *widget, StfDialogData *data)
 static void
 cancel_clicked (G_GNUC_UNUSED GtkWidget *widget, StfDialogData *data)
 {
-	if (gnumeric_dialog_question_yes_no
+	if (go_gtk_query_yes_no
 	    (GTK_WINDOW (data->dialog),
-	     _("Are you sure you want to cancel?"),
-	     FALSE)) {
+	     FALSE,
+	     _("Are you sure you want to cancel?"))) {
 		data->canceled = TRUE;
 		gtk_widget_destroy (GTK_WIDGET (data->dialog));
 		gtk_main_quit ();
@@ -298,7 +298,7 @@ stf_dialog_editables_enter (StfDialogData *pagedata)
 	gnumeric_editable_enters
 		(pagedata->window,
 		 GTK_WIDGET (&pagedata->fixed.fixed_colend->entry));
-	number_format_selector_editable_enters
+	go_format_sel_editable_enters
 		(pagedata->format.format_selector,
 	         pagedata->window);
 #endif
@@ -334,7 +334,7 @@ stf_dialog (WorkbookControlGUI *wbcg,
 	g_return_val_if_fail (source != NULL, NULL);
 	g_return_val_if_fail (data != NULL, NULL);
 
-	gui = gnm_glade_xml_new (GNM_CMD_CONTEXT (wbcg),
+	gui = gnm_glade_xml_new (GO_CMD_CONTEXT (wbcg),
 		"dialog-stf.glade", NULL, NULL);
 	if (gui == NULL)
 		return NULL;
@@ -377,7 +377,7 @@ stf_dialog (WorkbookControlGUI *wbcg,
 	prepare_page (&pagedata);
 	frob_buttons (&pagedata);
 
-	gnumeric_set_transient (wbcg_toplevel (wbcg), GTK_WINDOW (pagedata.dialog));
+	wbcg_set_transient_for (wbcg, GTK_WINDOW (pagedata.dialog));
 	gtk_widget_show (GTK_WIDGET (pagedata.dialog));
 	gtk_main ();
 
@@ -474,7 +474,7 @@ stf_dialog_result_attach_formats_to_cr (DialogStfResult_t *dialogresult,
 	targetcol = 0;
 	for (col = 0; col < dialogresult->parseoptions->formats->len; col++) {
 		if (dialogresult->parseoptions->col_import_array[col]) {
-			GnmFormat *sf = g_ptr_array_index 
+			GOFormat *sf = g_ptr_array_index 
 				(dialogresult->parseoptions->formats, col);
 			GnmStyleRegion *sr = g_new (GnmStyleRegion, 1);
 			
@@ -482,8 +482,8 @@ stf_dialog_result_attach_formats_to_cr (DialogStfResult_t *dialogresult,
 			sr->range.start.row = 0;
 			sr->range.end.col   = targetcol;
 			sr->range.end.row   = dialogresult->rowcount - 1;
-			sr->style = mstyle_new_default ();
-			mstyle_set_format (sr->style, sf);
+			sr->style = gnm_style_new_default ();
+			gnm_style_set_format (sr->style, sf);
 			targetcol++;
 			
 			cr->styles = g_slist_prepend (cr->styles, sr);
