@@ -63,7 +63,7 @@ typedef struct {
 	GtkTreeSelection   *sheet_list_selection;
 } ZoomState;
 
-static struct {
+static const struct {
 	char const * const name;
 	gint const factor;
 } buttons[] = {
@@ -176,7 +176,7 @@ dialog_zoom (WorkbookControlGUI *wbcg, Sheet *sheet)
 
 	if (gnumeric_dialog_raise_if_exists (wbcg, ZOOM_DIALOG_KEY))
 		return;
-	gui = gnm_glade_xml_new (GNM_CMD_CONTEXT (wbcg),
+	gui = gnm_glade_xml_new (GO_CMD_CONTEXT (wbcg),
 		"dialog-zoom.glade", NULL, NULL);
 	if (gui == NULL)
 		return;
