@@ -7,6 +7,10 @@
  * Authors:
  *     Miguel de Icaza (miguel@kernel.org)
  *     Jody Goldberg (jody@gnome.org)
+ *
+ * Port to Maemo:
+ * 	Eduardo Lima  (eduardo.lima@indt.org.br)
+ * 	Renato Araujo (renato.filho@indt.org.br)
  */
 
 #include <gnumeric-config.h>
@@ -31,6 +35,9 @@
 #include "selection.h"
 #include "parse-util.h"
 #include "mstyle.h"
+#include "style-conditions.h"
+#include "gnm-style-impl.h"	/* cheesy */
+#include "position.h"		/* to eval conditions */
 #include "style-border.h"
 #include "style-color.h"
 #include "pattern.h"
@@ -265,6 +272,14 @@ item_grid_draw_merged_range (GdkDrawable *drawable, ItemGrid *ig,
 	if (l == r || t == b)
 		return;
 
+	if (style->conditions) {
+		GnmEvalPos ep;
+		int res;
+		eval_pos_init (&ep, (Sheet *)sheet, range->start.col, range->start.row);
+		if ((res = gnm_style_conditions_eval (style->conditions, &ep)) >= 0)
+			style = g_ptr_array_index (style->cond_styles, res);
+	}
+
 	/* Check for background THEN selection */
 	if (gnumeric_background_set_gc (style, gc,
 			ig->canvas_item.canvas, is_selected) ||
@@ -273,7 +288,7 @@ item_grid_draw_merged_range (GdkDrawable *drawable, ItemGrid *ig,
 		if (sheet->text_is_rtl)
 			gdk_draw_rectangle (drawable, gc, TRUE, r, t, l-r+1, b-t+1);
 		else
-		gdk_draw_rectangle (drawable, gc, TRUE, l, t, r-l+1, b-t+1);
+			gdk_draw_rectangle (drawable, gc, TRUE, l, t, r-l+1, b-t+1);
 	}
 
 	if (range->start.col < view->start.col)
@@ -370,7 +385,7 @@ item_grid_draw (FooCanvasItem *item, GdkDrawable *drawable, GdkEventExpose *expo
 	int end_row = gnm_canvas_find_row (gcanvas, expose->area.y+height+2, NULL);
 	int const start_y = y;
 
-	GnmRow sr, next_sr;
+	GnmStyleRow sr, next_sr;
 	GnmStyle const **styles;
 	GnmBorder const **borders, **prev_vert;
 	GnmBorder const *none =
@@ -881,8 +896,12 @@ item_grid_button_press (ItemGrid *ig, GdkEventButton *event)
 				      NULL);
 
 			if ((ig->last_click_time + double_click_time) > event->time &&
-			    wbcg_edit_start (scg->wbcg, FALSE, FALSE))
+			    wbcg_edit_start (scg->wbcg, FALSE, FALSE)) {
+#ifdef USE_HILDON
+				gtk_im_context_focus_in (GNM_CANVAS (canvas)->im_context);
+#endif
 				break;
+			}
 		}
 
 		ig->last_click_time = event->time;
@@ -910,16 +929,17 @@ item_grid_button_press (ItemGrid *ig, GdkEventButton *event)
  */
 
 static gboolean
-cb_extend_cell_range (GnmCanvas *gcanvas, int col, int row, gpointer ignored)
+cb_extend_cell_range (GnmCanvas *gcanvas, GnmCanvasSlideInfo const *info)
 {
-	sv_selection_extend_to (((SheetControl *) gcanvas->simple.scg)->view, col, row);
+	sv_selection_extend_to (((SheetControl *) gcanvas->simple.scg)->view,
+		info->col, info->row);
 	return TRUE;
 }
 
 static gboolean
-cb_extend_expr_range (GnmCanvas *gcanvas, int col, int row, gpointer ignored)
+cb_extend_expr_range (GnmCanvas *gcanvas, GnmCanvasSlideInfo const *info)
 {
-	scg_rangesel_extend_to (gcanvas->simple.scg, col, row);
+	scg_rangesel_extend_to (gcanvas->simple.scg, info->col, info->row);
 	return TRUE;
 }
 
