@@ -127,7 +127,6 @@ gnm_common_init (gboolean fast)
 	rendered_value_init ();
 	gnumeric_color_init ();
 	style_init ();
-	format_color_init ();
 	format_match_init ();
 	/* e_cursors_init (); */
 	functions_init ();
@@ -183,7 +182,6 @@ gnm_shutdown (void)
 	functions_shutdown ();
 	/* e_cursors_shutdown (); */
 	format_match_finish ();
-	format_color_shutdown ();
 	style_shutdown ();
 	gnumeric_color_shutdown ();
 	rendered_value_shutdown ();
