@@ -125,13 +125,14 @@
 #include "expr-name.h"
 #include "workbook.h"
 #include "sheet.h"
-#include "format.h"
+#include "gnm-format.h"
 #include "application.h"
 #include "parse-util.h"
 #include "gutils.h"
 #include "style.h"
 #include "value.h"
 #include "str.h"
+#include <goffice/utils/go-glib-extras.h>
 
 #include <string.h>
 #include <errno.h>
@@ -308,7 +309,7 @@ typedef struct {
 	gboolean force_explicit_sheet_references;
 	gboolean unknown_names_are_strings;
 
-	GnmExprConventions *convs;
+	GnmExprConventions const *convs;
 
 	GnmExprList *result;
 
@@ -531,7 +532,7 @@ parser_simple_val_or_name (GnmExpr *str_expr)
 				GnmParsePos pp = *state->pos;
 				pp.sheet = NULL;
 				/* Create a place holder */
-				nexpr = expr_name_add (&pp, str, NULL, NULL, TRUE);
+				nexpr = expr_name_add (&pp, str, NULL, NULL, TRUE, NULL);
 				res = gnm_expr_new_name (nexpr, NULL, NULL);
 			}
 		} else
@@ -597,7 +598,7 @@ int yyparse (void);
 #endif
 
 #if ! defined (YYSTYPE) && ! defined (YYSTYPE_IS_DECLARED)
-#line 476 "parser.y"
+#line 477 "parser.y"
 typedef union YYSTYPE {
 	GnmExpr		*expr;
 	GnmValue	*value;
@@ -607,7 +608,7 @@ typedef union YYSTYPE {
 	Workbook	*wb;
 } YYSTYPE;
 /* Line 196 of yacc.c.  */
-#line 611 "parser.c"
+#line 612 "parser.c"
 # define yystype YYSTYPE /* obsolescent; will be withdrawn */
 # define YYSTYPE_IS_DECLARED 1
 # define YYSTYPE_IS_TRIVIAL 1
@@ -619,7 +620,7 @@ typedef union YYSTYPE {
 
 
 /* Line 219 of yacc.c.  */
-#line 623 "parser.c"
+#line 624 "parser.c"
 
 #if ! defined (YYSIZE_T) && defined (__SIZE_TYPE__)
 # define YYSIZE_T __SIZE_TYPE__
@@ -863,13 +864,13 @@ static const yysigned_char yyrhs[] =
 /* YYRLINE[YYN] -- source line where rule number YYN was defined.  */
 static const unsigned short int yyrline[] =
 {
-       0,   505,   505,   511,   519,   525,   528,   529,   530,   531,
-     532,   533,   534,   535,   536,   537,   538,   539,   540,   541,
-     542,   543,   544,   545,   546,   551,   555,   556,   557,   559,
-     579,   584,   590,   591,   609,   630,   662,   663,   666,   689,
-     698,   710,   711,   712,   716,   720,   725,   730,   741,   751,
-     754,   755,   756,   762,   763,   768,   776,   781,   789,   790,
-     795,   803,   808
+       0,   506,   506,   512,   520,   526,   529,   530,   531,   532,
+     533,   534,   535,   536,   537,   538,   539,   540,   541,   542,
+     543,   544,   545,   546,   547,   552,   556,   557,   558,   560,
+     580,   585,   591,   592,   610,   631,   663,   664,   667,   690,
+     699,   711,   712,   713,   717,   721,   726,   731,   742,   752,
+     755,   756,   757,   763,   764,   769,   777,   782,   790,   791,
+     796,   804,   809
 };
 #endif
 
@@ -1716,7 +1717,7 @@ yyreduce:
   switch (yyn)
     {
         case 2:
-#line 505 "parser.y"
+#line 506 "parser.y"
     {
 		unregister_allocation ((yyvsp[0].expr));
 		unregister_allocation ((yyvsp[-1].list));
@@ -1725,7 +1726,7 @@ yyreduce:
     break;
 
   case 3:
-#line 511 "parser.y"
+#line 512 "parser.y"
     {
 		if (state->result != NULL) {
 			gnm_expr_list_unref (state->result);
@@ -1735,7 +1736,7 @@ yyreduce:
     break;
 
   case 4:
-#line 519 "parser.y"
+#line 520 "parser.y"
     {
 	       unregister_allocation ((yyvsp[-1].expr));
 	       unregister_allocation ((yyvsp[-2].list));
@@ -1745,102 +1746,102 @@ yyreduce:
     break;
 
   case 5:
-#line 525 "parser.y"
+#line 526 "parser.y"
     { (yyval.list) = NULL; register_expr_list_allocation ((yyval.list)); }
     break;
 
   case 6:
-#line 528 "parser.y"
-    { (yyval.expr) = (yyvsp[0].expr); }
-    break;
-
-  case 7:
 #line 529 "parser.y"
     { (yyval.expr) = (yyvsp[0].expr); }
     break;
 
-  case 8:
+  case 7:
 #line 530 "parser.y"
+    { (yyval.expr) = (yyvsp[0].expr); }
+    break;
+
+  case 8:
+#line 531 "parser.y"
     { (yyval.expr) = parser_simple_val_or_name ((yyvsp[0].expr)); }
     break;
 
   case 9:
-#line 531 "parser.y"
+#line 532 "parser.y"
     { (yyval.expr) = (yyvsp[0].expr); }
     break;
 
   case 10:
-#line 532 "parser.y"
+#line 533 "parser.y"
     { (yyval.expr) = build_binop ((yyvsp[-2].expr), GNM_EXPR_OP_ADD,	(yyvsp[0].expr)); }
     break;
 
   case 11:
-#line 533 "parser.y"
+#line 534 "parser.y"
     { (yyval.expr) = build_binop ((yyvsp[-2].expr), GNM_EXPR_OP_SUB,	(yyvsp[0].expr)); }
     break;
 
   case 12:
-#line 534 "parser.y"
+#line 535 "parser.y"
     { (yyval.expr) = build_binop ((yyvsp[-2].expr), GNM_EXPR_OP_MULT,	(yyvsp[0].expr)); }
     break;
 
   case 13:
-#line 535 "parser.y"
+#line 536 "parser.y"
     { (yyval.expr) = build_binop ((yyvsp[-2].expr), GNM_EXPR_OP_DIV,	(yyvsp[0].expr)); }
     break;
 
   case 14:
-#line 536 "parser.y"
+#line 537 "parser.y"
     { (yyval.expr) = build_binop ((yyvsp[-2].expr), GNM_EXPR_OP_EXP,	(yyvsp[0].expr)); }
     break;
 
   case 15:
-#line 537 "parser.y"
+#line 538 "parser.y"
     { (yyval.expr) = build_binop ((yyvsp[-2].expr), GNM_EXPR_OP_CAT,	(yyvsp[0].expr)); }
     break;
 
   case 16:
-#line 538 "parser.y"
+#line 539 "parser.y"
     { (yyval.expr) = build_binop ((yyvsp[-2].expr), GNM_EXPR_OP_EQUAL,	(yyvsp[0].expr)); }
     break;
 
   case 17:
-#line 539 "parser.y"
+#line 540 "parser.y"
     { (yyval.expr) = build_binop ((yyvsp[-2].expr), GNM_EXPR_OP_LT,		(yyvsp[0].expr)); }
     break;
 
   case 18:
-#line 540 "parser.y"
+#line 541 "parser.y"
     { (yyval.expr) = build_binop ((yyvsp[-2].expr), GNM_EXPR_OP_GT,		(yyvsp[0].expr)); }
     break;
 
   case 19:
-#line 541 "parser.y"
+#line 542 "parser.y"
     { (yyval.expr) = build_binop ((yyvsp[-2].expr), GNM_EXPR_OP_GTE,	(yyvsp[0].expr)); }
     break;
 
   case 20:
-#line 542 "parser.y"
+#line 543 "parser.y"
     { (yyval.expr) = build_binop ((yyvsp[-2].expr), GNM_EXPR_OP_NOT_EQUAL,	(yyvsp[0].expr)); }
     break;
 
   case 21:
-#line 543 "parser.y"
+#line 544 "parser.y"
     { (yyval.expr) = build_binop ((yyvsp[-2].expr), GNM_EXPR_OP_LTE,	(yyvsp[0].expr)); }
     break;
 
   case 22:
-#line 544 "parser.y"
+#line 545 "parser.y"
     { (yyval.expr) = build_logical ((yyvsp[-2].expr), TRUE,	(yyvsp[0].expr)); }
     break;
 
   case 23:
-#line 545 "parser.y"
+#line 546 "parser.y"
     { (yyval.expr) = build_logical ((yyvsp[-2].expr), FALSE, (yyvsp[0].expr)); }
     break;
 
   case 24:
-#line 546 "parser.y"
+#line 547 "parser.y"
     {
 		(yyval.expr) = build_intersect ((yyvsp[-2].expr), (yyvsp[0].expr));
 		if ((yyval.expr) == NULL) { YYERROR; }
@@ -1848,7 +1849,7 @@ yyreduce:
     break;
 
   case 25:
-#line 551 "parser.y"
+#line 552 "parser.y"
     {
 		GnmExpr *tmp = fold_negative ((yyvsp[0].expr));
 		(yyval.expr) = (tmp != NULL) ? tmp : build_unary_op (GNM_EXPR_OP_UNARY_NEG, (yyvsp[0].expr));
@@ -1856,22 +1857,22 @@ yyreduce:
     break;
 
   case 26:
-#line 555 "parser.y"
+#line 556 "parser.y"
     { (yyval.expr) = build_unary_op (GNM_EXPR_OP_UNARY_PLUS, (yyvsp[0].expr)); }
     break;
 
   case 27:
-#line 556 "parser.y"
+#line 557 "parser.y"
     { (yyval.expr) = build_not ((yyvsp[0].expr)); }
     break;
 
   case 28:
-#line 557 "parser.y"
+#line 558 "parser.y"
     { (yyval.expr) = build_unary_op (GNM_EXPR_OP_PERCENTAGE, (yyvsp[-1].expr)); }
     break;
 
   case 29:
-#line 559 "parser.y"
+#line 560 "parser.y"
     {
 		if ((yyvsp[-1].list) == NULL) {
 			report_err (state, g_error_new (1, PERR_INVALID_EMPTY,
@@ -1895,7 +1896,7 @@ yyreduce:
     break;
 
   case 30:
-#line 579 "parser.y"
+#line 580 "parser.y"
     {
 		unregister_allocation ((yyvsp[-1].list));
 		(yyval.expr) = build_array ((yyvsp[-1].list));
@@ -1904,7 +1905,7 @@ yyreduce:
     break;
 
   case 31:
-#line 584 "parser.y"
+#line 585 "parser.y"
     {
 		unregister_allocation ((yyvsp[-1].list));
 		(yyval.expr) = build_array ((yyvsp[-1].list));
@@ -1913,7 +1914,7 @@ yyreduce:
     break;
 
   case 33:
-#line 591 "parser.y"
+#line 592 "parser.y"
     {
 		GnmNamedExpr *nexpr = NULL;
 		char const *name = (yyvsp[0].expr)->constant.value->v_str.val->str;
@@ -1935,7 +1936,7 @@ yyreduce:
     break;
 
   case 34:
-#line 609 "parser.y"
+#line 610 "parser.y"
     {
 		GnmNamedExpr *nexpr = NULL;
 		char const *name = (yyvsp[0].expr)->constant.value->v_str.val->str;
@@ -1958,7 +1959,7 @@ yyreduce:
     break;
 
   case 35:
-#line 630 "parser.y"
+#line 631 "parser.y"
     {
 		char const *name = (yyvsp[-3].expr)->constant.value->v_str.val->str;
 		GnmFunc *f;
@@ -1992,7 +1993,7 @@ yyreduce:
     break;
 
   case 38:
-#line 666 "parser.y"
+#line 667 "parser.y"
     {
 		char const *wb_name = (yyvsp[-1].expr)->constant.value->v_str.val->str;
 		Workbook *wb = gnm_app_workbook_get_by_name (wb_name);
@@ -2014,7 +2015,7 @@ yyreduce:
     break;
 
   case 39:
-#line 689 "parser.y"
+#line 690 "parser.y"
     {
 		Sheet *sheet = parser_sheet_by_name (state->pos->wb, (yyvsp[-1].expr));
 		if (sheet != NULL) {
@@ -2027,7 +2028,7 @@ yyreduce:
     break;
 
   case 40:
-#line 698 "parser.y"
+#line 699 "parser.y"
     {
 		Sheet *sheet = parser_sheet_by_name ((yyvsp[-2].wb), (yyvsp[-1].expr));
 
@@ -2041,17 +2042,17 @@ yyreduce:
     break;
 
   case 41:
-#line 710 "parser.y"
+#line 711 "parser.y"
     { (yyval.expr) = (yyvsp[0].expr); }
     break;
 
   case 42:
-#line 711 "parser.y"
+#line 712 "parser.y"
     { (yyval.expr) = build_range_ctor ((yyvsp[-2].expr), (yyvsp[0].expr), NULL); }
     break;
 
   case 43:
-#line 712 "parser.y"
+#line 713 "parser.y"
     {
 		(yyval.expr) = build_range_ctor ((yyvsp[-2].expr), (yyvsp[0].expr), (yyvsp[-2].expr));
 		if ((yyval.expr) == NULL) { YYERROR; }
@@ -2059,7 +2060,7 @@ yyreduce:
     break;
 
   case 44:
-#line 716 "parser.y"
+#line 717 "parser.y"
     {
 		(yyval.expr) = build_range_ctor ((yyvsp[-2].expr), (yyvsp[0].expr), (yyvsp[0].expr));
 		if ((yyval.expr) == NULL) { YYERROR; }
@@ -2067,14 +2068,14 @@ yyreduce:
     break;
 
   case 45:
-#line 720 "parser.y"
+#line 721 "parser.y"
     {
 		(yyval.expr) = build_binop ((yyvsp[-2].expr), GNM_EXPR_OP_RANGE_CTOR, (yyvsp[0].expr));
 	}
     break;
 
   case 46:
-#line 725 "parser.y"
+#line 726 "parser.y"
     {
 		unregister_allocation ((yyvsp[0].expr));
 		(yyval.list) = gnm_expr_list_prepend (NULL, (yyvsp[0].expr));
@@ -2083,7 +2084,7 @@ yyreduce:
     break;
 
   case 47:
-#line 730 "parser.y"
+#line 731 "parser.y"
     {
 		GSList *tmp = (yyvsp[0].list);
 		unregister_allocation ((yyvsp[0].list));
@@ -2098,7 +2099,7 @@ yyreduce:
     break;
 
   case 48:
-#line 741 "parser.y"
+#line 742 "parser.y"
     {
 		GSList *tmp = (yyvsp[0].list);
 		unregister_allocation ((yyvsp[0].list));
@@ -2112,32 +2113,32 @@ yyreduce:
     break;
 
   case 49:
-#line 751 "parser.y"
+#line 752 "parser.y"
     { (yyval.list) = NULL; }
     break;
 
   case 50:
-#line 754 "parser.y"
+#line 755 "parser.y"
     { (yyval.expr) = (yyvsp[0].expr); }
     break;
 
   case 51:
-#line 755 "parser.y"
+#line 756 "parser.y"
     { (yyval.expr) = fold_negative ((yyvsp[0].expr)); }
     break;
 
   case 52:
-#line 756 "parser.y"
+#line 757 "parser.y"
     { (yyval.expr) = parse_string_as_value ((yyvsp[0].expr)); }
     break;
 
   case 53:
-#line 762 "parser.y"
+#line 763 "parser.y"
     { (yyval.list) = NULL; }
     break;
 
   case 54:
-#line 763 "parser.y"
+#line 764 "parser.y"
     {
 		unregister_allocation ((yyvsp[0].expr));
 		(yyval.list) = g_slist_prepend (NULL, (yyvsp[0].expr));
@@ -2146,7 +2147,7 @@ yyreduce:
     break;
 
   case 55:
-#line 768 "parser.y"
+#line 769 "parser.y"
     {
 		unregister_allocation ((yyvsp[0].list));
 		unregister_allocation ((yyvsp[-2].expr));
@@ -2156,7 +2157,7 @@ yyreduce:
     break;
 
   case 56:
-#line 776 "parser.y"
+#line 777 "parser.y"
     {
 		unregister_allocation ((yyvsp[0].list));
 		(yyval.list) = g_slist_prepend (NULL, (yyvsp[0].list));
@@ -2165,7 +2166,7 @@ yyreduce:
     break;
 
   case 57:
-#line 781 "parser.y"
+#line 782 "parser.y"
     {
 		unregister_allocation ((yyvsp[0].list));
 		unregister_allocation ((yyvsp[-2].list));
@@ -2175,12 +2176,12 @@ yyreduce:
     break;
 
   case 58:
-#line 789 "parser.y"
+#line 790 "parser.y"
     { (yyval.list) = NULL; }
     break;
 
   case 59:
-#line 790 "parser.y"
+#line 791 "parser.y"
     {
 		unregister_allocation ((yyvsp[0].expr));
 		(yyval.list) = g_slist_prepend (NULL, (yyvsp[0].expr));
@@ -2189,7 +2190,7 @@ yyreduce:
     break;
 
   case 60:
-#line 795 "parser.y"
+#line 796 "parser.y"
     {
 		unregister_allocation ((yyvsp[0].list));
 		unregister_allocation ((yyvsp[-2].expr));
@@ -2199,7 +2200,7 @@ yyreduce:
     break;
 
   case 61:
-#line 803 "parser.y"
+#line 804 "parser.y"
     {
 		unregister_allocation ((yyvsp[0].list));
 		(yyval.list) = g_slist_prepend (NULL, (yyvsp[0].list));
@@ -2208,7 +2209,7 @@ yyreduce:
     break;
 
   case 62:
-#line 808 "parser.y"
+#line 809 "parser.y"
     {
 		unregister_allocation ((yyvsp[0].list));
 		unregister_allocation ((yyvsp[-2].list));
@@ -2222,7 +2223,7 @@ yyreduce:
     }
 
 /* Line 1126 of yacc.c.  */
-#line 2226 "parser.c"
+#line 2227 "parser.c"
 
   yyvsp -= yylen;
   yyssp -= yylen;
@@ -2490,7 +2491,7 @@ yyreturn:
 }
 
 
-#line 815 "parser.y"
+#line 816 "parser.y"
 
 
 static char const *
@@ -2508,7 +2509,7 @@ find_matching_close (char const *str, char const **res)
 			return str;
 		else if (*str == '\'' || *str == '\"') {
 			GString *dummy = g_string_new (NULL);
-			char const *end = gnm_strunescape (dummy, str);
+			char const *end = go_strunescape (dummy, str);
 			g_string_free (dummy, TRUE);
 			if (end == NULL)
 				return str + strlen (str);
@@ -2544,13 +2545,14 @@ ignore_space_before (gunichar c)
 	case ')':
 	case '#':
 	case '"': case '\'':  /* Refers to opening quote only.  */
-	case 0x00AC: /* NOT */
-	case 0x2215: /* '/' */
-	case 0x2227: /* AND */
-	case 0x2228: /* OR  */
-	case 0x2260: /* NE  */
-	case 0x2264: /* LTE */
-	case 0x2265: /* GTE */
+	case UNICODE_LOGICAL_NOT_C:
+	case UNICODE_LOGICAL_AND_C:
+	case UNICODE_LOGICAL_OR_C:
+	case UNICODE_MINUS_SIGN_C:
+	case UNICODE_DIVISION_SLASH_C:
+	case UNICODE_NOT_EQUAL_TO_C:
+	case UNICODE_LESS_THAN_OR_EQUAL_TO_C:
+	case UNICODE_GREATER_THAN_OR_EQUAL_TO_C:
 	case 0:
 		return TRUE;
 	default:
@@ -2569,13 +2571,14 @@ ignore_space_after (gunichar c)
 	case '>': case '<': case '=':
 	case '(':
 	case '"': case '\'':  /* Refers to closing quote only [not actually hit].  */
-	case 0x00AC: /* NOT */
-	case 0x2215: /* '/' */
-	case 0x2227: /* AND */
-	case 0x2228: /* OR  */
-	case 0x2260: /* NE  */
-	case 0x2264: /* LTE */
-	case 0x2265: /* GTE */
+	case UNICODE_LOGICAL_NOT_C:
+	case UNICODE_LOGICAL_AND_C:
+	case UNICODE_LOGICAL_OR_C:
+	case UNICODE_MINUS_SIGN_C:
+	case UNICODE_DIVISION_SLASH_C:
+	case UNICODE_NOT_EQUAL_TO_C:
+	case UNICODE_LESS_THAN_OR_EQUAL_TO_C:
+	case UNICODE_GREATER_THAN_OR_EQUAL_TO_C:
 	case 0:
 		return TRUE;
 	default:
@@ -2738,7 +2741,7 @@ yylex (void)
 	if (c == state->separator)
 		return eat_space (state, SEPARATOR);
 
-	if (start != (end = state->convs->ref_parser (&ref, start, state->pos))) {
+	if (start != (end = state->convs->ref_parser (&ref, start, state->pos, state->convs))) {
 		state->ptr = end;
 		if (state->force_absolute_col_references) {
 			if (ref.a.col_relative) {
@@ -2810,7 +2813,7 @@ yylex (void)
 			gnm_float d;
 
 			errno = 0;
-			d = strtognum (start, &end);
+			d = gnm_strto (start, &end);
 			if (start == end) {
 				g_warning ("%s is not a double, but was expected to be one", start);
 			}  else if (errno != ERANGE) {
@@ -2846,7 +2849,7 @@ yylex (void)
 				gnm_float d;
 
 				errno = 0;
-				d = strtognum (start, &end);
+				d = gnm_strto (start, &end);
 				if (errno != ERANGE) {
 					v = value_new_float (d);
 					state->ptr = end;
@@ -2899,7 +2902,7 @@ yylex (void)
 	case '\'':
 	case '"': {
 		GString *s = g_string_new (NULL);
-		char const *end = gnm_strunescape (s, start);
+		char const *end = go_strunescape (s, start);
 
 		if (end == NULL) {
 			size_t len = strlen (start);
@@ -2964,14 +2967,14 @@ yylex (void)
 		return (state->array_col_separator == ',')
 			? ARRAY_START_COMMA : ARRAY_START_BSLASH;
 
-	case 0x00AC: return NOT;
-	case 0x2212: return '-';
-	case 0x2215: return '/';
-	case 0x2227: return AND;
-	case 0x2228: return OR;
-	case 0x2260: return eat_space (state, NE);
-	case 0x2264: return eat_space (state, LTE);
-	case 0x2265: return eat_space (state, GTE);
+	case UNICODE_LOGICAL_NOT_C: return NOT;
+	case UNICODE_MINUS_SIGN_C: return '-';
+	case UNICODE_DIVISION_SLASH_C: return '/';
+	case UNICODE_LOGICAL_AND_C: return AND;
+	case UNICODE_LOGICAL_OR_C: return OR;
+	case UNICODE_NOT_EQUAL_TO_C: return eat_space (state, NE);
+	case UNICODE_LESS_THAN_OR_EQUAL_TO_C: return eat_space (state, LTE);
+	case UNICODE_GREATER_THAN_OR_EQUAL_TO_C: return eat_space (state, GTE);
 	}
 
 	if (ignore_space_after (c))
@@ -3004,7 +3007,7 @@ yyerror (char const *s)
 GnmExpr const *
 gnm_expr_parse_str (char const *expr_text, GnmParsePos const *pos,
 		    GnmExprParseFlags flags,
-		    GnmExprConventions *convs,
+		    GnmExprConventions const *convs,
 		    GnmParseError *error)
 {
 	GnmExpr const *expr;
