@@ -21,11 +21,12 @@
 #include "cell.h"
 #include "style.h"
 #include "workbook.h"
+#include "gnumeric-gconf.h"
 
 #include <stdlib.h>
 #include <stdio.h>
 #include <glib.h>
-#include <string.h>
+#include <glib/gi18n.h>
 #include <string.h>
 
 #undef RANGE_DEBUG
@@ -263,19 +264,12 @@ range_name (GnmRange const *src)
 
 	g_return_val_if_fail (src != NULL, "");
 
-	if (src->start.col != src->end.col ||
-	    src->start.row != src->end.row) {
-		char *name_col = g_strdup (col_name (src->start.col));
-		char *name_row = g_strdup (row_name (src->start.row));
-		sprintf (buffer, "%s%s:%s%s", name_col, name_row,
+	sprintf (buffer, "%s%s",
+		 col_name (src->start.col), row_name (src->start.row));
+
+	if (src->start.col != src->end.col || src->start.row != src->end.row)
+		sprintf (buffer + strlen(buffer), ":%s%s",
 			 col_name (src->end.col), row_name (src->end.row));
-		g_free (name_row);
-		g_free (name_col);
-	} else {
-		sprintf (buffer, "%s%s",
-			 col_name (src->start.col),
-			 row_name (src->start.row));
-	}
 
 	return buffer;
 }
@@ -296,64 +290,49 @@ gboolean
 range_has_header (Sheet const *sheet, GnmRange const *src,
 		  gboolean top, gboolean ignore_styles)
 {
-	GnmCell *ca, *cb;
-	GnmValue *valuea, *valueb;
-	GnmStyle *stylea, *styleb;
+	GnmCell const *a, *b;
 	int length, i;
 
 	/* There is only one row or col */
 	if (top) {
-		if (src->end.row <= src->start.row) {
+		if (src->end.row <= src->start.row)
 			return FALSE;
-		}
 		length = src->end.col - src->start.col + 1;
 	} else {
-		if (src->end.col <= src->start.col) {
+		if (src->end.col <= src->start.col)
 			return FALSE;
-		}
 		length = src->end.row - src->start.row + 1;
 	}
 
-	for (i = 0; i<length; i++) {
+	for (i = 0; i < length; i++) {
 		if (top) {
-			ca = sheet_cell_get (sheet, src->start.col + i,
-					     src->start.row);
-			cb = sheet_cell_get (sheet, src->start.col + i,
-					     src->start.row + 1);
+			a = sheet_cell_get (sheet,
+				src->start.col + i, src->start.row);
+			b = sheet_cell_get (sheet,
+				src->start.col + i, src->start.row + 1);
 		} else {
-			ca = sheet_cell_get (sheet, src->start.col,
-					     src->start.row + i);
-			cb = sheet_cell_get (sheet, src->start.col + 1,
-					     src->start.row + i);
+			a = sheet_cell_get (sheet,
+				src->start.col, src->start.row + i);
+			b = sheet_cell_get (sheet,
+				src->start.col + 1, src->start.row + i);
 		}
 
-
-		if (!ca || !cb) {
+		/* be anal */
+		if (a == NULL || a->value == NULL || b == NULL || b->value == NULL)
 			continue;
-		}
 
-		/* Look for value differences */
-		valuea = ca->value;
-		valueb = cb->value;
-
-		if (VALUE_IS_NUMBER (valuea)) {
-			if (!VALUE_IS_NUMBER (valueb)) {
+		if (VALUE_IS_NUMBER (a->value)) {
+			if (!VALUE_IS_NUMBER (b->value))
 				return TRUE;
-			}
-		} else {
-			if (valuea->type != valueb->type) {
-				return TRUE;
-			}
-		}
+			/* check for style differences */
+		} else if (a->value->type != b->value->type)
+			return TRUE;
 
-		if (!ignore_styles) {
-			/* Look for style differences */
-			stylea = cell_get_mstyle (ca);
-			styleb = cell_get_mstyle (cb);
-
-			if (!mstyle_equal (stylea, styleb))
-				return TRUE;
-		}
+		/* Look for style differences */
+		if (!ignore_styles &&
+		    !gnm_style_equal_header (cell_get_mstyle (a),
+					  cell_get_mstyle (b), top))
+			return TRUE;
 	}
 
 	return FALSE;
@@ -658,7 +637,8 @@ range_fragment_free (GSList *fragments)
 gboolean
 range_intersection (GnmRange *r, GnmRange const *a, GnmRange const *b)
 {
-	g_return_val_if_fail (range_overlap (a, b), FALSE);
+	if (!range_overlap (a, b))
+		return FALSE;
 
 	r->start.col = MAX (a->start.col, b->start.col);
 	r->start.row = MAX (a->start.row, b->start.row);
@@ -692,52 +672,51 @@ range_normalize (GnmRange *src)
 	}
 }
 
+static GnmValue *
+cb_find_extents (Sheet *sheet, int col, int row, GnmCell *cell,
+		 GnmCellPos *extent)
+{
+	if (extent->col < col)
+		extent->col = col;
+	if (extent->row < row)
+		extent->row = row;
+	return NULL;
+}
+
 /**
  * range_trim:
  * @sheet: sheet cells are contained on
- * @range: range to trim empty cells from
- * @cols: trim from right, vs bottom
+ * @r:	   range to trim empty cells from
+ * @cols:  trim from right
+ * @rows:  trim from bottom
  *
  * This removes empty rows/cols from the
  * right hand or bottom edges of the range
- * depending on the value of @location.
+ * depending on the value of @cols or @rows.
  *
- * WARNING! FOR LARGE RANGES THIS IS EXPENSIVE!
- *
- * Return value: TRUE if the range was totally empty, else FALSE.
+ * Return value: TRUE if the range was totally empty.
  **/
 gboolean
-range_trim (Sheet const *sheet, GnmRange *range, gboolean cols)
+range_trim (Sheet const *sheet, GnmRange *r,
+	    gboolean cols, gboolean rows)
 {
-	int start, *move;
+	GnmCellPos extent = { -1, -1 };
 
-	/* Setup the pointers to the fields which will
-	 * be changed (to remove the empty cells)
-	 */
-	if (cols) {
-		start = range->start.col;
-		move = &range->end.col;
-		range->start.col = *move;
-		if (*move > sheet->cols.max_used)
-			*move = sheet->cols.max_used;
-	} else {
-		start = range->start.row;
-		move = &range->end.row;
-		range->start.row = *move;
-		if (*move > sheet->rows.max_used)
-			*move = sheet->rows.max_used;
-	}
+	g_return_val_if_fail (IS_SHEET (sheet), TRUE);
+	g_return_val_if_fail (r != NULL, TRUE);
 
-	for (; *move >= start ; (*move)--)
-		if (!sheet_is_region_empty ((Sheet *)sheet, range))
-			break;
+	sheet_foreach_cell_in_range (
+		(Sheet *)sheet, CELL_ITER_IGNORE_BLANK,
+		r->start.col, r->start.row, r->end.col, r->end.row,
+		(CellIterFunc) cb_find_extents, &extent);
 
+	if (extent.col < 0 || extent.row < 0)
+		return TRUE;
 	if (cols)
-		range->start.col = start;
-	else
-		range->start.row = start;
-
-	return *move < start;
+		r->end.col = extent.col;
+	if (rows)
+		r->end.row = extent.row;
+	return FALSE;
 }
 
 /**
@@ -998,13 +977,14 @@ range_transpose (GnmRange *range, GnmCellPos const *origin)
 }
 
 GnmSheetRange *
-global_range_new (Sheet *sheet, GnmRange const *r)
+gnm_sheet_range_new (Sheet *sheet, GnmRange const *r)
 {
-	GnmSheetRange *gr = g_new0 (GnmSheetRange, 1);
+	GnmSheetRange *gr;
 
 	g_return_val_if_fail (IS_SHEET (sheet), NULL);
 	g_return_val_if_fail (r != NULL, NULL);
 
+	gr = g_new0 (GnmSheetRange, 1);
 	gr->sheet = sheet;
 	gr->range = *r;
 
@@ -1012,14 +992,14 @@ global_range_new (Sheet *sheet, GnmRange const *r)
 }
 
 /**
- * value_to_global_range :
- * @v :
+ * gnm_sheet_range_from_value :
  * @res :
+ * @v :
  *
- * convert @v into a global range and return in @res
+ * Convert @v into a GnmSheetRange and return in @res
  **/
 gboolean
-value_to_global_range (GnmValue const *v, GnmSheetRange *res)
+gnm_sheet_range_from_value (GnmSheetRange *res, GnmValue const *v)
 {
 	g_return_val_if_fail (v->type == VALUE_CELLRANGE, FALSE);
 
@@ -1030,15 +1010,13 @@ value_to_global_range (GnmValue const *v, GnmSheetRange *res)
 }
 
 void
-global_range_free (GnmSheetRange *gr)
+gnm_sheet_range_free (GnmSheetRange *gr)
 {
-	g_return_if_fail (gr != NULL);
-
 	g_free (gr);
 }
 
 gboolean
-global_range_overlap (GnmSheetRange const *a, GnmSheetRange const *b)
+gnm_sheet_range_overlap (GnmSheetRange const *a, GnmSheetRange const *b)
 {
 	g_return_val_if_fail (a != NULL, FALSE);
 	g_return_val_if_fail (b != NULL, FALSE);
@@ -1050,15 +1028,15 @@ global_range_overlap (GnmSheetRange const *a, GnmSheetRange const *b)
 }
 
 GnmSheetRange *
-global_range_dup (GnmSheetRange const *src)
+gnm_sheet_range_dup (GnmSheetRange const *src)
 {
 	g_return_val_if_fail (src != NULL, NULL);
 
-	return global_range_new (src->sheet, &src->range);
+	return gnm_sheet_range_new (src->sheet, &src->range);
 }
 
 char *
-global_range_name    (Sheet *sheet, GnmRange const *r)
+global_range_name (Sheet const *sheet, GnmRange const *r)
 {
 	char const * the_range_name = range_name (r);
 
@@ -1068,38 +1046,160 @@ global_range_name    (Sheet *sheet, GnmRange const *r)
 	return g_strdup_printf ("%s!%s", sheet->name_quoted, the_range_name);
 }
 
-/**
- * global_range_parse:
- * @sheet: the sheet where the cell range is evaluated. This really only needed if
- *         the range given does not include a sheet specification.
- * @str: a range specification (ex: "A1", "A1:C3", "Sheet1!A1:C3).
- *
- * Returns a (GnmValue *) of type VALUE_CELLRANGE if the @range was
- * succesfully parsed or NULL on failure.
+/* The maximal allowed width of a range.
+ * We have max_descriptor_width in the preferences, but who knows how much
+ * will be added later?  So we leave 20 chars for the other text...
  */
-GnmValue *
-global_range_parse (Sheet *sheet, char const *str)
+static guint
+max_range_name_width (void)
 {
-	GnmParsePos  pp;
-	GnmExpr const *expr;
+	guint max_width = gnm_app_prefs->max_descriptor_width;
 
-	g_return_val_if_fail (IS_SHEET (sheet), NULL);
-	g_return_val_if_fail (str != NULL, NULL);
+	return (max_width > 23 ? max_width - 20 : 3);
+}
 
-	expr = gnm_expr_parse_str (str,
-		parse_pos_init_sheet (&pp, sheet),
-		GNM_EXPR_PARSE_FORCE_EXPLICIT_SHEET_REFERENCES |
-		GNM_EXPR_PARSE_UNKNOWN_NAMES_ARE_STRINGS,
-		gnm_expr_conventions_default,
-		NULL);
+/**
+ * char const *undo_cell_pos_name
+ * @sheet:
+ * @pos:
+ *
+ * Returns the range name depending on the preference setting.
+ */
+char *
+undo_cell_pos_name (Sheet const *sheet, GnmCellPos const *pos)
+{
+	GnmRange r;
+	r.end = r.start = *pos;
+	return undo_range_name (sheet, &r);
+}
 
-	if (expr != NULL)  {
-		GnmValue *value = gnm_expr_get_range (expr);
-		gnm_expr_unref (expr);
-		return value;
+/**
+ * char const *undo_range_name
+ * @sheet:
+ * @r:
+ *
+ * Returns the range name depending on the preference setting.
+ */
+char *
+undo_range_name (Sheet const *sheet, GnmRange const *r)
+{
+	char const * the_range_name = range_name (r);
+	guint max_width;
+
+	max_width = max_range_name_width ();
+
+	if (sheet != NULL && gnm_app_prefs->show_sheet_name) {
+		char *n = g_strdup_printf ("%s!%s", sheet->name_quoted, the_range_name);
+
+		if (strlen (n) <= max_width)
+			return n;
+		else
+			g_free (n);
 	}
 
-	return NULL;
+	return g_strdup (strlen (the_range_name) <= max_width ? the_range_name : "");
+}
+
+
+/*
+ * Create range list name, but don't exceed max_width.
+ * Return TRUE iff the name is complete.
+ */
+static gboolean
+range_list_name_try (GString *names, Sheet const *sheet, GSList const *ranges,
+		     guint max_width)
+{
+	GSList const *l;
+	char const *n = range_name (ranges->data);
+
+	if (sheet == NULL)
+		g_string_assign (names, n);
+	else
+		g_string_printf (names, "%s!%s", sheet->name_quoted, n);
+
+	if (names->len > max_width) {
+		g_string_truncate (names, 0);
+		return FALSE;
+	}
+
+	for (l = ranges->next; l != NULL; l = l->next) {
+		gsize new_len;
+
+		n = range_name (l->data);
+
+		/* The string may not get too long. */
+		new_len = names->len + 2 + strlen (n);
+		if (sheet != NULL)
+			new_len += strlen (sheet->name_quoted) + 1;
+		if (new_len > max_width)
+			break;
+
+		if (sheet == NULL)
+			g_string_append_printf (names, ", %s", n);
+		else
+			g_string_append_printf (names, ", %s!%s",
+						sheet->name_quoted, n);
+	}
+
+	/* Have we reached the end? */
+	return (l == NULL);
+}
+
+
+/**
+ * char const *undo_range_list_name
+ * @sheet:
+ * @ranges : GSList containing GnmRange *'s
+ *
+ * Returns the range list name depending on the preference setting.
+ * (The result will be something like: "A1:C3, D4:E5"). The string will be
+ * truncated to max_descriptor_width.
+ */
+char *
+undo_range_list_name (Sheet const *sheet, GSList const *ranges)
+{
+	GString *names;
+	GString *trunc_names = NULL;
+	guint max_width;
+
+	g_return_val_if_fail (ranges != NULL, NULL);
+
+	max_width = max_range_name_width ();
+
+	names = g_string_new (NULL);
+
+	/* With the sheet name. */
+	if (sheet != NULL && gnm_app_prefs->show_sheet_name) {
+		if (range_list_name_try (names, sheet, ranges, max_width)) {
+			/* We have reached the end, return the data from names. */
+			return g_string_free (names, FALSE);
+		}
+		/* Store the partial result. */
+		if (names->len > 0) {
+			trunc_names = names;
+			names = g_string_new (NULL);
+		}
+	}
+
+	/* Without the sheet name. */
+	if (range_list_name_try (names, NULL, ranges, max_width)) {
+		/* We have reached the end, return the data from names. */
+		if (trunc_names != NULL)
+			g_string_free (trunc_names, TRUE);
+		return g_string_free (names, FALSE);
+	}
+
+	/* We have to use a truncated version. */
+	if (trunc_names != NULL) {
+		g_string_free (names, TRUE);
+		names = trunc_names;
+	}
+
+	/* Actually we can get slightly more than max_width here.
+	 * But max_width was computed as an estimate, anyway, so we don't care.
+	 */
+	g_string_append (names, _("..."));
+	return g_string_free (names, FALSE);
 }
 
 /**
@@ -1184,7 +1284,7 @@ global_range_list_foreach (GSList *gr_list, GnmEvalPos const *ep,
 gboolean
 global_range_contained (Sheet const *sheet, GnmValue const *a, GnmValue const *b)
 {
-	Sheet *target;
+	Sheet const *target;
 
 	g_return_val_if_fail (a != NULL, FALSE);
 	g_return_val_if_fail (b != NULL, FALSE);
