@@ -17,7 +17,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  */
 #include <gnumeric-config.h>
@@ -49,7 +49,7 @@ sc_finalize (GObject *obj)
 {
 	/* Commented out until needed */
 	/* SheetControl *sc = SHEET_CONTROL (obj); */
-	G_OBJECT_CLASS (parent_class)->finalize (obj);
+	parent_class->finalize (obj);
 }
 
 static void
@@ -85,8 +85,6 @@ sc_view (SheetControl const *sc)
 
 SC_VIRTUAL (resize, (SheetControl *sc, gboolean force_scroll), (sc, force_scroll))
 
-SC_VIRTUAL (set_zoom_factor, (SheetControl *sc), (sc))
-
 SC_VIRTUAL (redraw_all, (SheetControl *sc, gboolean headers), (sc, headers))
 SC_VIRTUAL (redraw_range,
 	    (SheetControl *sc, GnmRange const *r),
@@ -99,8 +97,6 @@ SC_VIRTUAL (redraw_headers,
 SC_VIRTUAL (ant, (SheetControl *sc), (sc))
 SC_VIRTUAL (unant, (SheetControl *sc), (sc))
 
-SC_VIRTUAL (adjust_preferences, (SheetControl *sc), (sc))
-
 SC_VIRTUAL (scrollbar_config, (SheetControl const *sc), (sc));
 
 SC_VIRTUAL (mode_edit, (SheetControl *sc), (sc));
@@ -119,3 +115,5 @@ SC_VIRTUAL (cursor_bound, (SheetControl *sc, GnmRange const *r), (sc, r))
 SC_VIRTUAL (set_panes, (SheetControl *sc), (sc))
 
 SC_VIRTUAL (object_create_view,	(SheetControl *sc, SheetObject *so), (sc, so));
+SC_VIRTUAL (scale_changed,	(SheetControl *sc), (sc))
+
