@@ -17,7 +17,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  */
 #include <gnumeric-config.h>
@@ -111,8 +111,8 @@ comment_foo_view_init (SheetObjectViewIface *sov_iface)
 typedef FooCanvasPolygon	CommentFooView;
 typedef FooCanvasPolygonClass	CommentFooViewClass;
 static GSF_CLASS_FULL (CommentFooView, comment_foo_view,
-	NULL, NULL,
-	FOO_TYPE_CANVAS_POLYGON, 0,
+	NULL, NULL, NULL, NULL,
+	NULL, FOO_TYPE_CANVAS_POLYGON, 0,
 	GSF_INTERFACE (comment_foo_view_init, SHEET_OBJECT_VIEW_TYPE))
 
 static void
@@ -275,15 +275,6 @@ cell_comment_read_xml_dom (SheetObject *so, char const *typename,
 	return FALSE;
 }
 
-static gboolean
-cell_comment_write_xml_dom (SheetObject const *so, XmlParseContext const *ctxt,
-			    xmlNodePtr tree)
-{
-	GnmComment const *cc = CELL_COMMENT (so);
-	xml_node_set_cstr (tree, "Author", cc->author);
-	xml_node_set_cstr (tree, "Text", cc->text);
-	return FALSE;
-}
 static void
 cell_comment_write_xml_sax (SheetObject const *so, GsfXMLOut *output)
 {
@@ -335,7 +326,6 @@ cell_comment_class_init (GObjectClass *gobject_class)
 	/* SheetObject class method overrides */
 	sheet_object_class->new_view		= &cell_comment_new_view;
 	sheet_object_class->read_xml_dom	= &cell_comment_read_xml_dom;
-	sheet_object_class->write_xml_dom	= &cell_comment_write_xml_dom;
 	sheet_object_class->write_xml_sax	= &cell_comment_write_xml_sax;
 	sheet_object_class->print		= &cell_comment_print;
 	sheet_object_class->copy		= &cell_comment_copy;
