@@ -3,7 +3,7 @@
 /*
  * sheet-object-graph.c: Wrapper for GNOME Office graphs in gnumeric
  *
- * Copyright (C) 2003-2004 Jody Goldberg (jody@gnome.org)
+ * Copyright (C) 2003-2005 Jody Goldberg (jody@gnome.org)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of version 2 of the GNU General Public
@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  */
 #include <gnumeric-config.h>
@@ -36,31 +36,38 @@
 #include "commands.h"
 #include "application.h"
 #include "xml-io.h"
+#include <graph.h>
 
 #include <goffice/graph/gog-graph.h>
 #include <goffice/graph/gog-object.h>
 #include <goffice/graph/gog-object-xml.h>
 #include <goffice/graph/gog-data-allocator.h>
+#include <goffice/graph/gog-data-set.h>
 #include <goffice/graph/gog-renderer-gnome-print.h>
 #include <goffice/graph/gog-renderer-pixbuf.h>
 #include <goffice/graph/gog-renderer-svg.h>
 #include <goffice/graph/gog-control-foocanvas.h>
 #include <goffice/utils/go-file.h>
 #include <goffice/utils/go-units.h>
-#include <graph.h>
+#include <goffice/utils/go-libxml-extras.h>
+#include <goffice/utils/go-glib-extras.h>
+#include <goffice/utils/go-format.h>
+#include <goffice/app/go-cmd-context.h>
 
 #include <gsf/gsf-impl-utils.h>
 #include <gsf/gsf-utils.h>
 #include <gsf/gsf-output-stdio.h>
+#include <gsf/gsf-libxml.h>
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtkimagemenuitem.h>
 #include <gtk/gtkstock.h>
-#include <libfoocanvas/foo-canvas-line.h>
-#include <libfoocanvas/foo-canvas-rect-ellipse.h>
-#include <libfoocanvas/foo-canvas-polygon.h>
-#include <libfoocanvas/foo-canvas-text.h>
+#include <goffice/cut-n-paste/foocanvas/foo-canvas-line.h>
+#include <goffice/cut-n-paste/foocanvas/foo-canvas-rect-ellipse.h>
+#include <goffice/cut-n-paste/foocanvas/foo-canvas-polygon.h>
+#include <goffice/cut-n-paste/foocanvas/foo-canvas-text.h>
 #include <math.h>
 #include <string.h>
+#include <locale.h>
 
 static void
 so_graph_view_destroy (SheetObjectView *sov)
@@ -94,8 +101,8 @@ so_graph_foo_view_init (SheetObjectViewIface *sov_iface)
 typedef GogControlFooCanvas		SOGraphFooView;
 typedef GogControlFooCanvasClass	SOGraphFooViewClass;
 static GSF_CLASS_FULL (SOGraphFooView, so_graph_foo_view,
-	NULL, NULL,
-	GOG_CONTROL_FOOCANVAS_TYPE, 0,
+	NULL, NULL, NULL, NULL,
+	NULL, GOG_CONTROL_FOOCANVAS_TYPE, 0,
 	GSF_INTERFACE (so_graph_foo_view_init, SHEET_OBJECT_VIEW_TYPE))
 
 /****************************************************************************/
@@ -140,7 +147,7 @@ sog_datas_set_sheet (SheetObjectGraph *sog, Sheet *sheet)
 }
 
 static void
-sheet_object_graph_finalize (GObject *obj)
+gnm_sog_finalize (GObject *obj)
 {
 	SheetObjectGraph *sog = SHEET_OBJECT_GRAPH (obj);
 
@@ -157,7 +164,7 @@ sheet_object_graph_finalize (GObject *obj)
 }
 
 static SheetObjectView *
-sheet_object_graph_new_view (SheetObject *so, SheetObjectViewContainer *container)
+gnm_sog_new_view (SheetObject *so, SheetObjectViewContainer *container)
 {
 	GnmCanvas *gcanvas = ((GnmPane *)container)->gcanvas;
 	SheetObjectGraph *sog = SHEET_OBJECT_GRAPH (so);
@@ -168,6 +175,52 @@ sheet_object_graph_new_view (SheetObject *so, SheetObjectViewContainer *containe
 	return gnm_pane_object_register (so, view, TRUE);
 }
 
+static GtkTargetList *
+gnm_sog_get_target_list (SheetObject const *so)
+{
+	GtkTargetList *tl = gtk_target_list_new (NULL, 0);
+	char *mime_str = go_image_format_to_mime ("svg");
+	GSList *mimes, *ptr;
+	char *mime;
+
+	mimes = go_strsplit_to_slist (mime_str, ',');
+	for (ptr = mimes; ptr != NULL; ptr = ptr->next) {
+		mime = (char *) ptr->data;
+
+		if (mime != NULL && *mime != '\0') 
+			gtk_target_list_add (tl, gdk_atom_intern (mime, FALSE),
+					     0, 0);
+	}
+	g_free (mime_str);
+	go_slist_free_custom (mimes, g_free);
+	/* No need to eliminate duplicates. */
+	gtk_target_list_add_image_targets (tl, 0, TRUE);
+
+	return tl;
+}
+
+static GtkTargetList *
+gnm_sog_get_object_target_list (SheetObject const *so)
+{
+	GtkTargetList *tl, *tl2;
+	GtkTargetPair *pair;
+	tl = gtk_target_list_new (NULL, 0);
+	gtk_target_list_add (tl, gdk_atom_intern
+					("application/x-goffice-graph", FALSE), 0, 0);
+	if (IS_SHEET_OBJECT_IMAGEABLE (so)) {
+		GList *ptr;
+		tl2 = sheet_object_get_target_list (so);
+		ptr = tl2->list;
+		while (ptr) {
+			pair = (GtkTargetPair*) ptr->data;
+			gtk_target_list_add (tl,pair->target, pair->flags, pair->info);
+			ptr = ptr->next;
+		}
+		gtk_target_list_unref (tl2);
+	}
+	return tl;
+}
+
 static gboolean
 sog_gsf_gdk_pixbuf_save (const gchar *buf,
 			 gsize count,
@@ -183,6 +236,79 @@ sog_gsf_gdk_pixbuf_save (const gchar *buf,
 	return ok;
 }
 
+static void
+gnm_sog_write_image (SheetObject const *so, const char *format,
+				GsfOutput *output, GError **err)
+{
+	SheetObjectGraph *sog = SHEET_OBJECT_GRAPH (so);
+	gboolean res = FALSE;
+	double coords[4];
+	double w, h;
+	
+	if (so->sheet) {
+		sheet_object_position_pts_get (SHEET_OBJECT (sog), coords);
+		w = fabs (coords[2] - coords[0]) + 1.;
+		h = fabs (coords[3] - coords[1]) + 1.;
+	} else {
+		w = GPOINTER_TO_UINT 
+			(g_object_get_data (G_OBJECT (so), "pt-width-at-copy"));
+		h = GPOINTER_TO_UINT 
+			(g_object_get_data (G_OBJECT (so), "pt-height-at-copy"));
+	}
+
+	g_return_if_fail (w > 0 && h > 0);
+
+	if (strcmp (format, "svg") == 0) {
+		res = gog_graph_export_to_svg (sog->graph, output, w, h, 1.0);
+	} else {
+		GogRendererPixbuf *prend = GOG_RENDERER_PIXBUF (sog->renderer);
+		GdkPixbuf *pixbuf = gog_renderer_pixbuf_get (prend);
+
+		if (!pixbuf) {
+			gog_renderer_pixbuf_update (prend, w, h, 1.);
+			pixbuf = gog_renderer_pixbuf_get (prend);
+		}
+		res = gdk_pixbuf_save_to_callback (pixbuf,
+						   sog_gsf_gdk_pixbuf_save,
+						   output, format,
+						   err, NULL);
+	}
+	if (!res && err && *err == NULL)
+		*err = g_error_new (gsf_output_error_id (), 0,
+				    _("Unknown failure while saving image"));
+}
+
+static void
+gnm_sog_write_object (SheetObject const *so, const char *format,
+				 GsfOutput *output, GError **err)
+{
+	SheetObjectGraph *sog = SHEET_OBJECT_GRAPH (so);
+	GsfXMLOut *xout;
+	char *old_num_locale, *old_monetary_locale;
+	GogObject *graph;
+
+	g_return_if_fail (strcmp (format, "application/x-goffice-graph") == 0);
+
+	old_num_locale = g_strdup (go_setlocale (LC_NUMERIC, NULL));
+	go_setlocale (LC_NUMERIC, "C");
+	old_monetary_locale = g_strdup (go_setlocale (LC_MONETARY, NULL));
+	go_setlocale (LC_MONETARY, "C");
+	go_set_untranslated_bools ();
+
+	graph = gog_object_dup (GOG_OBJECT (sog->graph),
+		NULL, gog_dataset_dup_to_simple);
+	xout = gsf_xml_out_new (output);
+	gog_object_write_xml_sax (GOG_OBJECT (graph), xout);
+	g_object_unref (xout);
+	g_object_unref (graph);
+
+	/* go_setlocale restores bools to locale translation */
+	go_setlocale (LC_MONETARY, old_monetary_locale);
+	g_free (old_monetary_locale);
+	go_setlocale (LC_NUMERIC, old_num_locale);
+	g_free (old_num_locale);
+}
+
 /* 
  * The following are useful formats to save in:
  *  png
@@ -196,60 +322,40 @@ sog_gsf_gdk_pixbuf_save (const gchar *buf,
 static void
 sog_cb_save_as (SheetObject *so, SheetControl *sc)
 {
-	WorkbookControlGUI *wbcg;
-	char *uri;
-	GError *err = NULL;
-	gboolean ret = FALSE;
-	GsfOutput *output;
-	GSList *l = NULL;
-	GnmImageFormat fmts[] = {
+	static GOImageType const fmts[] = {
 		{(char *) "svg",  (char *) N_("SVG (vector graphics)"), (char *) "svg", FALSE},
 		{(char *) "png",  (char *) N_("PNG (raster graphics)"), (char *) "png", TRUE},
 		{(char *) "jpeg", (char *) N_("JPEG (photograph)"),     (char *) "jpg", TRUE}
 	};
-	GnmImageFormat *sel_fmt = &fmts[0];
+
+	WorkbookControlGUI *wbcg;
+	char *uri;
+	GError *err = NULL;
+	GsfOutput *output;
+	GSList *l = NULL;
+	GOImageType const *sel_fmt = &fmts[0];
 	guint i;
 	SheetObjectGraph *sog = SHEET_OBJECT_GRAPH (so);
 
 	g_return_if_fail (sog != NULL);
 
-	for (i = 0; i < sizeof fmts / sizeof fmts[0]; i++)
-		l = g_slist_prepend (l, &fmts[i]);
+	for (i = 0; i < G_N_ELEMENTS (fmts); i++)
+		l = g_slist_prepend (l, (gpointer) (fmts + i));
 	l = g_slist_reverse (l);
 
+#warning "This violates model gui barrier"
 	wbcg = scg_get_wbcg (SHEET_CONTROL_GUI (sc));
-
-#warning this violate model gui barrier
-	uri = gui_get_image_save_info (wbcg, l, &sel_fmt);
+	uri = gui_get_image_save_info (wbcg_toplevel (wbcg), l, &sel_fmt);
 	if (!uri)
 		goto out;
 	output = go_file_create (uri, &err);
 	if (!output)
 		goto out;
-	if (strcmp (sel_fmt->name, "svg") == 0) {
-		double coords [4];
-		sheet_object_position_pts_get (SHEET_OBJECT (sog), coords);
-		ret = gog_graph_export_to_svg (sog->graph, output,
-					       fabs (coords[2] - coords[0]),
-					       fabs (coords[3] - coords[1]),
-					       1.);
-	} else {
-		GdkPixbuf *pixbuf = gog_renderer_pixbuf_get (
-			GOG_RENDERER_PIXBUF (sog->renderer));
-
-		ret = gdk_pixbuf_save_to_callback (pixbuf,
-						   sog_gsf_gdk_pixbuf_save,
-						   output, sel_fmt->name,
-						   &err, NULL);
-	}
-	gsf_output_close (output);
+	sheet_object_write_image (so, sel_fmt->name, output, &err);
 	g_object_unref (output);
 		
-	if (!ret && err == NULL)
-		err = g_error_new (gsf_output_error_id (), 0,
-				   _("Unknown failure while saving image"));
-	if (!ret)
-		gnm_cmd_context_error (GNM_CMD_CONTEXT (wbcg), err);
+	if (err != NULL)
+		go_cmd_context_error (GO_CMD_CONTEXT (wbcg), err);
 
 out:
 	g_free (uri);
@@ -257,16 +363,16 @@ out:
 }
 
 static void
-sheet_object_graph_populate_menu (SheetObject *so, GPtrArray *actions)
+gnm_sog_populate_menu (SheetObject *so, GPtrArray *actions)
 {
 	static SheetObjectAction const sog_action =
 		{ GTK_STOCK_SAVE_AS, N_("_Save as image"), NULL, 0, sog_cb_save_as };
 	SHEET_OBJECT_CLASS (parent_klass)->populate_menu (so, actions);
-	gnm_ptr_array_insert (actions, (gpointer) &sog_action, 1);
+	go_ptr_array_insert (actions, (gpointer) &sog_action, 1);
 }
 
 static gboolean
-sheet_object_graph_read_xml_dom (SheetObject *so, char const *typename,
+gnm_sog_read_xml_dom (SheetObject *so, char const *typename,
 				 XmlParseContext const *ctxt, xmlNodePtr tree)
 {
 	xmlNodePtr child = e_xml_get_child_by_name (tree, "GogObject");
@@ -279,25 +385,29 @@ sheet_object_graph_read_xml_dom (SheetObject *so, char const *typename,
 	return FALSE;
 }
 
-static gboolean
-sheet_object_graph_write_xml_dom (SheetObject const *so,
-				  XmlParseContext const *ctxt, xmlNode *parent)
-{
-	SheetObjectGraph *sog = SHEET_OBJECT_GRAPH (so);
-	xmlNode *res = gog_object_write_xml (GOG_OBJECT (sog->graph), ctxt->doc);
-	if (res != NULL)
-		xmlAddChild (parent, res);
-	return FALSE;
-}
 static void
-sheet_object_graph_write_xml_sax (SheetObject const *so, GsfXMLOut *output)
+gnm_sog_write_xml_sax (SheetObject const *so, GsfXMLOut *output)
 {
 	SheetObjectGraph const *sog = SHEET_OBJECT_GRAPH (so);
 	gog_object_write_xml_sax (GOG_OBJECT (sog->graph), output);
 }
 
 static void
-sheet_object_graph_copy (SheetObject *dst, SheetObject const *src)
+sog_xml_finish (GogObject *graph, SheetObject *so)
+{
+	sheet_object_graph_set_gog (so, GOG_GRAPH (graph));
+	g_object_unref (graph);
+}
+
+static void
+gnm_sog_prep_sax_parser (SheetObject *so, GsfXMLIn *xin, xmlChar const **attrs)
+{
+	gog_object_sax_push_parser (xin, attrs,
+		(GogObjectSaxHandler) sog_xml_finish, so);
+}
+
+static void
+gnm_sog_copy (SheetObject *dst, SheetObject const *src)
 {
 	SheetObjectGraph const *sog = SHEET_OBJECT_GRAPH (src);
 	GogGraph *graph   = gog_graph_dup (sog->graph);
@@ -306,7 +416,7 @@ sheet_object_graph_copy (SheetObject *dst, SheetObject const *src)
 }
 
 static void
-sheet_object_graph_print (SheetObject const *so, GnomePrintContext *ctx,
+gnm_sog_print (SheetObject const *so, GnomePrintContext *ctx,
 			  double width, double height)
 {
 	gog_graph_print_to_gnome_print (
@@ -316,10 +426,10 @@ sheet_object_graph_print (SheetObject const *so, GnomePrintContext *ctx,
 typedef struct {
 	SheetObject *so;
 	WorkbookControl *wbc;
-} sheet_object_graph_user_config_t;
+} gnm_sog_user_config_t;
 
 static void
-sheet_object_graph_user_config_free_data (gpointer data,
+gnm_sog_user_config_free_data (gpointer data,
 					  GClosure *closure)
 {
 	g_free (data);
@@ -327,18 +437,18 @@ sheet_object_graph_user_config_free_data (gpointer data,
 }
 
 static void
-cb_update_graph (GogGraph *graph, sheet_object_graph_user_config_t *data)
+cb_update_graph (GogGraph *graph, gnm_sog_user_config_t *data)
 {
 	cmd_so_graph_config (data->wbc, data->so, G_OBJECT (graph), 
 			     G_OBJECT (SHEET_OBJECT_GRAPH (data->so)->graph));
 }
 
 static void
-sheet_object_graph_user_config (SheetObject *so, SheetControl *sc)
+gnm_sog_user_config (SheetObject *so, SheetControl *sc)
 {
 	SheetObjectGraph *sog = SHEET_OBJECT_GRAPH (so);
 	WorkbookControlGUI *wbcg;
-	sheet_object_graph_user_config_t *data;
+	gnm_sog_user_config_t *data;
 	GClosure *closure;
 
 	g_return_if_fail (sog != NULL);
@@ -346,19 +456,19 @@ sheet_object_graph_user_config (SheetObject *so, SheetControl *sc)
 
 	wbcg = scg_get_wbcg (SHEET_CONTROL_GUI (sc));
 	
-	data = g_new0 (sheet_object_graph_user_config_t, 1);
+	data = g_new0 (gnm_sog_user_config_t, 1);
 	data->so = so;
 	data->wbc = WORKBOOK_CONTROL (wbcg);
  
 	closure = g_cclosure_new (G_CALLBACK (cb_update_graph), data,
-		(GClosureNotify) sheet_object_graph_user_config_free_data);
+		(GClosureNotify) gnm_sog_user_config_free_data);
 	
  	sheet_object_graph_guru (wbcg, sog->graph, closure);
 	g_closure_sink (closure);
 }
 
 static gboolean
-sheet_object_graph_set_sheet (SheetObject *so, Sheet *sheet)
+gnm_sog_set_sheet (SheetObject *so, Sheet *sheet)
 {
 	SheetObjectGraph *sog = SHEET_OBJECT_GRAPH (so);
 	if (sog->graph != NULL)
@@ -367,7 +477,7 @@ sheet_object_graph_set_sheet (SheetObject *so, Sheet *sheet)
 }
 
 static gboolean
-sheet_object_graph_remove_from_sheet (SheetObject *so)
+gnm_sog_remove_from_sheet (SheetObject *so)
 {
 	SheetObjectGraph *sog = SHEET_OBJECT_GRAPH (so);
 	if (sog->graph != NULL)
@@ -376,63 +486,79 @@ sheet_object_graph_remove_from_sheet (SheetObject *so)
 }
 
 static void
-sheet_object_graph_default_size (SheetObject const *so, double *w, double *h)
+gnm_sog_default_size (SheetObject const *so, double *w, double *h)
 {
 	*w = GO_CM_TO_PT ((double)12);
 	*h = GO_CM_TO_PT ((double)8);
 }
 
 static void
-sheet_object_graph_bounds_changed (SheetObject *so)
+gnm_sog_bounds_changed (SheetObject *so)
 {
+	SheetObjectGraph *sog = SHEET_OBJECT_GRAPH (so);
+
 	/* If it has not been realized there is no renderer yet */
-	if (SHEET_OBJECT_GRAPH (so)->renderer != NULL) {
+	if (sog->renderer != NULL) {
 		double coords [4];
 		sheet_object_position_pts_get (so, coords);
-		g_object_set (SHEET_OBJECT_GRAPH (so)->renderer,
-			"logical_width_pts",  fabs (coords[2] - coords[0]),
-			"logical_height_pts", fabs (coords[3] - coords[1]),
-			NULL);
+		gog_graph_set_size (sog->graph, fabs (coords[2] - coords[0]),
+				    fabs (coords[3] - coords[1]));
 	}
 }
 
 static void
-sheet_object_graph_class_init (GObjectClass *klass)
+gnm_sog_class_init (GObjectClass *klass)
 {
 	SheetObjectClass	*so_class  = SHEET_OBJECT_CLASS (klass);
 
 	parent_klass = g_type_class_peek_parent (klass);
 
 	/* Object class method overrides */
-	klass->finalize = sheet_object_graph_finalize;
+	klass->finalize = gnm_sog_finalize;
 
 	/* SheetObject class method overrides */
-	so_class->new_view	     = sheet_object_graph_new_view;
-	so_class->bounds_changed     = sheet_object_graph_bounds_changed;
-	so_class->populate_menu	     = sheet_object_graph_populate_menu;
-	so_class->read_xml_dom	     = sheet_object_graph_read_xml_dom;
-	so_class->write_xml_dom	     = sheet_object_graph_write_xml_dom;
-	so_class->write_xml_sax	     = sheet_object_graph_write_xml_sax;
-	so_class->copy               = sheet_object_graph_copy;
-	so_class->user_config        = sheet_object_graph_user_config;
-	so_class->assign_to_sheet    = sheet_object_graph_set_sheet;
-	so_class->remove_from_sheet  = sheet_object_graph_remove_from_sheet;
-	so_class->print		     = sheet_object_graph_print;
-	so_class->default_size	     = sheet_object_graph_default_size;
+	so_class->new_view		= gnm_sog_new_view;
+	so_class->bounds_changed	= gnm_sog_bounds_changed;
+	so_class->populate_menu		= gnm_sog_populate_menu;
+	so_class->read_xml_dom		= gnm_sog_read_xml_dom;
+	so_class->write_xml_sax		= gnm_sog_write_xml_sax;
+	so_class->prep_sax_parser	= gnm_sog_prep_sax_parser;
+	so_class->copy			= gnm_sog_copy;
+	so_class->user_config		= gnm_sog_user_config;
+	so_class->assign_to_sheet	= gnm_sog_set_sheet;
+	so_class->remove_from_sheet	= gnm_sog_remove_from_sheet;
+	so_class->print			= gnm_sog_print;
+	so_class->default_size		= gnm_sog_default_size;
 
 	so_class->rubber_band_directly = FALSE;
 }
 
 static void
-sheet_object_graph_init (GObject *obj)
+gnm_sog_init (GObject *obj)
 {
 	SheetObject *so = SHEET_OBJECT (obj);
 	so->anchor.direction = SO_DIR_DOWN_RIGHT;
 }
 
-GSF_CLASS (SheetObjectGraph, sheet_object_graph,
-	   sheet_object_graph_class_init, sheet_object_graph_init,
-	   SHEET_OBJECT_TYPE);
+static void
+sog_imageable_init (SheetObjectImageableIface *soi_iface)
+{
+	soi_iface->get_target_list = gnm_sog_get_target_list;
+	soi_iface->write_image	   = gnm_sog_write_image;
+}
+
+static void
+sog_exportable_init (SheetObjectExportableIface *soe_iface)
+{
+	soe_iface->get_target_list = gnm_sog_get_object_target_list;
+	soe_iface->write_object	   = gnm_sog_write_object;
+}
+
+GSF_CLASS_FULL (SheetObjectGraph, sheet_object_graph,
+		NULL, NULL, gnm_sog_class_init,NULL,
+		gnm_sog_init, SHEET_OBJECT_TYPE, 0,
+		GSF_INTERFACE (sog_imageable_init, SHEET_OBJECT_IMAGEABLE_TYPE) \
+		GSF_INTERFACE (sog_exportable_init, SHEET_OBJECT_EXPORTABLE_TYPE));
 
 /**
  * sheet_object_graph_new :
@@ -518,7 +644,7 @@ sheet_object_graph_guru (WorkbookControlGUI *wbcg, GogGraph *graph,
 			 GClosure *closure)
 {
 	GtkWidget *dialog = gog_guru (graph, GOG_DATA_ALLOCATOR (wbcg),
-		       GNM_CMD_CONTEXT (wbcg), wbcg_toplevel (wbcg),
+		       GO_CMD_CONTEXT (wbcg), wbcg_toplevel (wbcg),
 		       closure);
 	wbcg_edit_attach_guru (wbcg, dialog);
 	g_object_set_data_full (G_OBJECT (dialog),
