@@ -59,11 +59,9 @@ typedef struct {
 				      char const *type_name,	/* for versioning */
 				      XmlParseContext const *ctxt,
 				      xmlNodePtr	tree);
-	gboolean    (*write_xml_dom) (SheetObject const *so,
-				      XmlParseContext const *ctxt,
-				      xmlNodePtr	tree);
+	void      (*prep_sax_parser) (SheetObject *so,
+				      GsfXMLIn *xin, xmlChar const **attrs);
 	void        (*write_xml_sax) (SheetObject const *so, GsfXMLOut *output);
-	GsfXMLInNode *read_xml_sax;
 
 	/* Called with 0,0 set to the top, left corner of the object, and the
 	 * graphics context saved */
@@ -137,4 +135,26 @@ void so_vc_anchor_to_coords (SheetObjectViewContainer *sovc,
 void so_vc_coords_to_anchor (SheetObjectViewContainer *sovc,
 			     double const *input, SheetObjectAnchor *output);
 
+/***************************************************************************/
+
+struct _SheetObjectImageableIface {
+	GTypeInterface base;
+
+	GtkTargetList *(*get_target_list) (SheetObject const *so);
+	void           (*write_image)	  (SheetObject const *so, 
+					   const char *format,
+					   GsfOutput *output, GError **err);
+};
+
+/***************************************************************************/
+
+struct _SheetObjectExportableIface {
+	GTypeInterface base;
+
+	GtkTargetList *(*get_target_list) (SheetObject const *so);
+	void           (*write_object)	  (SheetObject const *so, 
+					   const char *format,
+					   GsfOutput *output, GError **err);
+};
+
 #endif /* GNUMERIC_SHEET_OBJECT_PRIV_H */
