@@ -3,7 +3,7 @@
 /*
  * sheet-object-widget.c: SheetObject wrappers for simple gtk widgets.
  *
- * Copyright (C) 2000-2003 Jody Goldberg (jody@gnome.org)
+ * Copyright (C) 2000-2005 Jody Goldberg (jody@gnome.org)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -116,7 +116,7 @@ static GSF_CLASS_FULL (SOWidgetFooView, so_widget_foo_view,
 #define SOW_CLASS(so)	 	     (SHEET_OBJECT_WIDGET_CLASS (G_OBJECT_GET_CLASS(so)))
 
 #define SOW_MAKE_TYPE(n1, n2, fn_config, fn_set_sheet, fn_clear_sheet,			\
-		      fn_copy, fn_write_dom, fn_read_dom, fn_write_sax,			\
+		      fn_copy, fn_read_dom, fn_write_sax,				\
 	              fn_get_property, fn_set_property, class_init_code)		\
 static void										\
 sheet_widget_ ## n1 ## _class_init (GObjectClass *object_class)				\
@@ -130,7 +130,6 @@ sheet_widget_ ## n1 ## _class_init (GObjectClass *object_class)				\
 	so_class->assign_to_sheet	= fn_set_sheet;					\
 	so_class->remove_from_sheet	= fn_clear_sheet;				\
 	so_class->copy			= fn_copy;					\
-	so_class->write_xml_dom		= fn_write_dom;					\
 	so_class->read_xml_dom		= fn_read_dom;					\
 	so_class->write_xml_sax		= fn_write_sax;					\
 	sow_class->create_widget     	= &sheet_widget_ ## n1 ## _create_widget;	\
@@ -166,19 +165,6 @@ sax_write_dep (GsfXMLOut *output, GnmDependent const *dep, char const *id)
 	}
 }
 
-static void
-dom_write_dep (xmlNodePtr tree, GnmDependent const *dep, char const *id)
-{
-	if (dep->expression != NULL) {
-		GnmParsePos pos;
-		char *val = gnm_expr_as_string (dep->expression,
-			parse_pos_init_sheet (&pos, dep->sheet),
-			gnm_expr_conventions_default);
-		xml_node_set_cstr (tree, id, val);
-		g_free (val);
-	}
-}
-
 static void
 read_dep (GnmDependent *dep, char const *name,
 	  xmlNodePtr tree, XmlParseContext const *context)
@@ -208,6 +194,7 @@ sheet_object_widget_new_view (SheetObject *so, SheetObjectViewContainer *contain
 		"widget", view_widget,
 		"size_pixels", FALSE,
 		NULL);
+	/* g_warning ("%p is widget for so %p", view_widget, so);*/
 	gtk_widget_show_all (view_widget);
 	foo_canvas_item_hide (view_item);
 	return gnm_pane_widget_register (so, view_widget, view_item);
@@ -295,18 +282,6 @@ sheet_widget_frame_write_xml_sax (SheetObject const *so, GsfXMLOut *output)
 	gsf_xml_out_add_cstr (output, "Label", swf->label);
 }
 
-static gboolean
-sheet_widget_frame_write_xml_dom (SheetObject const 	*so,
-				  XmlParseContext const *context,
-				  xmlNodePtr tree)
-{
-	SheetWidgetFrame *swf = SHEET_WIDGET_FRAME (so);
-
-	xml_node_set_cstr (tree, "Label", swf->label);
-
-	return FALSE;
-}
-
 static gboolean
 sheet_widget_frame_read_xml_dom (SheetObject *so, char const *typename,
 				 XmlParseContext const *context,
@@ -467,7 +442,6 @@ SOW_MAKE_TYPE (frame, Frame,
 	       NULL,
 	       NULL,
 	       &sheet_widget_frame_copy,
-	       &sheet_widget_frame_write_xml_dom,
 	       &sheet_widget_frame_read_xml_dom,
 	       &sheet_widget_frame_write_xml_sax,
 	       NULL,
@@ -543,19 +517,6 @@ sheet_widget_button_write_xml_sax (SheetObject const *so, GsfXMLOut *output)
 	gsf_xml_out_add_cstr (output, "Label", swb->label);
 }
 
-static gboolean
-sheet_widget_button_write_xml_dom (SheetObject const	 *so,
-				   XmlParseContext const *context,
-				   xmlNodePtr tree)
-{
-	// FIXME: markup
-	SheetWidgetButton *swb = SHEET_WIDGET_BUTTON (so);
-
-	xml_node_set_cstr (tree, "Label", swb->label);
-
-	return FALSE;
-}
-
 static gboolean
 sheet_widget_button_read_xml_dom (SheetObject *so, char const *typename,
 				  XmlParseContext const *context,
@@ -665,7 +626,6 @@ SOW_MAKE_TYPE (button, Button,
 	       NULL,
 	       NULL,
 	       sheet_widget_button_copy,
-	       sheet_widget_button_write_xml_dom,
 	       sheet_widget_button_read_xml_dom,
 	       sheet_widget_button_write_xml_sax,
 	       sheet_widget_button_get_property,
@@ -1046,22 +1006,6 @@ sheet_widget_adjustment_write_xml_sax (SheetObject const *so, GsfXMLOut *output)
 	sax_write_dep (output, &swa->dep, "Input");
 }
 
-static gboolean
-sheet_widget_adjustment_write_xml_dom (SheetObject const     *so,
-				       XmlParseContext const *context,
-				       xmlNodePtr tree)
-{
-	SheetWidgetAdjustment *swa = SHEET_WIDGET_ADJUSTMENT (so);
-
-	xml_node_set_double (tree, "Min", swa->adjustment->lower, 2);
-	xml_node_set_double (tree, "Max", swa->adjustment->upper, 2); /* allow scrolling to max */
-	xml_node_set_double (tree, "Inc", swa->adjustment->step_increment, 2);
-	xml_node_set_double (tree, "Page", swa->adjustment->page_increment, 2);
-	xml_node_set_double  (tree, "Value", swa->adjustment->value, 2);
-	dom_write_dep (tree, &swa->dep, "Input");
-	return FALSE;
-}
-
 static gboolean
 sheet_widget_adjustment_read_xml_dom (SheetObject *so, char const *typename,
 				      XmlParseContext const *context,
@@ -1121,7 +1065,6 @@ SOW_MAKE_TYPE (adjustment, Adjustment,
 	       &sheet_widget_adjustment_set_sheet,
 	       &sheet_widget_adjustment_clear_sheet,
 	       &sheet_widget_adjustment_copy,
-	       &sheet_widget_adjustment_write_xml_dom,
 	       &sheet_widget_adjustment_read_xml_dom,
 	       &sheet_widget_adjustment_write_xml_sax,
 	       NULL,
@@ -1657,18 +1600,6 @@ sheet_widget_checkbox_write_xml_sax (SheetObject const *so, GsfXMLOut *output)
 	sax_write_dep (output, &swc->dep, "Input");
 }
 
-static gboolean
-sheet_widget_checkbox_write_xml_dom (SheetObject const 	   *so,
-				     XmlParseContext const *context,
-				     xmlNodePtr tree)
-{
-	SheetWidgetCheckbox *swc = SHEET_WIDGET_CHECKBOX (so);
-	xml_node_set_cstr (tree, "Label", swc->label);
-	xml_node_set_int  (tree, "Value", swc->value);
-	dom_write_dep (tree, &swc->dep, "Input");
-	return FALSE;
-}
-
 static gboolean
 sheet_widget_checkbox_read_xml_dom (SheetObject *so, char const *typename,
 				    XmlParseContext const *context,
@@ -1729,7 +1660,6 @@ SOW_MAKE_TYPE (checkbox, Checkbox,
 	       &sheet_widget_checkbox_set_sheet,
 	       &sheet_widget_checkbox_clear_sheet,
 	       &sheet_widget_checkbox_copy,
-	       &sheet_widget_checkbox_write_xml_dom,
 	       &sheet_widget_checkbox_read_xml_dom,
 	       &sheet_widget_checkbox_write_xml_sax,
 	       &sheet_widget_checkbox_get_property,
@@ -1960,7 +1890,6 @@ SOW_MAKE_TYPE (radio_button, RadioButton,
 	       NULL,
 	       NULL,
 	       NULL,
-	       NULL,
 	       &sheet_widget_radio_button_get_property,
 	       &sheet_widget_radio_button_set_property,
 	       {
@@ -2112,17 +2041,6 @@ sheet_widget_list_base_write_xml_sax (SheetObject const *so, GsfXMLOut *output)
 	sax_write_dep (output, &swl->output_dep, "Output");
 }
 
-static gboolean
-sheet_widget_list_base_write_xml_dom (SheetObject const 	*so,
-				      XmlParseContext const *context,
-				      xmlNodePtr tree)
-{
-	SheetWidgetListBase *swl = SHEET_WIDGET_LIST_BASE (so);
-	dom_write_dep (tree, &swl->content_dep, "Content");
-	dom_write_dep (tree, &swl->output_dep, "Output");
-	return FALSE;
-}
-
 static gboolean
 sheet_widget_list_base_read_xml_dom (SheetObject *so, char const *typename,
 				     XmlParseContext const *context,
@@ -2151,7 +2069,6 @@ SOW_MAKE_TYPE (list_base, ListBase,
 	       &sheet_widget_list_base_set_sheet,
 	       &sheet_widget_list_base_clear_sheet,
 	       NULL,
-	       &sheet_widget_list_base_write_xml_dom,
 	       &sheet_widget_list_base_read_xml_dom,
 	       &sheet_widget_list_base_write_xml_sax,
 	       NULL,
