@@ -13,13 +13,28 @@ typedef enum {
 	GNM_SHEET_XLM
 } GnmSheetType;
 
-struct _Sheet {
-	GObject	base; /* not really used yet */
+typedef enum {
+	GNM_SHEET_VISIBILITY_VISIBLE,
+	GNM_SHEET_VISIBILITY_HIDDEN,
+	GNM_SHEET_VISIBILITY_VERY_HIDDEN
+} GnmSheetVisibility;
+GType gnm_sheet_visibility_get_type (void);
+#define GNM_SHEET_VISIBILITY_TYPE (gnm_sheet_visibility_get_type ())
 
-	int         signature;
+struct _Sheet {
+	GObject	base;
 
 	int         index_in_wb;
 	Workbook    *workbook;
+	gboolean    being_invalidated;
+	struct {
+		/* An alternating list of ref'd names and expressions.  */
+		GSList *name_exprs;
+		/* An alternating list of deps and ref'd expressions.  */
+		GSList *dep_exprs;
+		/* A list of deps to relink.  */
+		GSList *relink;
+	} revive;
 
 	GPtrArray   *sheet_views;
 
@@ -38,7 +53,7 @@ struct _Sheet {
 
 	double      last_zoom_factor_used;
 
-	GList       *sheet_objects;	/* List of objects in this sheet */
+	GSList      *sheet_objects;	/* List of objects in this sheet */
 	GnmCellPos   max_object_extent;
 
 	gboolean    pristine;
@@ -52,11 +67,12 @@ struct _Sheet {
 	gboolean    hide_col_header;
 	gboolean    hide_row_header;
 	gboolean    is_protected;
-	gboolean    is_visible;
+	GnmSheetVisibility visibility;
 
 	gboolean    display_outlines;
 	gboolean    outline_symbols_below;
 	gboolean    outline_symbols_right;
+	gboolean    text_is_rtl;
 
 	gboolean    has_filtered_rows;
 
@@ -76,13 +92,17 @@ struct _Sheet {
 	GnmColor	 *tab_color;
 	GnmColor	 *tab_text_color;
 	GnmSheetType	  sheet_type;
+	GnmExprConventions const *convs;
 
 	/* This needs to move elsewhere and get shared.  */
 	PangoContext *context;
 };
 
-#define SHEET_SIGNATURE 0x12349876
-#define IS_SHEET(x) (((x) != NULL) && ((x)->signature == SHEET_SIGNATURE))
+#define GNM_SHEET_TYPE	(gnm_sheet_get_type ())
+#define SHEET(obj)     (G_TYPE_CHECK_INSTANCE_CAST ((obj), GNM_SHEET_TYPE, Sheet))
+#define IS_SHEET(o)    (G_TYPE_CHECK_INSTANCE_TYPE ((o), GNM_SHEET_TYPE))
+#define GNM_SHEET_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), GNM_SHEET_TYPE, GnmSheetClass))
+GType     gnm_sheet_get_type	 (void);
 
 Sheet    *sheet_new		 (Workbook *wb, char const *name);
 Sheet    *sheet_new_with_type	 (Workbook *wb, char const *name,
@@ -90,14 +110,6 @@ Sheet    *sheet_new_with_type	 (Workbook *wb, char const *name,
 Sheet    *sheet_dup		 (Sheet const *source_sheet);
 void      sheet_destroy		 (Sheet *sheet);
 void      sheet_destroy_contents (Sheet *sheet);
-void      sheet_rename		 (Sheet *sheet, char const *new_name);
-void	  sheet_set_tab_color	 (Sheet *sheet, GnmColor *tab_color,
-				  GnmColor *text_color);
-
-void      sheet_set_zoom_factor	 (Sheet *sheet, double factor,
-				  gboolean force, gboolean respan);
-
-void	  sheet_set_visibility	 (Sheet *sheet, gboolean visible);
 
 /* GnmCell management */
 GnmCell  *sheet_cell_get	 (Sheet const *sheet, int col, int row);
@@ -208,17 +220,18 @@ void     sheet_colrow_gutter 	     (Sheet *sheet,
 
 gboolean sheet_range_splits_array    (Sheet const *sheet,
 				      GnmRange const *r, GnmRange const *ignore,
-				      GnmCmdContext *cc, char const *cmd);
+				      GOCmdContext *cc, char const *cmd);
 gboolean sheet_range_splits_region   (Sheet const *sheet,
 				      GnmRange const *r, GnmRange const *ignore,
-				      GnmCmdContext *cc, char const *cmd);
+				      GOCmdContext *cc, char const *cmd);
 gboolean sheet_ranges_split_region   (Sheet const *sheet, GSList const *ranges,
-				      GnmCmdContext *cc, char const *cmd);
+				      GOCmdContext *cc, char const *cmd);
 gboolean sheet_range_contains_region (Sheet const *sheet, GnmRange const *r,
-				      GnmCmdContext *cc, char const *cmd);
+				      GOCmdContext *cc, char const *cmd);
 void	 sheet_range_bounding_box    (Sheet const *sheet, GnmRange *r);
 
 /* Redraw */
+#define sheet_is_visible(_sheet) ((_sheet)->visibility == GNM_SHEET_VISIBILITY_VISIBLE)
 void     sheet_redraw_all       (Sheet const *sheet, gboolean header);
 void     sheet_redraw_cell      (GnmCell const *cell);
 void     sheet_redraw_range     (Sheet const *sheet, GnmRange const *r);
@@ -233,10 +246,6 @@ void	 sheet_flag_recompute_spans	(Sheet const *s);
 void	 sheet_update_only_grid		(Sheet const *s);
 void     sheet_update                   (Sheet const *s);
 void	 sheet_scrollbar_config		(Sheet const *s);
-void	 sheet_toggle_show_formula	(Sheet       *s);
-void	 sheet_toggle_hide_zeros	(Sheet       *s);
-void     sheet_adjust_preferences	(Sheet const *s,
-					 gboolean redraw, gboolean resize);
 
 void     sheet_set_dirty	(Sheet *sheet, gboolean is_dirty);
 gboolean sheet_is_pristine	(Sheet const *sheet);
@@ -257,11 +266,12 @@ void	     sheet_cell_set_text    (GnmCell *cell, char const *str,
 GnmValue const *sheet_cell_get_value(Sheet *sheet, int col, int row);
 void	     sheet_range_set_text   (GnmParsePos const *pos, GnmRange const *r, char const *str);
 void	     sheet_apply_style	    (Sheet  *sheet, GnmRange const *range, GnmStyle *mstyle);
+void	     sheet_apply_border	    (Sheet  *sheet, GnmRange const *range, GnmBorder **borders);
 void	     sheet_queue_respan     (Sheet const *sheet, int start_row, int end_row);
 void	     sheet_range_calc_spans (Sheet *sheet, GnmRange const *r, SpanCalcFlags flags);
 void	     sheet_cell_calc_span   (GnmCell *cell, SpanCalcFlags flags);
 
-void	     sheet_adjust_outline_dir (Sheet *sheet, gboolean is_cols);
+void	     sheet_set_outline_direction (Sheet *sheet, gboolean is_cols);
 
 /* Implementation for commands, no undo */
 struct _GnmRelocUndo {
@@ -271,18 +281,18 @@ struct _GnmRelocUndo {
 
 gboolean  sheet_insert_cols (Sheet *sheet,
 			     int col, int count, ColRowStateList *states,
-			     GnmRelocUndo *reloc_storage, GnmCmdContext *cc);
+			     GnmRelocUndo *reloc_storage, GOCmdContext *cc);
 gboolean  sheet_delete_cols (Sheet *sheet,
 			     int col, int count, ColRowStateList *states,
-			     GnmRelocUndo *reloc_storage, GnmCmdContext *cc);
+			     GnmRelocUndo *reloc_storage, GOCmdContext *cc);
 gboolean  sheet_insert_rows (Sheet *sheet,
 			     int row, int count, ColRowStateList *states,
-			     GnmRelocUndo *reloc_storage, GnmCmdContext *cc);
+			     GnmRelocUndo *reloc_storage, GOCmdContext *cc);
 gboolean  sheet_delete_rows (Sheet *sheet,
 			     int row, int count, ColRowStateList *states,
-			     GnmRelocUndo *reloc_storage, GnmCmdContext *cc);
+			     GnmRelocUndo *reloc_storage, GOCmdContext *cc);
 void      sheet_move_range   (GnmExprRelocateInfo const *rinfo,
-			      GnmRelocUndo *reloc_storage, GnmCmdContext *cc);
+			      GnmRelocUndo *reloc_storage, GOCmdContext *cc);
 
 
 typedef enum {
@@ -299,10 +309,8 @@ typedef enum {
 void  sheet_clear_region (Sheet *sheet,
 			  int start_col, int start_row,
 			  int end_col, int end_row,
-			  int clear_flags, GnmCmdContext *cc);
+			  int clear_flags, GOCmdContext *cc);
 
-void	sheet_attach_view (Sheet *sheet, SheetView *sv);
-void    sheet_detach_view (SheetView *sv);
 SheetView *sheet_get_view (Sheet const *sheet, WorkbookView const *wbv);
 
 #define SHEET_FOREACH_VIEW(sheet, view, code)					\
