@@ -24,6 +24,10 @@
 #include <goffice/utils/go-file.h>
 #include <gsf/gsf-utils.h>
 #include <string.h>
+#ifdef WIN32
+#define POPT_STATIC
+#endif
+#include <popt.h>
 
 static gboolean ssconvert_show_version = FALSE;
 static gboolean ssconvert_list_exporters = FALSE;
@@ -33,9 +37,6 @@ static char const *ssconvert_import_encoding = NULL;
 static char const *ssconvert_import_id = NULL;
 static char const *ssconvert_export_id = NULL;
 
-#ifdef WIN32
-#define POPT_STATIC
-#endif
 struct poptOption
 ssconvert_popt_options[] = {
 	{ NULL, '\0', POPT_ARG_INTL_DOMAIN, (char *)GETTEXT_PACKAGE, 0, NULL, NULL },
@@ -183,9 +184,7 @@ main (int argc, char const *argv [])
 	int		 res = 0;
 	GOCmdContext	*cc;
 	poptContext ctx;
-	gchar **args;
-	
-	args = go_shell_argv_to_glib_encoding (argc, argv);
+	gchar const **args = go_shell_argv_to_glib_encoding (argc, argv);
 
 	gnm_pre_parse_init (args[0]);
 
@@ -194,7 +193,7 @@ main (int argc, char const *argv [])
 	ssconvert_popt_options[3].arg = &gnumeric_data_dir;
 #endif
 
-	ctx = poptGetContext (NULL, argc, (gchar const **) args, ssconvert_popt_options, 0);
+	ctx = poptGetContext (NULL, argc, args, ssconvert_popt_options, 0);
 	while (poptGetNextOpt (ctx) > 0)
 		;
 
@@ -233,14 +232,7 @@ main (int argc, char const *argv [])
 	poptFreeContext (ctx);
 	g_object_unref (cc);
 	gnm_shutdown ();
-
-	if (argv != (gchar const **) args) {
-		gint i;
-
-		for (i = 0; i < argc; ++i)
-			g_free (args[i]);
-		g_free (args);
-	}
+	go_shell_argv_to_glib_encoding_free ();
 
 	return res;
 }
