@@ -102,8 +102,7 @@ int stf_parse_options_fixed_splitpositions_nth         (StfParseOptions_t *parse
 GPtrArray	*stf_parse_general			(StfParseOptions_t *parseoptions,
 							 GStringChunk *lines_chunk,
 							 char const *data,
-							 char const *data_end,
-							 int maxlines);
+							 char const *data_end);
 void		 stf_parse_general_free			(GPtrArray *lines);
 GPtrArray	*stf_parse_lines			(StfParseOptions_t *parseoptions,
 							 GStringChunk *lines_chunk,
