@@ -12,9 +12,8 @@
 #include "style.h"
 #include "style-font.h"
 
-#include "format.h"
+#include "gnm-format.h"
 #include "style-color.h"
-#include "global-gnome-font.h"
 #include "application.h"
 #include "sheet.h"
 #include "cell.h"
@@ -38,8 +37,6 @@ static GHashTable *style_font_negative_hash;
 double gnumeric_default_font_width;
 static char *gnumeric_default_font_name;
 static double gnumeric_default_font_size;
-static PangoFontFamily	**pango_families;
-static GStringChunk	 *size_names;
 
 /**
  * get_substitute_font
@@ -185,10 +182,7 @@ style_font_new_simple (PangoContext *context,
 			}
 		}
 
-		font->gnome_print_font = gnm_font_find_closest_from_weight_slant (font_name,
-			bold ? GNOME_FONT_BOLD : GNOME_FONT_REGULAR, italic, size_pts);
-
-		font->pango.font_descr = pango_font_describe (font->pango.font);
+		font->pango.font_descr = pango_font_description_copy (desc);
 
 		metrics = pango_font_get_metrics (font->pango.font,
 				gtk_get_default_language ());
@@ -313,10 +307,6 @@ style_font_unref (GnmFont *sf)
 		pango_font_description_free (sf->pango.font_descr);
 		sf->pango.font_descr = NULL;
 	}
-	if (sf->gnome_print_font != NULL) {
-		gnome_font_unref (sf->gnome_print_font);
-		sf->gnome_print_font = NULL;
-	}
 	g_hash_table_remove (style_font_hash, sf);
 	g_free (sf->font_name);
 	g_free (sf);
@@ -352,15 +342,6 @@ style_font_hash_func (gconstpointer v)
 	return k->size_pts + g_str_hash (k->font_name);
 }
 
-static int
-compare_family_pointers_by_name (gconstpointer a, gconstpointer b)
-{
-	PangoFontFamily * const * const fa = a;
-	PangoFontFamily * const * const fb = b;
-	return g_utf8_collate (pango_font_family_get_name (*fa),
-			       pango_font_family_get_name (*fb));
-}
-
 /**
  * gnm_pango_context_get :
  *
@@ -395,7 +376,6 @@ font_init (void)
 {
 	PangoContext *context;
 	GnmFont *gnumeric_default_font = NULL;
-	int n_families, i;
 
 	gnumeric_default_font_name = g_strdup (gnm_app_prefs->default_font.name);
 	gnumeric_default_font_size = gnm_app_prefs->default_font.size;
@@ -433,29 +413,6 @@ font_init (void)
 
 	gnumeric_default_font_width = gnumeric_default_font->approx_width.pts.digit;
 	style_font_unref (gnumeric_default_font);
-
-	size_names = g_string_chunk_new (128);
-
-	pango_context_list_families (context,
-		&pango_families, &n_families);
-	qsort (pango_families, n_families, sizeof (*pango_families),
-	       compare_family_pointers_by_name);
-
-	for (i = 0 ; i < n_families ; i++)
-		gnumeric_font_family_list = g_list_prepend (
-			gnumeric_font_family_list,
-			(gpointer) pango_font_family_get_name (pango_families[i]));
-
-	gnumeric_font_family_list = g_list_reverse (gnumeric_font_family_list);
-
-	for (i = 0; gnumeric_point_sizes [i] != 0; i++){
-		char buffer[4 * sizeof (int)];
-		sprintf (buffer, "%d", gnumeric_point_sizes [i]);
-		gnumeric_point_size_list = g_list_prepend (
-			gnumeric_point_size_list,
-			g_string_chunk_insert (size_names, buffer));
-	}
-
 	g_object_unref (G_OBJECT (context));
 }
 
@@ -464,21 +421,11 @@ font_shutdown (void)
 {
 	g_free (gnumeric_default_font_name);
 	gnumeric_default_font_name = NULL;
-
-	g_free (pango_families);
-	pango_families = NULL;
-	g_list_free (gnumeric_font_family_list);
-	gnumeric_font_family_list = NULL;
-	g_list_free (gnumeric_point_size_list);
-	gnumeric_point_size_list = NULL;
-	g_string_chunk_free (size_names);
-	size_names = NULL;
 }
 
 void
 style_init (void)
 {
-	number_format_init ();
 	style_font_hash = g_hash_table_new (style_font_hash_func,
 					    style_font_equal);
 	style_font_negative_hash = g_hash_table_new (style_font_hash_func,
@@ -507,7 +454,6 @@ void
 style_shutdown (void)
 {
 	font_shutdown ();
-	number_format_shutdown ();
 	{
 		/* Make a list of the fonts, then unref them.  */
 		GSList *fonts = NULL, *tmp;
@@ -541,22 +487,22 @@ required_updates_for_style (GnmStyle const *style)
 	SpanCalcFlags res = SPANCALC_SIMPLE;
 
 	gboolean const row_height =
-	     mstyle_is_element_set (style, MSTYLE_FONT_SIZE) ||
-	     mstyle_is_element_set (style, MSTYLE_WRAP_TEXT) ||
-	     mstyle_is_element_set (style, MSTYLE_ROTATION);
+		gnm_style_is_element_set (style, MSTYLE_FONT_SIZE) ||
+		gnm_style_is_element_set (style, MSTYLE_WRAP_TEXT) ||
+		gnm_style_is_element_set (style, MSTYLE_ROTATION) ||
+		gnm_style_is_element_set (style, MSTYLE_FONT_SCRIPT);
 	gboolean const size_change = row_height ||
-	     mstyle_is_element_set (style, MSTYLE_FONT_NAME) ||
-	     mstyle_is_element_set (style, MSTYLE_FONT_BOLD) ||
-	     mstyle_is_element_set (style, MSTYLE_FONT_ITALIC);
+		gnm_style_is_element_set (style, MSTYLE_FONT_NAME) ||
+		gnm_style_is_element_set (style, MSTYLE_FONT_BOLD) ||
+		gnm_style_is_element_set (style, MSTYLE_FONT_ITALIC);
 	gboolean const format_change =
-	    (mstyle_is_element_set (style, MSTYLE_FORMAT) ||
-	     mstyle_is_element_set (style, MSTYLE_INDENT) ||
-	     mstyle_is_element_set (style, MSTYLE_ALIGN_H) ||
-	     mstyle_is_element_set (style, MSTYLE_ALIGN_V) ||
-	     mstyle_is_element_set (style, MSTYLE_FONT_STRIKETHROUGH) ||
-	     mstyle_is_element_set (style, MSTYLE_FONT_UNDERLINE) ||
-	     mstyle_is_element_set (style, MSTYLE_COLOR_FORE) ||
-	     mstyle_is_element_set (style, MSTYLE_ROTATION));
+		gnm_style_is_element_set (style, MSTYLE_FORMAT) ||
+		gnm_style_is_element_set (style, MSTYLE_INDENT) ||
+		gnm_style_is_element_set (style, MSTYLE_ALIGN_H) ||
+		gnm_style_is_element_set (style, MSTYLE_ALIGN_V) ||
+		gnm_style_is_element_set (style, MSTYLE_FONT_STRIKETHROUGH) ||
+		gnm_style_is_element_set (style, MSTYLE_FONT_UNDERLINE) ||
+		gnm_style_is_element_set (style, MSTYLE_FONT_COLOR);
 
 	if (row_height)
 		res |= SPANCALC_ROW_HEIGHT;
@@ -573,10 +519,10 @@ required_updates_for_style (GnmStyle const *style)
  * Select the appropriate horizontal alignment depending on the style and cell
  * value.
  */
-StyleHAlignFlags
+GnmHAlign
 style_default_halign (GnmStyle const *mstyle, GnmCell const *c)
 {
-	StyleHAlignFlags align = mstyle_get_align_h (mstyle);
+	GnmHAlign align = gnm_style_get_align_h (mstyle);
 	GnmValue *v;
 
 	if (align != HALIGN_GENERAL)
@@ -594,10 +540,12 @@ style_default_halign (GnmStyle const *mstyle, GnmCell const *c)
 			return HALIGN_CENTER;
 
 		case VALUE_INTEGER:
-		case VALUE_FLOAT:
-			if (mstyle_get_rotation (mstyle) > 0)
+		case VALUE_FLOAT: {
+			double a = gnm_style_get_rotation (mstyle);
+			if (a > 0 && a < 180)
 				return HALIGN_LEFT;
 			return HALIGN_RIGHT;
+		}
 
 		case VALUE_ARRAY:
 			/* Tail recurse into the array */
@@ -607,59 +555,9 @@ style_default_halign (GnmStyle const *mstyle, GnmCell const *c)
 			}
 
 		default:
-			if (mstyle_get_rotation (mstyle) < 0)
+			if (gnm_style_get_rotation (mstyle) > 180)
 				return HALIGN_RIGHT;
 			return HALIGN_LEFT;
 		}
 	return HALIGN_RIGHT;
 }
-
-/**
- * gnm_font_find_closest_from_weight_slant :
- *
- * A wrapper around gnome-print because it is stupid.
- * At least this will warn us when it is stupid
- **/
-GnomeFont *
-gnm_font_find_closest_from_weight_slant (const guchar *family, 
-					 GnomeFontWeight weight, 
-					 gboolean italic, gdouble size)
-{
-	GnomeFont *font;
-	guchar const *fam;
-	guchar   *name;
-
-	g_return_val_if_fail (family != NULL, NULL);
-
-	while (1) {
-		font = gnome_font_find_closest_from_weight_slant 
-			(family, weight, italic, size);
-		fam = gnome_font_get_family_name  (font);
-		if (fam != NULL && g_ascii_strcasecmp (family, fam) == 0)
-			return font;
-
-		name = gnome_font_get_full_name (font);
-		g_warning ("GnomePrint: Requested %s but using %s (%s)", 
-			   family, fam, name);
-		g_free (name);
-
-		/* put in some fallbacks */
-		if (!g_ascii_strcasecmp (family, "Sans"))
-			family = "Sans Regular";
-		else if (!g_ascii_strcasecmp (family, "Helvetica"))
-			family = "Sans";
-		else if (!g_ascii_strcasecmp (family, "Albany"))
-			family = "Arial";
-		/* one of the arials */
-		else if (!g_ascii_strncasecmp (family, "Arial ", 6))
-			family = "Arial";
-		else if (!g_ascii_strcasecmp (family, "Arial"))
-			family = "Sans";
-		else
-			return font;
-
-		g_warning ("Trying to fallback to '%s'", family);
-	}
-	/* notreached */
-	return font;
-}
