@@ -19,8 +19,8 @@
 --
 -- You should have received a copy of the GNU General Public License
 -- along with GLPK; see the file COPYING. If not, write to the Free
--- Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
--- 02111-1307, USA.
+-- Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+-- 02110-1301  USA.
 ----------------------------------------------------------------------*/
 
 #include <limits.h>
@@ -76,7 +76,7 @@ int ies_default_tagx(IESITEM *item)
                case LPX_UP:
                   tagx = LPX_NU; break;
                case LPX_DB:
-                  tagx = (gnumabs(item->lb) <= gnumabs(item->ub) ? LPX_NL :
+                  tagx = (gnm_abs(item->lb) <= gnm_abs(item->ub) ? LPX_NL :
                      LPX_NF);
                   break;
                case LPX_FX:
