@@ -1,11 +1,11 @@
 /* glplpx1.c (problem creating and modifying routines) */
 
 /*----------------------------------------------------------------------
--- Copyright (C) 2000, 2001, 2002 Andrew Makhorin <mao@mai2.rcnet.ru>,
---               Department for Applied Informatics, Moscow Aviation
---               Institute, Moscow, Russia. All rights reserved.
+-- Copyright (C) 2000, 2001, 2002, 2003 Andrew Makhorin, Department
+-- for Applied Informatics, Moscow Aviation Institute, Moscow, Russia.
+-- All rights reserved. E-mail: <mao@mai2.rcnet.ru>.
 --
--- This file is a part of GLPK (GNU Linear Programming Kit).
+-- This file is part of GLPK (GNU Linear Programming Kit).
 --
 -- GLPK is free software; you can redistribute it and/or modify it
 -- under the terms of the GNU General Public License as published by
@@ -19,13 +19,14 @@
 --
 -- You should have received a copy of the GNU General Public License
 -- along with GLPK; see the file COPYING. If not, write to the Free
--- Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
--- 02111-1307, USA.
+-- Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+-- 02110-1301  USA.
 ----------------------------------------------------------------------*/
 
 #include <ctype.h>
 #include <math.h>
 #include <string.h>
+#include "glplib.h"
 #include "glplpx.h"
 
 /*----------------------------------------------------------------------
@@ -335,8 +336,12 @@ void lpx_add_cols(LPX *lp, int ncs)
 
 int lpx_check_name(char *name)
 {     int t;
+#if 1
+      /* empty name is not allowed */
+      if (name[0] == '\0') return 1;
+#endif
       for (t = 0; name[t] != '\0'; t++)
-         if (t == 255 || !isgraph(name[t])) return 1;
+         if (t == 255 || !isgraph((unsigned char)name[t])) return 1;
       return 0;
 }
 
@@ -494,7 +499,7 @@ void lpx_set_row_bnds(LPX *lp, int i, int typx, gnm_float lb, gnm_float ub)
          case LPX_DB:
             lp->lb[i] = lb * lp->rs[i], lp->ub[i] = ub * lp->rs[i];
             if (lp->tagx[i] != LPX_BS)
-               lp->tagx[i] = (gnumabs(lb) <= gnumabs(ub) ? LPX_NL : LPX_NU);
+               lp->tagx[i] = (gnm_abs(lb) <= gnm_abs(ub) ? LPX_NL : LPX_NU);
             break;
          case LPX_FX:
             lp->lb[i] = lp->ub[i] = lb * lp->rs[i];
@@ -566,7 +571,7 @@ void lpx_set_col_bnds(LPX *lp, int j, int typx, gnm_float lb, gnm_float ub)
          case LPX_DB:
             lp->lb[j] = lb / lp->rs[j], lp->ub[j] = ub / lp->rs[j];
             if (lp->tagx[j] != LPX_BS)
-               lp->tagx[j] = (gnumabs(lb) <= gnumabs(ub) ? LPX_NL : LPX_NU);
+               lp->tagx[j] = (gnm_abs(lb) <= gnm_abs(ub) ? LPX_NL : LPX_NU);
             break;
          case LPX_FX:
             lp->lb[j] = lp->ub[j] = lb / lp->rs[j];
@@ -1335,7 +1340,7 @@ void lpx_del_items(LPX *lp)
 -- parameter lp points to, freeing all the memory allocated to it. */
 
 void lpx_delete_prob(LPX *lp)
-{     delete_pool(lp->pool);
+{     dmp_delete_pool(lp->pool);
       ufree(lp->buf);
       ufree(lp->name);
       ufree(lp->typx);
