@@ -21,19 +21,14 @@ WorkbookControl *wb_control_wrapper_new (WorkbookControl *wbc,
 					 void *extra);
 
 void wb_control_update_title	     (WorkbookControl *wbc);
-void wb_control_prefs_update	     (WorkbookControl *wbc);
 void wb_control_style_feedback	     (WorkbookControl *wbc, GnmStyle const *changes);
-void wb_control_zoom_feedback	     (WorkbookControl *wbc);
 void wb_control_edit_line_set        (WorkbookControl *wbc, char const *text);
 void wb_control_selection_descr_set  (WorkbookControl *wbc, char const *text);
 void wb_control_auto_expr_value	     (WorkbookControl *wbc);
 
 void wb_control_sheet_add	     (WorkbookControl *wbc, SheetView *sv);
 void wb_control_sheet_remove	     (WorkbookControl *wbc, Sheet *sheet);
-void wb_control_sheet_rename	     (WorkbookControl *wbc, Sheet *sheet);
 void wb_control_sheet_focus	     (WorkbookControl *wbc, Sheet *sheet);
-void wb_control_sheet_move	     (WorkbookControl *wbc, Sheet *sheet,
-				      int new_pos);
 void wb_control_sheet_remove_all     (WorkbookControl *wbc);
 
 void wb_control_undo_redo_truncate   (WorkbookControl *wbc, int n, gboolean is_undo);
@@ -68,7 +63,6 @@ enum {
 #define MS_GURU_MENU_ITEMS \
     (MS_PRINT_SETUP | MS_SEARCH_REPLACE | MS_DEFINE_NAME | MS_CONSOLIDATE)
 
-void wb_control_menu_state_sheet_prefs	(WorkbookControl *wbc, Sheet const *s);
 void wb_control_menu_state_sheet_count	(WorkbookControl *wbc);
 void wb_control_menu_state_update	(WorkbookControl *wbc, int flags);
 void wb_control_update_action_sensitivity (WorkbookControl *wbc);
