@@ -16,11 +16,10 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  */
 #include <gnumeric-config.h>
-#include <glib/gi18n.h>
 #include "gnumeric.h"
 #include "workbook-view.h"
 
@@ -32,9 +31,8 @@
 #include "sheet-merge.h"
 #include "sheet-style.h"
 #include "str.h"
-#include "format.h"
+#include "gnm-format.h"
 #include "func.h"
-#include "file.h"
 #include "expr.h"
 #include "expr-name.h"
 #include "expr-impl.h"
@@ -45,23 +43,24 @@
 #include "position.h"
 #include "cell.h"
 #include "gutils.h"
-#include "io-context.h"
 #include "command-context.h"
+#include "auto-format.h"
 
+#include <goffice/app/file.h>
+#include <goffice/app/io-context.h>
 #include <gsf/gsf.h>
 #include <gsf/gsf-impl-utils.h>
 #include <gsf/gsf-output-stdio.h>
 #include <gsf/gsf-input.h>
+#include <glib/gi18n.h>
+#include <glib/gstdio.h>
 #include <locale.h>
 #include <string.h>
 #include <unistd.h>
 #include <errno.h>
-#include <sys/stat.h>
+#include <stdlib.h>
 #include "mathfunc.h"
 #include <goffice/utils/go-file.h>
-#ifdef WITH_GNOME
-#include <libgnome/gnome-url.h>
-#endif
 
 /* WorkbookView signals */
 enum {
@@ -131,6 +130,23 @@ wb_view_sheet_add (WorkbookView *wbv, Sheet *new_sheet)
 
 	WORKBOOK_VIEW_FOREACH_CONTROL (wbv, control,
 		wb_control_sheet_add (control, new_view););
+
+	g_object_unref (new_view);
+}
+
+void
+wb_view_sheet_remove (WorkbookView *wbv, Sheet *sheet)
+{
+	SheetView *sv;
+
+	g_return_if_fail (IS_WORKBOOK_VIEW (wbv));
+
+	WORKBOOK_VIEW_FOREACH_CONTROL (wbv, control,
+		wb_control_sheet_remove (control, sheet););
+
+	sv = sheet_get_view (sheet, wbv);
+	if (sv)
+		sv_dispose (sv);
 }
 
 gboolean
@@ -177,19 +193,12 @@ wb_view_preferred_size (WorkbookView *wbv, int w, int h)
 	wbv->preferred_height = h;
 }
 
-void
-wb_view_prefs_update (WorkbookView *view)
-{
-	WORKBOOK_VIEW_FOREACH_CONTROL(view, control,
-		wb_control_prefs_update	(control););
-}
-
 void
 wb_view_format_feedback (WorkbookView *wbv, gboolean display)
 {
 	SheetView *sv;
 	GnmStyle *style;
-	GnmFormat *sf_style, *sf_cell;
+	GOFormat *sf_style, *sf_cell;
 	GnmCell *cell;
 
 	g_return_if_fail (IS_WORKBOOK_VIEW (wbv));
@@ -199,25 +208,25 @@ wb_view_format_feedback (WorkbookView *wbv, gboolean display)
 		return;
 
 	style = sheet_style_get (sv->sheet, sv->edit_pos.col, sv->edit_pos.row);
-	sf_style = mstyle_get_format (style);
-	if (style_format_is_general (sf_style) &&
+	sf_style = gnm_style_get_format (style);
+	if (go_format_is_general (sf_style) &&
 	    (cell = sheet_cell_get (sv->sheet, sv->edit_pos.col, sv->edit_pos.row)) &&
 	    cell->value && VALUE_FMT (cell->value))
 		sf_cell = VALUE_FMT (cell->value);
 	else
 		sf_cell = sf_style;
 
-	if (style_format_equal (sf_cell, sf_style)) {
+	if (go_format_eq (sf_cell, sf_style)) {
 		if (style == wbv->current_format)
 			return;
-		mstyle_ref (style);
+		gnm_style_ref (style);
 	} else {
-		style = mstyle_copy (style);
-		mstyle_set_format (style, sf_cell);
+		style = gnm_style_dup (style);
+		gnm_style_set_format (style, sf_cell);
 	}
 
 	if (wbv->current_format != NULL)
-		mstyle_unref (wbv->current_format);
+		gnm_style_unref (wbv->current_format);
 	wbv->current_format = style;
 
 	if (display) {
@@ -237,7 +246,6 @@ wb_view_menus_update (WorkbookView *wbv)
 	if (sheet != NULL) {
 		WORKBOOK_VIEW_FOREACH_CONTROL (wbv, wbc, {
 			wb_control_menu_state_update (wbc, MS_ALL);
-			wb_control_menu_state_sheet_prefs (wbc, sheet);
 			wb_control_update_action_sensitivity (wbc);
 		});
 	}
@@ -362,6 +370,20 @@ wb_view_auto_expr (WorkbookView *wbv, char const *descr, char const *func_name)
 		wb_view_auto_expr_recalc (wbv, TRUE);
 }
 
+void
+wb_view_auto_expr_precision (WorkbookView *wbv, gboolean use_max_precision)
+{
+	use_max_precision = !!use_max_precision;
+
+	if (wbv->auto_expr_use_max_precision == use_max_precision)
+		return;
+
+	wbv->auto_expr_use_max_precision = use_max_precision;
+	if (wbv->current_sheet != NULL)
+		wb_view_auto_expr_recalc (wbv, TRUE);
+}
+
+
 static void
 wb_view_auto_expr_value_display (WorkbookView *wbv)
 {
@@ -408,19 +430,43 @@ wb_view_auto_expr_recalc (WorkbookView *wbv, gboolean display)
 	ei.func_call = (GnmExprFunction const *)wbv->auto_expr;
 
 	v = function_call_with_list (&ei, selection, 0);
-	gnm_expr_list_unref (selection);
 
-	if (wbv->auto_expr_value_as_string)
-		g_free (wbv->auto_expr_value_as_string);
+	g_free (wbv->auto_expr_value_as_string);
 
 	if (v) {
-		char const *val_str = value_peek_string (v);
-		wbv->auto_expr_value_as_string =
-			g_strconcat (wbv->auto_expr_desc, "=", val_str, NULL);
+		GString *str = g_string_new (wbv->auto_expr_desc);
+		GOFormat const *format = NULL;
+		GOFormat *tmp_format = NULL;
+
+		g_string_append_c (str, '=');
+		if (!wbv->auto_expr_use_max_precision) {
+			format = VALUE_FMT (v);
+			if (!format) {
+				const GnmExpr *fcall =
+					gnm_expr_new_funcall (gnm_expr_get_func_def (wbv->auto_expr),
+							      selection);
+				selection = NULL;
+				format = tmp_format =
+					auto_style_format_suggest (fcall, ei.pos);
+				gnm_expr_unref (fcall);
+			}
+		}
+
+		if (format) {
+			format_value_gstring (str, format, v, NULL,
+					      -1, workbook_date_conv (wb_view_workbook (wbv)));
+			if (tmp_format)
+				go_format_unref (tmp_format);
+		} else {
+			g_string_append (str, value_peek_string (v));
+		}
+
+		wbv->auto_expr_value_as_string = g_string_free (str, FALSE);
 		value_release (v);
 	} else
 		wbv->auto_expr_value_as_string = g_strdup (_("Internal ERROR"));
 
+	gnm_expr_list_unref (selection);
 	wb_view_auto_expr_value_display (wbv);
 }
 
@@ -494,7 +540,7 @@ wb_view_finalize (GObject *object)
 		wbv->auto_expr_value_as_string = NULL;
 	}
 	if (wbv->current_format != NULL) {
-		mstyle_unref (wbv->current_format);
+		gnm_style_unref (wbv->current_format);
 		wbv->current_format = NULL;
 	}
 
@@ -539,6 +585,7 @@ workbook_view_new (Workbook *wb)
 	wbv->auto_expr      = NULL;
 	wbv->auto_expr_desc = NULL;
 	wbv->auto_expr_value_as_string = NULL;
+	wbv->auto_expr_use_max_precision = FALSE;
 	wb_view_auto_expr (wbv, _("Sum"), "sum");
 
 	wbv->current_format = NULL;
@@ -557,7 +604,7 @@ workbook_view_new (Workbook *wb)
 }
 
 static void
-wbv_save_to_uri (WorkbookView *wbv, GnmFileSaver const *fs,
+wbv_save_to_uri (WorkbookView *wbv, GOFileSaver const *fs,
 		 char const *uri, IOContext *io_context)
 {
 	char *msg = NULL;
@@ -567,7 +614,7 @@ wbv_save_to_uri (WorkbookView *wbv, GnmFileSaver const *fs,
 	if (output == NULL) {
 		char *str = g_strdup_printf (_("Can't open '%s' for writing: %s"),
 					     uri, err->message);
-		gnm_cmd_context_error_export (GNM_CMD_CONTEXT (io_context), str);
+		go_cmd_context_error_export (GO_CMD_CONTEXT (io_context), str);
 		g_error_free (err);
 		g_free (str);
 		return;
@@ -576,14 +623,17 @@ wbv_save_to_uri (WorkbookView *wbv, GnmFileSaver const *fs,
 	if (output != NULL) {
 		GError const *save_err;
 
-		g_print ("Writing %s\n", uri);
-		gnm_file_saver_save (fs, io_context, wbv, GSF_OUTPUT (output));
-		save_err = gsf_output_error (GSF_OUTPUT (output));
+		g_printerr ("Writing %s\n", uri);
+		go_file_saver_save (fs, io_context, wbv, output);
+		if (!gsf_output_is_closed (output))
+			gsf_output_close (output);
+		save_err = gsf_output_error (output);
 		if (save_err) {
 			msg = g_strdup (save_err->message);
-			g_object_unref (G_OBJECT (output));
+			g_printerr ("  ==> %s\n", msg);
+			g_object_unref (output);
 		} else {
-			g_object_unref (G_OBJECT (output));
+			g_object_unref (output);
 			return;
 		}
 	}
@@ -592,14 +642,15 @@ wbv_save_to_uri (WorkbookView *wbv, GnmFileSaver const *fs,
 		msg = g_strdup_printf (_("An unexplained error happened while saving %s"),
 				       uri);
 
-	gnm_cmd_context_error_export (GNM_CMD_CONTEXT (io_context), msg);
+	if (!gnumeric_io_error_occurred (io_context))
+		go_cmd_context_error_export (GO_CMD_CONTEXT (io_context), msg);
 	g_free (msg);
 }
 
 /**
  * wb_view_save_as:
  * @wbv         : Workbook View
- * @fs          : GnmFileSaver object
+ * @fs          : GOFileSaver object
  * @uri         : URI to save as.
  * @context     :
  *
@@ -610,30 +661,30 @@ wbv_save_to_uri (WorkbookView *wbv, GnmFileSaver const *fs,
  * Return value: TRUE if file was successfully saved and FALSE otherwise.
  */
 gboolean
-wb_view_save_as (WorkbookView *wbv, GnmFileSaver *fs, char const *uri,
-		 GnmCmdContext *context)
+wb_view_save_as (WorkbookView *wbv, GOFileSaver *fs, char const *uri,
+		 GOCmdContext *context)
 {
 	IOContext *io_context;
 	Workbook  *wb;
 	gboolean has_error, has_warning;
 
 	g_return_val_if_fail (IS_WORKBOOK_VIEW (wbv), FALSE);
-	g_return_val_if_fail (IS_GNM_FILE_SAVER (fs), FALSE);
+	g_return_val_if_fail (IS_GO_FILE_SAVER (fs), FALSE);
 	g_return_val_if_fail (uri != NULL, FALSE);
-	g_return_val_if_fail (IS_GNM_CMD_CONTEXT (context), FALSE);
+	g_return_val_if_fail (IS_GO_CMD_CONTEXT (context), FALSE);
 
 	wb = wb_view_workbook (wbv);
 	io_context = gnumeric_io_context_new (context);
 
-	gnm_cmd_context_set_sensitive (context, FALSE);
+	go_cmd_context_set_sensitive (context, FALSE);
 	wbv_save_to_uri (wbv, fs, uri, io_context);
-	gnm_cmd_context_set_sensitive (context, TRUE);
+	go_cmd_context_set_sensitive (context, TRUE);
 
 	has_error   = gnumeric_io_error_occurred (io_context);
 	has_warning = gnumeric_io_warning_occurred (io_context);
 	if (!has_error) {
 		if (workbook_set_saveinfo (wb,
-			gnm_file_saver_get_format_level (fs), fs) &&
+			go_file_saver_get_format_level (fs), fs) &&
 		    workbook_set_uri (wb, uri))
 			workbook_set_dirty (wb, FALSE);
 	}
@@ -656,24 +707,24 @@ wb_view_save_as (WorkbookView *wbv, GnmFileSaver *fs, char const *uri,
  * Return value: TRUE if file was successfully saved and FALSE otherwise.
  */
 gboolean
-wb_view_save (WorkbookView *wbv, GnmCmdContext *context)
+wb_view_save (WorkbookView *wbv, GOCmdContext *context)
 {
 	IOContext	*io_context;
 	Workbook	*wb;
-	GnmFileSaver	*fs;
+	GOFileSaver	*fs;
 	gboolean has_error, has_warning;
 
 	g_return_val_if_fail (IS_WORKBOOK_VIEW (wbv), FALSE);
-	g_return_val_if_fail (IS_GNM_CMD_CONTEXT (context), FALSE);
+	g_return_val_if_fail (IS_GO_CMD_CONTEXT (context), FALSE);
 
 	wb = wb_view_workbook (wbv);
 	fs = workbook_get_file_saver (wb);
 	if (fs == NULL)
-		fs = gnm_file_saver_get_default ();
+		fs = go_file_saver_get_default ();
 
 	io_context = gnumeric_io_context_new (context);
 	if (fs == NULL)
-		gnm_cmd_context_error_export (GNM_CMD_CONTEXT (io_context),
+		go_cmd_context_error_export (GO_CMD_CONTEXT (io_context),
 			_("Default file saver is not available."));
 	else {
 		char const *uri = workbook_get_uri (wb);
@@ -731,7 +782,7 @@ gnm_mailto_url_show (char const *url, char const *working_dir, GError **err)
 	}
 
 	if (err)
-		*err = g_error_new (gnm_error_invalid (), 0,
+		*err = g_error_new (go_error_invalid (), 0,
 				    "Missing handler for mailto URLs.");
 }
 #endif
@@ -740,26 +791,26 @@ static gboolean
 cb_cleanup_sendto (gpointer path)
 {
 	char *dir = g_path_get_dirname (path);
-	unlink (path);	g_free (path);	/* the attachment */
-	rmdir (dir);	g_free (dir);	/* the tempdir */
+	g_unlink (path); g_free (path);	/* the attachment */
+	g_rmdir (dir); g_free (dir);	/* the tempdir */
 	return FALSE;
 }
 
 gboolean
-wb_view_sendto (WorkbookView *wbv, GnmCmdContext *context)
+wb_view_sendto (WorkbookView *wbv, GOCmdContext *context)
 {
-	gboolean problem = FALSE;
+	gboolean problem;
 	IOContext	*io_context;
 	Workbook	*wb;
-	GnmFileSaver	*fs;
+	GOFileSaver	*fs;
 
 	g_return_val_if_fail (IS_WORKBOOK_VIEW (wbv), FALSE);
-	g_return_val_if_fail (IS_GNM_CMD_CONTEXT (context), FALSE);
+	g_return_val_if_fail (IS_GO_CMD_CONTEXT (context), FALSE);
 
 	wb = wb_view_workbook (wbv);
 	fs = workbook_get_file_saver (wb);
 	if (fs == NULL)
-		fs = gnm_file_saver_get_default ();
+		fs = go_file_saver_get_default ();
 
 	io_context = gnumeric_io_context_new (context);
 	if (fs != NULL) {
@@ -770,39 +821,37 @@ wb_view_sendto (WorkbookView *wbv, GnmCmdContext *context)
 #ifdef HAVE_MKDTEMP
 		template = g_build_filename (g_get_tmp_dir (),
 			GNM_SEND_DIR "XXXXXX", NULL);
-		mkdtemp (template);
+		problem = (mkdtemp (template) == NULL);
 #else
 		while (1) {
-			char dirname[sizeof (GNM_SEND_DIR) + sizeof (pid_t) * 4 + 8 + 2 + 1];
-			sprintf (dirname, GNM_SEND_DIR "%ld-%08d",
+			char *dirname = g_strdup_printf
+				("%s%ld-%08d",
+				 GNM_SEND_DIR,
 				 (long)getpid (),
 				 (int)(1e8 * random_01 ()));
-
 			template = g_build_filename (g_get_tmp_dir (), dirname, NULL);
-#ifdef G_OS_WIN32
-			if (mkdir (template) == 0) {
-				problem = TRUE;
+			g_free (dirname);
+
+			if (g_mkdir (template, 0700) == 0) {
+				problem = FALSE;
 				break;
 			}
-#else
-			if (mkdir (template, 0700) == 0) {
-				problem = TRUE;
-				break;
-			}
-#endif
 
 			if (errno != EEXIST) {
-				g_free (template);
-
-				gnm_cmd_context_error_export (GNM_CMD_CONTEXT (io_context),
+				go_cmd_context_error_export (GO_CMD_CONTEXT (io_context),
 					_("Failed to create temporary file for sending."));
 				gnumeric_io_error_display (io_context);
 				problem = TRUE;
-				goto out;
+				break;
 			}
 		}
 #endif
 
+		if (problem) {
+			g_free (template);
+			goto out;
+		}
+
 		full_name = g_build_filename (template, basename, NULL);
 		g_free (basename);
 		uri = go_filename_to_uri (full_name);
@@ -827,12 +876,12 @@ wb_view_sendto (WorkbookView *wbv, GnmCmdContext *context)
 			url = g_strdup_printf ("mailto:someone?attach=%s", tmp);
 			g_free (tmp);
 #ifdef WITH_GNOME
-			gnome_url_show (url, &err);
+			go_url_show (url);
 #else
 			gnm_mailto_url_show (url, template, &err);
 #endif
 			if (err != NULL) {
-				gnm_cmd_context_error (GNM_CMD_CONTEXT (io_context), err);
+				go_cmd_context_error (GO_CMD_CONTEXT (io_context), err);
 				g_error_free (err);
 				gnumeric_io_error_display (io_context);
 				problem = TRUE;
@@ -848,15 +897,13 @@ wb_view_sendto (WorkbookView *wbv, GnmCmdContext *context)
 		g_timeout_add (1000 * 10, cb_cleanup_sendto, full_name);
 		g_free (uri);
 	} else {
-		gnm_cmd_context_error_export (GNM_CMD_CONTEXT (io_context),
+		go_cmd_context_error_export (GO_CMD_CONTEXT (io_context),
 			_("Default file saver is not available."));
 		gnumeric_io_error_display (io_context);
 		problem = TRUE;
 	}
 
-#ifndef HAVE_MKDTEMP
  out:
-#endif
 	g_object_unref (G_OBJECT (io_context));
 
 	return !problem;
@@ -864,7 +911,7 @@ wb_view_sendto (WorkbookView *wbv, GnmCmdContext *context)
 
 WorkbookView *
 wb_view_new_from_input  (GsfInput *input,
-			 GnmFileOpener const *optional_fmt,
+			 GOFileOpener const *optional_fmt,
 			 IOContext *io_context,
 			 char const *optional_enc)
 {
@@ -872,7 +919,7 @@ wb_view_new_from_input  (GsfInput *input,
 
 	g_return_val_if_fail (GSF_IS_INPUT(input), NULL);
 	g_return_val_if_fail (optional_fmt == NULL ||
-			      IS_GNM_FILE_OPENER (optional_fmt), NULL);
+			      IS_GO_FILE_OPENER (optional_fmt), NULL);
 
 	/* NOTE : we could support gzipped anything here if we wanted to
 	 * by adding a wrapper, but there is no framework for remembering that
@@ -887,19 +934,19 @@ wb_view_new_from_input  (GsfInput *input,
 
 		for (pl = FILE_PROBE_FILE_NAME; pl < FILE_PROBE_LAST && optional_fmt == NULL; pl++) {
 			for (l = get_file_openers (); l != NULL; l = l->next) {
-				GnmFileOpener const *tmp_fo = GNM_FILE_OPENER (l->data);
+				GOFileOpener const *tmp_fo = GO_FILE_OPENER (l->data);
 				int new_input_refs;
 				/* A name match needs to be a content match too */
-				if (gnm_file_opener_probe (tmp_fo, input, pl) &&
+				if (go_file_opener_probe (tmp_fo, input, pl) &&
 				    (pl == FILE_PROBE_CONTENT ||
-				     !gnm_file_opener_can_probe	(tmp_fo, FILE_PROBE_CONTENT) ||
-				     gnm_file_opener_probe (tmp_fo, input, FILE_PROBE_CONTENT)))
+				     !go_file_opener_can_probe	(tmp_fo, FILE_PROBE_CONTENT) ||
+				     go_file_opener_probe (tmp_fo, input, FILE_PROBE_CONTENT)))
 					optional_fmt = tmp_fo;
 
 				new_input_refs = G_OBJECT (input)->ref_count;
 				if (new_input_refs != input_refs) {
 					g_warning ("Format %s's probe changed input ref_count from %d to %d.",
-						   gnm_file_opener_get_id (tmp_fo),
+						   go_file_opener_get_id (tmp_fo),
 						   input_refs,
 						   new_input_refs);
 					input_refs = new_input_refs;
@@ -912,15 +959,21 @@ wb_view_new_from_input  (GsfInput *input,
 	}
 
 	if (optional_fmt != NULL) {
+		char const *input_name;
 		Workbook *new_wb;
 		gboolean old;
 
 		new_wbv = workbook_view_new (NULL);
 		new_wb = wb_view_workbook (new_wbv);
+		if (NULL != (input_name = gsf_input_name (input))) {
+			char *uri = go_shell_arg_to_uri (input_name);
+			workbook_set_uri (new_wb, uri);
+			g_free (uri);
+		}
 
 		/* disable recursive dirtying while loading */
 		old = workbook_enable_recursive_dirty (new_wb, FALSE);
-		gnm_file_opener_open (optional_fmt, optional_enc, io_context, new_wbv, input);
+		go_file_opener_open (optional_fmt, optional_enc, io_context, new_wbv, input);
 		workbook_enable_recursive_dirty (new_wb, old);
 
 		if (gnumeric_io_error_occurred (io_context)) {
@@ -936,7 +989,7 @@ wb_view_new_from_input  (GsfInput *input,
 			workbook_set_dirty (new_wb, FALSE);
 		}
 	} else
-		gnm_cmd_context_error_import (GNM_CMD_CONTEXT (io_context),
+		go_cmd_context_error_import (GO_CMD_CONTEXT (io_context),
 			_("Unsupported file format."));
 
 	return new_wbv;
@@ -945,9 +998,9 @@ wb_view_new_from_input  (GsfInput *input,
 /**
  * wb_view_new_from_uri :
  * @uri          : URI for file
- * @optional_fmt : Optional GnmFileOpener
+ * @optional_fmt : Optional GOFileOpener
  * @io_context   : Optional context to display errors.
- * @optional_enc : Optional encoding for GnmFileOpener that understand it
+ * @optional_enc : Optional encoding for GOFileOpener that understand it
  *
  * Reads @uri file using given file opener @optional_fmt, or probes for a valid
  * possibility if @optional_fmt is NULL.  Reports problems to @io_context.
@@ -956,7 +1009,7 @@ wb_view_new_from_input  (GsfInput *input,
  */
 WorkbookView *
 wb_view_new_from_uri (char const *uri,
-		      GnmFileOpener const *optional_fmt,
+		      GOFileOpener const *optional_fmt,
 		      IOContext *io_context,
 		      char const *optional_enc)
 {
@@ -970,7 +1023,7 @@ wb_view_new_from_uri (char const *uri,
 	if (input != NULL) {
 		WorkbookView *res;
 
-		g_print ("Reading %s\n", uri);
+		g_printerr ("Reading %s\n", uri);
 		res = wb_view_new_from_input (input,
 					      optional_fmt, io_context,
 					      optional_enc);
@@ -988,7 +1041,7 @@ wb_view_new_from_uri (char const *uri,
 		msg = g_strdup_printf (_("An unexplained error happened while opening %s"),
 				       uri);
 
-	gnm_cmd_context_error_import (GNM_CMD_CONTEXT (io_context), msg);
+	go_cmd_context_error_import (GO_CMD_CONTEXT (io_context), msg);
 	g_free (msg);
 
 	return NULL;
