@@ -9,7 +9,7 @@
  *
  * (C) 1998, 1999, 2000 Miguel de Icaza
  * (C) 2000-2001 Ximian, Inc.
- * (C) 2002 Jody Goldberg
+ * (C) 2002-2005 Jody Goldberg
  */
 #include <gnumeric-config.h>
 #include <glib/gi18n.h>
@@ -45,8 +45,8 @@
 #include <bonobo/bonobo-main.h>
 #else
 #include <gtk/gtkmain.h> /* for gtk_main_quit */
-#endif /* WITH_GTK */
 #endif
+#endif /* WITH_GTK */
 #include <gsf/gsf-impl-utils.h>
 #include <string.h>
 #include <errno.h>
@@ -82,13 +82,6 @@ workbook_dispose (GObject *wb_object)
 
 	wb->during_destruction = TRUE;
 
-	if (wb->uri) {
-		if (wb->file_format_level >= FILE_FL_MANUAL_REMEMBER)
-			gnm_app_history_add (wb->uri);
-	       g_free (wb->uri);
-	       wb->uri = NULL;
-	}
-
 	if (wb->file_saver)
 		workbook_set_saveinfo (wb, wb->file_format_level, NULL);
 
@@ -101,7 +94,7 @@ workbook_dispose (GObject *wb_object)
 	command_list_release (wb->redo_commands);
 	wb->redo_commands = NULL;
 
-	workbook_deps_destroy (wb);
+	dependents_workbook_destroy (wb);
 
 	/* Copy the set of sheets, the list changes under us. */
 	sheets = workbook_sheets (wb);
@@ -116,14 +109,16 @@ workbook_dispose (GObject *wb_object)
 		 * the sheet from the workbook.  It is ugly, but should
 		 * be ok for debug code.
 		 */
-		if (gnumeric_debugging > 0)
+		if (gnumeric_debugging > 0) {
+			g_printerr ("Dependencies for %s:\n", sheet->name_unquoted);
 			gnm_dep_container_dump (sheet->deps);
+		}
 	}
 
 	/* Now remove the sheets themselves */
 	for (ptr = sheets; ptr != NULL ; ptr = ptr->next) {
 		Sheet *sheet = ptr->data;
-		workbook_sheet_detach (wb, sheet, FALSE);
+		workbook_sheet_delete (sheet);
 	}
 	g_list_free (sheets);
 
@@ -134,15 +129,17 @@ workbook_dispose (GObject *wb_object)
 	if (wb->wb_views != NULL) {
 		WORKBOOK_FOREACH_VIEW (wb, view, {
 			workbook_detach_view (view);
-			g_object_unref (G_OBJECT (view));
+			g_object_unref (view);
 		});
 		if (wb->wb_views != NULL)
 			g_warning ("Unexpected left over views");
 	}
 
-	if (wb->sheet_local_functions != NULL) {
-		g_hash_table_destroy (wb->sheet_local_functions);
-		wb->sheet_local_functions = NULL;
+	if (wb->uri) {
+		if (wb->file_format_level >= FILE_FL_MANUAL_REMEMBER)
+			gnm_app_history_add (wb->uri);
+	       g_free (wb->uri);
+	       wb->uri = NULL;
 	}
 
 	workbook_parent_class->dispose (wb_object);
@@ -604,8 +601,8 @@ workbook_foreach_cell_in_range (GnmEvalPos const *pos,
 	g_return_val_if_fail (cell_range != NULL, NULL);
 	g_return_val_if_fail (cell_range->type == VALUE_CELLRANGE, NULL);
 
-	rangeref_normalize (&cell_range->v_range.cell, pos,
-			    &start_sheet, &end_sheet, &r);
+	gnm_rangeref_normalize (&cell_range->v_range.cell, pos,
+		&start_sheet, &end_sheet, &r);
 
 	if (start_sheet != end_sheet) {
 		GnmValue *res;
@@ -742,15 +739,12 @@ workbook_attach_view (Workbook *wb, WorkbookView *wbv)
 void
 workbook_detach_view (WorkbookView *wbv)
 {
-	SheetView *sv;
-
 	g_return_if_fail (IS_WORKBOOK_VIEW (wbv));
 	g_return_if_fail (IS_WORKBOOK (wbv->wb));
 
 	WORKBOOK_FOREACH_SHEET (wbv->wb, sheet, {
-		sv = sheet_get_view (sheet, wbv);
+		SheetView *sv = sheet_get_view (sheet, wbv);
 		sv_dispose (sv);
-		g_object_unref (G_OBJECT (sv));
 	});
 
 	g_ptr_array_remove (wbv->wb->wb_views, wbv);
@@ -792,15 +786,6 @@ workbook_sheet_count (Workbook const *wb)
 	return wb->sheets ? wb->sheets->len : 0;
 }
 
-static void
-cb_dep_unlink (GnmDependent *dep, gpointer value, gpointer user_data)
-{
-	GnmCellPos *pos = NULL;
-	if (dependent_is_cell (dep))
-		pos = &DEP_TO_CELL (dep)->pos;
-	dependent_unlink (dep, pos);
-}
-
 static void
 pre_sheet_index_change (Workbook *wb)
 {
@@ -810,16 +795,8 @@ pre_sheet_index_change (Workbook *wb)
 
 	if (wb->sheet_order_dependents != NULL)
 		g_hash_table_foreach (wb->sheet_order_dependents,
-			(GHFunc) cb_dep_unlink, NULL);
-}
-
-static void
-cb_dep_link (GnmDependent *dep, gpointer value, gpointer user_data)
-{
-	GnmCellPos *pos = NULL;
-	if (dependent_is_cell (dep))
-		pos = &DEP_TO_CELL (dep)->pos;
-	dependent_link (dep, pos);
+				      (GHFunc)dependent_unlink,
+				      NULL);
 }
 
 static void
@@ -829,7 +806,8 @@ post_sheet_index_change (Workbook *wb)
 
 	if (wb->sheet_order_dependents != NULL)
 		g_hash_table_foreach (wb->sheet_order_dependents,
-			(GHFunc) cb_dep_link, NULL);
+				      (GHFunc)dependent_link,
+				      NULL);
 
 	wb->being_reordered = FALSE;
 
@@ -898,15 +876,15 @@ workbook_focus_other_sheet (Workbook *wb, Sheet *sheet)
 	Sheet *focus = NULL;
 	int sheet_index = sheet->index_in_wb;
 
-	for (i = sheet_index - 1; !focus && i >= 0; i++) {
+	for (i = sheet_index; !focus && --i >= 0; ) {
 		Sheet *this_sheet = g_ptr_array_index (wb->sheets, i);
-		if (this_sheet->is_visible)
+		if (this_sheet->visibility == GNM_SHEET_VISIBILITY_VISIBLE)
 			focus = this_sheet;
 	}
 
-	for (i = sheet_index + 1; !focus && i < (int)wb->sheets->len; i++) {
+	for (i = sheet_index; !focus && ++i < (int)wb->sheets->len; ) {
 		Sheet *this_sheet = g_ptr_array_index (wb->sheets, i);
-		if (this_sheet->is_visible)
+		if (this_sheet->visibility == GNM_SHEET_VISIBILITY_VISIBLE)
 			focus = this_sheet;
 	}
 
@@ -947,32 +925,26 @@ workbook_sheet_hide_controls (Workbook *wb, Sheet *sheet)
 	if (!wb->during_destruction)
 		focus = workbook_focus_other_sheet (wb, sheet);
 
-	/* Remove all controls */
-	WORKBOOK_FOREACH_CONTROL (wb, view, control,
-		wb_control_sheet_remove (control, sheet););
+	WORKBOOK_FOREACH_CONTROL (wb, wbv, wbc,
+		wb_control_sheet_remove (wbc, sheet););
 
 	return focus != NULL;
 }
 
-static void
-workbook_sheet_unhide_controls (Workbook *wb, Sheet *sheet)
-{
-	g_return_if_fail (IS_WORKBOOK (wb));
-	g_return_if_fail (IS_SHEET (sheet));
-
-	WORKBOOK_FOREACH_VIEW (wb, wbv,
-		if (sheet_get_view (sheet, wbv) == NULL)
-			wb_view_sheet_add (wbv, sheet););
-}
-
 static void
 cb_sheet_visibility_change (Sheet *sheet,
 			    G_GNUC_UNUSED GParamSpec *pspec,
 			    G_GNUC_UNUSED gpointer data)
 {
-	if (sheet->is_visible)
-		workbook_sheet_unhide_controls (sheet->workbook, sheet);
-	else
+	g_return_if_fail (IS_SHEET (sheet));
+
+	if (sheet->visibility == GNM_SHEET_VISIBILITY_VISIBLE) {
+		SheetView *sv;
+		WORKBOOK_FOREACH_CONTROL (sheet->workbook, wbv, wbc, {
+			sv = sheet_get_view (sheet, wbv);
+			if (NULL == sv_get_control (sv, wbc))
+				wb_control_sheet_add (wbc, sv);});
+	} else
 		workbook_sheet_hide_controls (sheet->workbook, sheet);
 }
 
@@ -1002,15 +974,16 @@ workbook_sheet_attach_at_pos (Workbook *wb, Sheet *new_sheet, int pos)
 			     new_sheet->name_case_insensitive,
 			     new_sheet);
 
-	post_sheet_index_change (wb);
-
 	WORKBOOK_FOREACH_VIEW (wb, view,
 		wb_view_sheet_add (view, new_sheet););
 
 	g_signal_connect (G_OBJECT (new_sheet),
-			  "notify::visible",
+			  "notify::visibility",
 			  G_CALLBACK (cb_sheet_visibility_change),
 			  NULL);
+
+	/* Do not signal until after adding the views [#314208] */
+	post_sheet_index_change (wb);
 }
 
 /**
@@ -1029,73 +1002,6 @@ workbook_sheet_attach (Workbook *wb, Sheet *new_sheet)
 	g_object_unref (new_sheet);
 }
 
-static void
-cb_tweak_3d (GnmDependent *dep, gpointer value, GnmExprRewriteInfo *rwinfo)
-{
-	GnmExpr const *newtree = gnm_expr_rewrite (dep->expression, rwinfo);
-	if (newtree != NULL) {
-		dependent_set_expr (dep, newtree);
-		gnm_expr_unref (newtree);
-	}
-}
-
-/**
- * workbook_sheet_detach:
- * @wb: workbook.
- * @sheet: the sheet that we want to detach from the workbook
- * @recalc : force a recalc afterward
- *
- * Detaches @sheet from the workbook @wb.
- */
-void
-workbook_sheet_detach (Workbook *wb, Sheet *sheet, gboolean recalc)
-{
-	int sheet_index;
-	gboolean still_visible_sheets;
-
-	g_return_if_fail (IS_WORKBOOK (wb));
-	g_return_if_fail (IS_SHEET (sheet));
-	g_return_if_fail (sheet->workbook == wb);
-	g_return_if_fail (workbook_sheet_by_name (wb, sheet->name_unquoted) == sheet);
-
-	g_signal_handlers_disconnect_by_func (sheet, cb_sheet_visibility_change, NULL);
-
-	sheet_index = sheet->index_in_wb;
-	still_visible_sheets = workbook_sheet_hide_controls (wb, sheet);
-	pre_sheet_index_change (wb);
-	/* If we are not destroying things, Check for 3d refs that start or end
-	 * on this sheet */
-	if (wb->sheet_order_dependents != NULL) {
-		GnmExprRewriteInfo rwinfo;
-
-		rwinfo.type = GNM_EXPR_REWRITE_SHEET;
-		rwinfo.u.sheet = sheet;
-		g_hash_table_foreach (wb->sheet_order_dependents,
-			(GHFunc) cb_tweak_3d, &rwinfo);
-	}
-
-	/* Remove our reference to this sheet */
-	g_ptr_array_remove_index (wb->sheets, sheet_index);
-	workbook_sheet_index_update (wb, sheet_index);
-	sheet->index_in_wb = -1;
-	g_hash_table_remove (wb->sheet_hash_private, sheet->name_case_insensitive);
-	post_sheet_index_change (wb);
-
-	/* Clear the controls first, before we potentially update */
-	SHEET_FOREACH_VIEW (sheet, view, {
-		sv_dispose (view);
-		g_object_unref (G_OBJECT (view));
-	});
-
-	g_signal_emit_by_name (G_OBJECT (sheet), "detached_from_workbook", wb);
-	g_object_unref (sheet);
-
-	g_signal_emit (G_OBJECT (wb), signals [SHEET_DELETED], 0);
-
-	if (recalc && still_visible_sheets)
-		workbook_recalc_all (wb);
-}
-
 /**
  * workbook_sheet_add :
  * @wb :
@@ -1125,44 +1031,53 @@ workbook_sheet_add (Workbook *wb, int pos, gboolean make_dirty)
 }
 
 /**
- * Unlike workbook_sheet_detach, this function not only detaches the given
- * sheet from its parent workbook, But also invalidates all references to the
- * deleted sheet from other sheets and clears all references In the clipboard
- * to this sheet.  Finally, it also detaches the sheet from the workbook.
+ * workbook_sheet_delete:
+ * @sheet: the sheet that we want to delete from its workbook
+ *
+ * This function detaches the given sheet from its parent workbook and
+ * invalidates all references to the deleted sheet from other sheets and
+ * clears all references in the clipboard to this sheet.
  */
 void
 workbook_sheet_delete (Sheet *sheet)
 {
 	Workbook *wb;
+	int sheet_index;
+	gboolean still_visible_sheets = FALSE;
 
         g_return_if_fail (IS_SHEET (sheet));
         g_return_if_fail (IS_WORKBOOK (sheet->workbook));
 
 	wb = sheet->workbook;
+	sheet_index = sheet->index_in_wb;
 
-	if (!sheet->pristine) {
-		/*
-		 * FIXME : Deleting a sheet plays havoc with our data structures.
-		 * Be safe for now and empty the undo/redo queues
-		 */
-		command_list_release (wb->undo_commands);
-		command_list_release (wb->redo_commands);
-		wb->undo_commands = NULL;
-		wb->redo_commands = NULL;
-		WORKBOOK_FOREACH_CONTROL (wb, view, control,
-			wb_control_undo_redo_truncate (control, 0, TRUE);
-			wb_control_undo_redo_truncate (control, 0, FALSE);
-			wb_control_undo_redo_labels (control, NULL, NULL);
-		);
+	g_signal_handlers_disconnect_by_func (sheet, cb_sheet_visibility_change, NULL);
+
+	if (!wb->during_destruction) {
+		workbook_focus_other_sheet (wb, sheet);
+		/* During destruction this was already done.  */
+		dependents_invalidate_sheet (sheet, FALSE);
+		still_visible_sheets = workbook_sheet_hide_controls (wb, sheet);
 	}
 
-	workbook_focus_other_sheet (wb, sheet);
-
-	/* Important to do these BEFORE detaching the sheet */
-	sheet_deps_destroy (sheet);
-
 	/* All is fine, remove the sheet */
-	workbook_sheet_detach (wb, sheet, TRUE);
+	pre_sheet_index_change (wb);
+	g_ptr_array_remove_index (wb->sheets, sheet_index);
+	workbook_sheet_index_update (wb, sheet_index);
+	sheet->index_in_wb = -1;
+	g_hash_table_remove (wb->sheet_hash_private, sheet->name_case_insensitive);
+	post_sheet_index_change (wb);
+
+	/* Clear the controls first, before we potentially update */
+	SHEET_FOREACH_VIEW (sheet, view, sv_dispose (view););
+
+	g_signal_emit_by_name (G_OBJECT (sheet), "detached_from_workbook", wb);
+	g_object_unref (sheet);
+
+	g_signal_emit (G_OBJECT (wb), signals[SHEET_DELETED], 0);
+
+	if (still_visible_sheets)
+		workbook_recalc_all (wb);
 }
 
 /**
@@ -1195,8 +1110,6 @@ workbook_sheet_move (Sheet *sheet, int direction)
 			sheet->index_in_wb = max_pos;
 		}
 
-		WORKBOOK_FOREACH_CONTROL (wb, view, control,
-			wb_control_sheet_move (control, sheet, old_pos););
 		sheet_set_dirty (sheet, TRUE);
 	}
 
@@ -1258,113 +1171,6 @@ workbook_sheet_get_free_name (Workbook *wb,
 	return name;
 }
 
-/**
- * workbook_sheet_rename_check:
- * @wb:          workbook to look for
- * @sheet_indices:   list of sheet indices (ignore -1)
- * @new_names:   list of new names
- *
- * Check whether changing of the names of the sheets as indicated
- * would be possible.
- *
- * Returns TRUE when it is possible.
- **/
-gboolean
-workbook_sheet_rename_check (Workbook *wb,
-			     GSList *sheet_indices,
-			     GSList *new_names,
-			     GSList *sheet_indices_deleted,
-			     GOCmdContext *cc)
-{
-	GSList *sheet_index = sheet_indices;
-	GSList *new_name = new_names;
-	gint max_sheet = workbook_sheet_count (wb);
-
-	/* First we check whether the sheet_indices are valid */
-	while (sheet_index) {
-		gint n = GPOINTER_TO_INT (sheet_index->data);
-		if (n < -1 || n >= max_sheet) {
-			g_warning ("Invalid sheet index %i", n);
-			return FALSE;
-		}
-		sheet_index = sheet_index->next;
-	}
-
-
-	/* Then we just check whether the names are valid.*/
-	sheet_index = sheet_indices;
-	while (new_name && sheet_index) {
-		if (new_name != NULL) {
-			char *the_new_name = new_name->data;
-			Sheet *tmp;
-
-			if (the_new_name == NULL && 
-			    GPOINTER_TO_INT (sheet_index->data) != -1) {
-				go_cmd_context_error_invalid (cc, 
-					_("Sheet name is NULL"), the_new_name);
-				return FALSE;
-			}
-
-			if (the_new_name != NULL) {
-				
-				/* Is the sheet name valid utf-8 ?*/
-				if (!g_utf8_validate (the_new_name, -1, NULL)) {
-					go_cmd_context_error_invalid (cc, 
-						_("Sheet name is not valid utf-8"),
-						the_new_name);
-					return FALSE;
-				}
-				
-				/* Is the sheet name to short ?*/
-				if (1 > g_utf8_strlen (the_new_name, -1)) {
-					go_cmd_context_error_invalid (cc,
-						_("Sheet name must have at least 1 letter"),
-						the_new_name);
-					return FALSE;
-				}
-				
-				/* Is the sheet name already in use ?*/
-				tmp = workbook_sheet_by_name (wb, the_new_name);
-				
-				if (tmp != NULL) {
-					/* Perhaps it is a sheet also to be renamed */
-					GSList *tmp_sheets = g_slist_find 
-						(sheet_indices, 
-						 GINT_TO_POINTER(tmp->index_in_wb));
-					if (NULL == tmp_sheets) {
-					/* Perhaps it is a sheet to be deleted */					
-						tmp_sheets = g_slist_find 
-						    (sheet_indices_deleted, 
-						     GINT_TO_POINTER(tmp->index_in_wb));
-					}
-
-					if (NULL == tmp_sheets) {
-						go_cmd_context_error_invalid (cc,
-							_("There is already a sheet named"),
-							the_new_name);
-						return FALSE;
-					}
-				}
-				
-				/* Will we try to use the same name a second time ?*/
-				if (new_name->next != NULL &&
-				    g_slist_find_custom (new_name->next, 
-							 the_new_name, 
-							 go_str_compare) != NULL) {
-					go_cmd_context_error_invalid (cc,
-						_("You may not use this name twice"),
-						the_new_name);
-					return FALSE;
-				}
-			}
-		}
-		new_name = new_name->next;
-		sheet_index = sheet_index->next;
-	}
-	
-	return TRUE;
-}
-
 /**
  * workbook_sheet_rename:
  * @wb:          workbook to look for
@@ -1409,75 +1215,6 @@ workbook_sheet_rename (Workbook *wb,
 	return FALSE;
 }
 
-/**
- * workbook_sheet_change_protection:
- * @wb:          workbook to look for
- * @sheets   :   list of sheet indices (ignore -1)
- * @locks    :   list of new locks
- *
- * Adjusts the locks
- *
- * Returns FALSE when it was successful
- **/
-gboolean    
-workbook_sheet_change_protection  (Workbook *wb,
-				   GSList *sheets,
-				   GSList *locks)
-{
-	g_return_val_if_fail (g_slist_length (sheets) 
-			      == g_slist_length (locks), 
-			      TRUE);
-
-	while (sheets) {
-		int sheetno = GPOINTER_TO_INT (sheets->data);
-		gboolean protected = GPOINTER_TO_INT (locks->data);
-		Sheet *sheet = workbook_sheet_by_index (wb, sheetno);
-
-		if (sheet != NULL)
-			g_object_set (sheet, "protected", protected, NULL);
-
-		sheets = sheets->next;
-		locks = locks->next;
-	}
-	return FALSE;
-	
-}
-
-/**
- * workbook_sheet_change_visibility:
- * @wb:          workbook to look for
- * @sheets   :   list of sheet indices (ignore -1)
- * @visibility:  list of new visibility
- *
- * Adjusts the visibility
- *
- * Returns FALSE when it was successful
- **/
-gboolean    
-workbook_sheet_change_visibility  (Workbook *wb,
-				   GSList *sheets,
-				   GSList *visibility)
-{
-	g_return_val_if_fail (g_slist_length (sheets) 
-			      == g_slist_length (visibility), 
-			      TRUE);
-
-	while (sheets) {
-		Sheet *sheet = workbook_sheet_by_index 
-			(wb, GPOINTER_TO_INT (sheets->data));
-		if (sheet != NULL) {
-			gboolean visible = GPOINTER_TO_INT (visibility->data);
-			g_object_set (sheet,
-				      "visible", visible,
-				      NULL);
-		}
-		sheets = sheets->next;
-		visibility = visibility->next;
-	}
-	return FALSE;
-	
-}
-
 /**
  * workbook_find_command :
  * @wb : #Workbook
@@ -1502,56 +1239,6 @@ workbook_find_command (Workbook *wb, gboolean is_undo, gpointer cmd)
 	return 0;
 }
 
-/**
- * workbook_sheet_recolor:
- * @wb:          workbook to look for
- * @sheets   :   list of sheet indices (ignore -1)
- * @fore     :   list of new text colors
- * @back     :   list of new background colors
- *
- * Adjusts the foreground colors
- *
- * Returns FALSE when it was successful
- **/
-gboolean    
-workbook_sheet_recolor  (Workbook *wb,
-			 GSList *sheets,
-			 GSList *fore, 
-			 GSList *back)
-{
-	g_return_val_if_fail (g_slist_length (sheets) 
-			      == g_slist_length (fore), 
-			      TRUE);
-	g_return_val_if_fail (g_slist_length (sheets) 
-			      == g_slist_length (back), 
-			      TRUE);
-
-	while (sheets) {
-		Sheet *sheet = workbook_sheet_by_index 
-			(wb, GPOINTER_TO_INT (sheets->data));
-		if (sheet != NULL) {
-			GdkColor *fc = (GdkColor *) fore->data;
-			GdkColor *bc = (GdkColor *) back->data;
-			GnmColor *fore_color = fc ?
-				style_color_new (fc->red, fc->green,
-						 fc->blue) : NULL;
-			GnmColor *back_color = bc ?
-				style_color_new (bc->red, bc->green,
-						 bc->blue) : NULL;
-			g_object_set (sheet,
-				      "tab-foreground", fore_color,
-				      "tab-background", back_color,
-				      NULL);
-			style_color_unref (fore_color);
-			style_color_unref (back_color);
-		}
-		sheets = sheets->next;
-		fore = fore->next;
-		back = back->next;
-	}
-	return FALSE;
-}
-
 /**
  * workbook_sheet_reorder:
  * @wb:          workbook to look for
@@ -1564,39 +1251,18 @@ workbook_sheet_recolor  (Workbook *wb,
 gboolean
 workbook_sheet_reorder (Workbook *wb, GSList *new_order)
 {
-	GSList *this_sheet;
-	gint old_pos, new_pos = 0;
+	GSList   *ptr;
+	Sheet    *sheet;
+	unsigned  pos = 0;
 
 	g_return_val_if_fail (IS_WORKBOOK (wb), FALSE);
-
-	if (new_order == NULL)
-		return TRUE;
+	g_return_val_if_fail (g_slist_length (new_order) == wb->sheets->len, FALSE);
 
 	pre_sheet_index_change (wb);
 
-	this_sheet = new_order;
-	while (this_sheet) {
-		Sheet *sheet = this_sheet->data;
-
-		if (sheet != NULL) {
-			old_pos = sheet->index_in_wb;
-			if (new_pos != old_pos) {
-				int max_pos = MAX (old_pos, new_pos);
-				int min_pos = MIN (old_pos, new_pos);
-
-				g_ptr_array_remove_index (wb->sheets, old_pos);
-				go_ptr_array_insert (wb->sheets, sheet, new_pos);
-				for (; max_pos >= min_pos ; max_pos--) {
-					Sheet *sheet = g_ptr_array_index (wb->sheets, max_pos);
-					sheet->index_in_wb = max_pos;
-				}
-				WORKBOOK_FOREACH_CONTROL (wb, view, control,
-							  wb_control_sheet_move (control,
-										 sheet, old_pos););
-			}
-			new_pos++;
-		}
-		this_sheet = this_sheet->next;
+	for (ptr = new_order; NULL != ptr ; ptr = ptr->next, pos++) {
+		g_ptr_array_index (wb->sheets, pos) = sheet = ptr->data;
+		sheet->index_in_wb = pos;
 	}
 
 	post_sheet_index_change (wb);
@@ -1604,43 +1270,6 @@ workbook_sheet_reorder (Workbook *wb, GSList *new_order)
 	return FALSE;
 }
 
-
-/**
- * workbook_sheet_reorder_by_idx:
- * @wb:          workbook to look for
- * @new_order:   list of sheet indices in order
- *
- * Adjusts the order of the sheets.
- *
- * Returns FALSE when it was successful
- **/
-gboolean
-workbook_sheet_reorder_by_idx (Workbook *wb, GSList *new_order)
-{
-	GSList *sheets = NULL;
-	gboolean result;
-
-	g_return_val_if_fail (IS_WORKBOOK (wb), FALSE);
-
-	if (new_order == NULL)
-		return TRUE;
-
-	while (new_order) {
-		sheets = g_slist_prepend (sheets, 
-					  workbook_sheet_by_index 
-					  (wb,
-					   GPOINTER_TO_INT (new_order->data)));
-		new_order = new_order->next;
-	}
-	sheets = g_slist_reverse (sheets);
-
-	result = workbook_sheet_reorder (wb, sheets);
-
-	g_slist_free (sheets);
-
-	return result;
-}
-
 /**
  * workbook_uses_1904 :
  * @wb :
@@ -1733,29 +1362,24 @@ workbook_sheet_state_restore (Workbook *wb, const WorkbookSheetState *wss)
 	int i;
 
 	/* Get rid of sheets that shouldn't be there.  */
-	for (i = workbook_sheet_count (wb) - 1; i >= 0; i--) {
+	for (i = workbook_sheet_count (wb) ; i-- > 0; ) {
 		Sheet *sheet = workbook_sheet_by_index (wb, i);
 		int j;
 		for (j = 0; j < wss->n_sheets; j++)
 			if (sheet == wss->sheets[j].sheet)
 				break;
-		if (j == wss->n_sheets) {
-			/*
-			 * This can also be triggered by an undo of sheet
-			 * insert.  In that case we actually are ready.
-			 */
-			g_warning ("We probably aren't completely ready for this.\n");
-			workbook_sheet_detach (wb, sheet, FALSE);
-		}
+		if (j == wss->n_sheets)
+			workbook_sheet_delete (sheet);
 	}
 
 	/* Attach new sheets and handle order.  */
 	for (i = 0; i < wss->n_sheets; i++) {
 		Sheet *sheet = wss->sheets[i].sheet;
 		if (sheet->index_in_wb != i) {
-			if (sheet->index_in_wb == -1)
+			if (sheet->index_in_wb == -1) {
 				workbook_sheet_attach_at_pos (wb, sheet, i);
-			else {
+				dependents_revive_sheet (sheet);
+			} else {
 				/*
 				 * There might be a smarter way of getting more
 				 * sheets into place faster.  This will at
